@@ -4,14 +4,14 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
 #   include	<stdio.h>
 #   include	<ctype.h>
 
-#   include	<debugon.h>
+#   include	<appDebugon.h>
 
 #   include	"docRtf.h"
 
@@ -20,7 +20,7 @@
 /*  Translate shape type to string.					*/
 /*									*/
 /************************************************************************/
-#   if 0
+
 static char * docRtfShapeTypeString(	int	shapeType )
     {
     static char		scratch[12];
@@ -223,7 +223,6 @@ static char * docRtfShapeTypeString(	int	shapeType )
 	    return scratch;
 	}
     }
-#   endif
 
 /************************************************************************/
 /*									*/
@@ -304,9 +303,11 @@ static RtfControlWord	docRtfShapeWords[]=
 	{ "shpbxpage",	RTFidSHPBXPAGE,	DOClevPARA, docRtfShpProperty, },
 	{ "shpbxmargin",RTFidSHPBXMARGIN,DOClevPARA,docRtfShpProperty, },
 	{ "shpbxcolumn",RTFidSHPBXCOLUMN,DOClevPARA,docRtfShpProperty, },
+	{ "shpbxignore",RTFidSHPBXIGNORE,DOClevPARA,docRtfShpProperty, },
 	{ "shpbypage",	RTFidSHPBYPAGE,	DOClevPARA, docRtfShpProperty, },
 	{ "shpbymargin",RTFidSHPBYMARGIN,DOClevPARA,docRtfShpProperty, },
 	{ "shpbycolumn",RTFidSHPBYCOLUMN,DOClevPARA,docRtfShpProperty, },
+	{ "shpbyignore",RTFidSHPBYIGNORE,DOClevPARA,docRtfShpProperty, },
 	{ "shpbypara",	RTFidSHPBYPARA,	DOClevPARA, docRtfShpProperty, },
 	{ "shpwr",	RTFidSHPWR,	DOClevPARA, docRtfShpProperty, },
 	{ "shpwrk",	RTFidSHPWRK,	DOClevPARA, docRtfShpProperty, },
@@ -554,7 +555,9 @@ static int docRtfShapeNumber(	SimpleInputStream *	sis,
     switch( rcw->rcwId )
 	{
 	case RTFSHP_shapeType:
-	    /*SDEB(docRtfShapeTypeString(val));*/
+	    if  ( rcw->rcwId != RTFSHP_shapeType )
+		{ SDEB(docRtfShapeTypeString(val));	}
+
 	    ds->ds_shapeType= val;
 	    break;
 
@@ -1103,6 +1106,8 @@ static RtfControlWord	docRtfShapeProperties[]=
 
     { "bWMode",			RTFSHP_bWMode,
 				DOClevPARA, docRtfShapeNumber, },
+    { "fLayoutInCell",		RTFSHP_fLayoutInCell,
+				DOClevPARA, docRtfShapeNumber, },
 };
 
 static int docRtfShapeProperty(	SimpleInputStream *	sis,
@@ -1118,6 +1123,8 @@ static int docRtfShapeProperty(	SimpleInputStream *	sis,
 
     int				i;
 
+    const int			listOnly= 1;
+
     const RtfControlWord *	rcwSn;
 
     control= docRtfFindControl( sis, rrc, &c, controlWord, &gotArg, &locArg );
@@ -1145,7 +1152,7 @@ static int docRtfShapeProperty(	SimpleInputStream *	sis,
 	controlWord[i++]= c;
 	}
 
-    rcwSn= docRtfFindWord( controlWord, docRtfShapeProperties, 1 );
+    rcwSn= docRtfFindWord( controlWord, docRtfShapeProperties, listOnly );
     if  ( ! rcwSn )
 	{ SDEB(controlWord);	}
 
