@@ -4,7 +4,7 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stdlib.h>
 #   include	<string.h>
@@ -12,21 +12,29 @@
 #   include	<ctype.h>
 #   include	<charnames.h>
 
-#   include	<debugon.h>
+#   include	<appDebugon.h>
+
+#   include	<appUnit.h>
 
 #   include	"docRtf.h"
 
 /************************************************************************/
+/*									*/
 /*  An empty tag table.							*/
+/*									*/
 /************************************************************************/
+
 RtfControlWord	docRtfEmptyTable[]=
     {
 	{ 0, }
     };
 
 /************************************************************************/
+/*									*/
 /*  Reset the attributes in an RtfReadingContext to the defaults.	*/
+/*									*/
 /************************************************************************/
+
 void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
     {
     int		i;
@@ -61,37 +69,50 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/****************************************/
 				/* Attributes of the current position.	*/
 				/****************************************/
+    docInitTextFrameProperties( &(rrc->rrcTextFrameProperties) );
     docInitParagraphProperties( &(rrc->rrcParagraphProperties) );
-    docInitRowProperties( &(rrc->rrcRowProperties) );
     docInitCellProperties( &(rrc->rrcCellProperties) );
+    docInitRowProperties( &(rrc->rrcRowProperties) );
+    docInitSectionProperties( &(rrc->rrcSectionProperties) );
+
+    rrc->rrcExternalItemKind= DOCinBODY;
+
+    docInitDocumentList( &(rrc->rrcDocumentList) );
+
     docInitBorderProperties( &(rrc->rrcBorderProperties) );
     /*docInitShapeProperties( &(rrc->rrcShapeProperties) );*/
     docInitShape( &(rrc->rrcShape) );
     docInitTabStop( &(rrc->rrcTabStop) );
-    appInitDocumentGeometry( &(rrc->rrcDocumentGeometry) );
+
+    docInitParagraphNumber( &(rrc->rrcParagraphNumber) );
+    rrc->rrcWhichPnText= -1;
+    rrc->rrcParagraphNumbers= (ParagraphNumber *)0;
+    rrc->rrcParagraphNumberCount= 0;
+
     rrc->rrcColor.rgb8Red= rrc->rrcColor.rgb8Green= rrc->rrcColor.rgb8Blue= 0;
     rrc->rrcGotComponent= 0;
-				/****************************************/
-				/*  Paragraph properties.		*/
-				/****************************************/
-    rrc->rrcDefaultFont= -1;
+
+    appInvalidateTime( &(rrc->rrcTm) );
+    rrc->rrcInfoText= (unsigned char *)0;
 				/****************************************/
 				/*  Document properties.		*/
 				/****************************************/
     docInitFont( &(rrc->rrcCurrentFont) );
-    rrc->rrcStyleNumber= 0;
-    rrc->rrcStyleBasedOn= 0;
-    rrc->rrcStyleNext= 0;
+    docInitStyle( &(rrc->rrcStyle) );
 				/****************************************/
 				/*  For reading the font table.		*/
 				/****************************************/
     rrc->rrcInsertedObject= (InsertedObject *)0;
     rrc->rrcFieldNumber= -1;
+    rrc->rrcBookmarkName[0]= '\0';
+    rrc->rrcBookmarkSize= 0;
+    rrc->rrcTopBookmark= (RtfBookmarkLevel *)0;
 				/****************************************/
 				/*  For reading 'objects' and pictures.	*/
 				/*  For reading 'fields'.		*/
 				/****************************************/
     rrc->rrcJustAfterPntext= 0;
+    rrc->rrcPageAfterPntext= 0;
 				/****************************************/
 				/*  For coping with the way word saves	*/
 				/*  {\pntext ... }			*/
@@ -104,11 +125,31 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 void docRtfCleanReadingContext(	RtfReadingContext *	rrc	)
     {
     docCleanParagraphProperties( &(rrc->rrcParagraphProperties) );
-    docCleanRowProperties( &(rrc->rrcRowProperties) );
     /*docCleanCellProperties( &(rrc->rrcRowProperties) );*/
+    docCleanRowProperties( &(rrc->rrcRowProperties) );
+    docCleanSectionProperties( &(rrc->rrcSectionProperties) );
     /*docCleanShapeProperties( &(rrc->rrcShapeProperties) );*/
     docCleanShape( &(rrc->rrcShape) );
     docCleanFont( &(rrc->rrcCurrentFont) );
+
+    docCleanDocumentList( &(rrc->rrcDocumentList) );
+
+    docCleanStyle( &(rrc->rrcStyle) );
+
+    if  ( rrc->rrcParagraphNumbers )
+	{ free( rrc->rrcParagraphNumbers );	}
+
+#   if 0
+    rbl= rrc->rrcTopBookmark;
+    while( rbl )
+	{
+	RtfBookmarkLevel *	prev= rbl->rblPrevious;
+
+	free( rbl );
+
+	rbl= prev;
+	}
+#   endif
     }
 
 int docRtfCopyReadingContext(	RtfReadingContext *		to,
@@ -116,20 +157,47 @@ int docRtfCopyReadingContext(	RtfReadingContext *		to,
     {
     ParagraphProperties		ppCopy;
     RowProperties		rpCopy;
+    SectionProperties		spCopy;
+    DocumentStyle		dsCopy;
+
+    PropertyMask		ppChgMask;
+    PropertyMask		ppUpdMask;
+
+    PROPmaskCLEAR( &ppChgMask );
+
+    PROPmaskCLEAR( &ppUpdMask );
+    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
 
     docInitParagraphProperties( &ppCopy );
-    if  ( docCopyParagraphProperties( &ppCopy,
-					    &(from->rrcParagraphProperties) ) )
+    if  ( docUpdParaProperties( &ppChgMask, &ppCopy,
+				&ppUpdMask, &(from->rrcParagraphProperties) ) )
 	{ LDEB(1); return -1;	}
 
     docInitRowProperties( &rpCopy );
     if  ( docCopyRowProperties( &rpCopy, &(from->rrcRowProperties) ) )
 	{ LDEB(1); return -1;	}
 
+    docInitSectionProperties( &spCopy );
+    if  ( docCopySectionProperties( &spCopy, &(from->rrcSectionProperties) ) )
+	{ LDEB(1); return -1;	}
+
+    docInitStyle( &dsCopy );
+    if  ( docCopyStyle( &dsCopy, &(from->rrcStyle) ) )
+	{ LDEB(1); return -1;	}
+
     *to= *from;
 
+    to->rrcParagraphNumbers= (ParagraphNumber *)0;
+    to->rrcParagraphNumberCount= 0;
+
     to->rrcParagraphProperties= ppCopy;
     to->rrcRowProperties= rpCopy;
+    to->rrcSectionProperties= spCopy;
+    to->rrcStyle= dsCopy;
+
+    to->rrcTopBookmark= from->rrcTopBookmark;
+
+    docInitDocumentList( &(to->rrcDocumentList) );
 
     return 0;
     }
@@ -144,9 +212,23 @@ void docRtfCopyReadingContextBack(	RtfReadingContext *	to,
     to->rrcCurrentLine= from->rrcCurrentLine;
     to->rrcFieldNumber= from->rrcFieldNumber;
     to->rrcJustAfterPntext= from->rrcJustAfterPntext;
+    to->rrcPageAfterPntext= from->rrcPageAfterPntext;
+
+    memcpy( to->rrcBookmarkName, from->rrcBookmarkName, DOCmaxBOOKMARK+1 );
+    to->rrcBookmarkSize= from->rrcBookmarkSize;
+    to->rrcTopBookmark= from->rrcTopBookmark;
+    from->rrcTopBookmark= (RtfBookmarkLevel *)0;
 
     to->rrcCurrentFont= from->rrcCurrentFont;
     docInitFont( &(from->rrcCurrentFont) );
+
+    to->rrcParagraphNumber= from->rrcParagraphNumber;
+
+    to->rrcExternalItemKind= from->rrcExternalItemKind;
+
+    to->rrcTm= from->rrcTm;
+
+    docInitDocumentList( &(to->rrcDocumentList) );
     }
 
 /************************************************************************/
@@ -154,6 +236,7 @@ void docRtfCopyReadingContextBack(	RtfReadingContext *	to,
 /*  Read a control word from the input stream.				*/
 /*									*/
 /************************************************************************/
+
 static int docRtfReadControlWord(	SimpleInputStream *	sis,
 					RtfReadingContext *	rrc,
 					int *			pC,
@@ -212,22 +295,50 @@ static int docRtfReadControlWord(	SimpleInputStream *	sis,
     /*HACK*/
     if  ( ! strcmp( controlWord, "endash" )	||
           ! strcmp( controlWord, "emdash" )	)
-	{ *pC= ISO1_hyphen; return 2;	}
+	{
+	*pC= ISO1_hyphen;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "rquote" )	)
-	{ *pC= ISO1_quoteright; return 2;	}
+	{
+	*pC= ISO1_quoteright;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "lquote" )	)
-	{ *pC= ISO1_quoteleft; return 2;	}
+	{
+	*pC= ISO1_quoteleft;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "ldblquote" )	)
-	{ *pC= ISO1_quotedbl; return 2;	}
+	{
+	*pC= ISO1_quotedbl;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "rdblquote" )	)
-	{ *pC= ISO1_quotedbl; return 2;	}
+	{
+	*pC= ISO1_quotedbl;
+	return 2;
+	}
 
     if  ( ! strcmp( controlWord, "bullet" )	)
-	{ *pC= ISO1_periodcentered; return 2;	}
+	{
+	*pC= ISO1_periodcentered;
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+	return 2;
+	}
 
     if  ( c == '-' )
 	{ sign= -1; c= sioInGetCharacter( sis );	}
@@ -253,6 +364,7 @@ static int docRtfReadControlWord(	SimpleInputStream *	sis,
 /*  Find a Control word.						*/
 /*									*/
 /************************************************************************/
+
 const RtfControlWord * docRtfFindWord(	const char *		controlWord,
 					const RtfControlWord *	contolWords,
 					int			listOnly )
@@ -366,10 +478,12 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 		    switch( c )
 			{
 			case '\\': case '{': case '}':
-			    goto defaultCase;
+			    *pC= c; return RTFfiCHAR;
+
 			case '~':
 			    c= ISO1_nobreakspace;
-			    goto defaultCase;
+			    *pC= c; return RTFfiCHAR;
+
 			case '\'':
 			    if  ( res == 1 )
 				{
@@ -382,11 +496,15 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 				sscanf( b, "%x", &c );
 				}
 			    goto defaultCase;
+
 			default:
-			    goto defaultCase;
+			    if  ( res == 2 )
+				{ *pC= c; return RTFfiCHAR;	}
+			    else{ goto defaultCase;		}
 			}
 		    }
 		return RTFfiWORD;
+
 	    case '{':
 		c= sioInGetCharacter( sis );
 		while( c == '\n' || c == '\r' )
@@ -395,6 +513,7 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 			{ rrc->rrcCurrentLine++;	}
 		    c= sioInGetCharacter( sis );
 		    }
+
 		if  ( c == '\\' )
 		    {
 		    sioInUngetLastRead( sis );
@@ -405,7 +524,14 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 		    if  ( res > 0 )
 			{
 			if  ( c == '*' )
-			    { return RTFfiSTARGROUP;	}
+			    {
+			    res= docRtfReadControlWord( sis, rrc, &c,
+						controlWord, pGotArg, pArg );
+			    if  ( res )
+				{ LDEB(res); return -1;	}
+
+			    return RTFfiSTARGROUP;
+			    }
 
 			if  ( c=='\'' )
 			    {
@@ -418,7 +544,10 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 			    sscanf( b, "%x", &c );
 			    }
 
-			rrc->rrcCharacterAhead= c;
+			if  ( res != 2 )
+			    { rrc->rrcCharacterAhead= rrc->rrcInputMapping[c]; }
+			else{ rrc->rrcCharacterAhead= c; }
+
 			return RTFfiTEXTGROUP;
 			}
 
@@ -427,17 +556,21 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 
 		sioInUngetLastRead( sis );
 		return RTFfiTEXTGROUP;
+
 	    case '\n':
 		rrc->rrcCurrentLine++;
 		/*FALLTHROUGH*/
 	    case '\r':
 		continue;
+
 	    case '}':
 		*pC= c; return RTFfiCLOSE;
+
 	    case '\t':
 		*pC= c; return RTFfiTAB;
+
 	    default: defaultCase:
-		*pC= c; return RTFfiCHAR;
+		*pC= rrc->rrcInputMapping[c]; return RTFfiCHAR;
 	    }
 	}
     }
@@ -447,8 +580,14 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 /*  Adjust the level in the hierarchy for docRtfReadGroup().		*/
 /*									*/
 /*  1)  Empty paragraphs get one empty particule to have a height.	*/
+/*  3)  Jade puts its tabs inside the text and Word can handle this,	*/
+/*	so use this HACK to also support this.				*/
+/*	The same applies for the way MS-Word inserts bookmarks before	*/
+/*	list texts.							*/
+/*  5)  Jade again.							*/
 /*									*/
 /************************************************************************/
+
 int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 			int			toLevel,
 			int			textLevel	)
@@ -473,14 +612,51 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 	}
 
     while( toLevel < rrc->rrcLevel )
-	{ bi= bi->biParent; rrc->rrcLevel--; }
+	{
+	if  ( ! bi )
+	    { XDEB(bi); return -1;		}
+	if  ( ! bi->biParent )
+	    { XDEB(bi->biParent); return -1;	}
+
+	if  ( bi->biLevel == DOClevPARA )
+	    {
+	    PropertyMask	ppChgMask;
+	    PropertyMask	ppUpdMask;
+
+	    if  ( docRtfFlushBookmarks( -1, bi, rrc ) )
+		{ LDEB(1); return -1;	}
+
+	    PROPmaskCLEAR( &ppChgMask );
+
+	    PROPmaskCLEAR( &ppUpdMask );
+	    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+
+	    /*  3  */
+	    if  ( docUpdParaProperties( &ppChgMask, &(bi->biParaProperties),
+				&ppUpdMask, &(rrc->rrcParagraphProperties) ) )
+		{ LDEB(1); return -1;	}
+	    }
+
+	if  ( bi->biLevel == DOClevSECT )
+	    {
+	    int		changed= 0;
+
+	    docRenumberNotes( &changed, rrc->rrcBd );
+	    }
+
+	bi= bi->biParent; rrc->rrcLevel--;
+	}
+
     rrc->rrcSplitLevel= rrc->rrcLevel;
 
     while( rrc->rrcLevel < toLevel && rrc->rrcLevel < textLevel )
 	{
 	BufferItem *	fresh;
 
-	fresh= docInsertItem( bi, -1, rrc->rrcLevel+ 1 );
+	if  ( rrc->rrcLevel == DOClevDOC )
+	    { ; }
+
+	fresh= docInsertItem( rrc->rrcBd, bi, -1, rrc->rrcLevel+ 1 );
 	if  ( ! fresh )
 	    {
 	    SLDEB(docLevelStr(rrc->rrcLevel),fresh);
@@ -494,17 +670,31 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 
 	if  ( rrc->rrcLevel == DOClevPARA )
 	    {
+	    PropertyMask	ppChgMask;
+	    PropertyMask	ppUpdMask;
+
 	    BufferItem *	cellBi= bi->biParent;
 	    BufferItem *	rowBi= cellBi->biParent;
 
-	    if  ( docCopyParagraphProperties( &(bi->biParaProperties),
-					 &(rrc->rrcParagraphProperties) ) )
+	    PROPmaskCLEAR( &ppChgMask );
+
+	    PROPmaskCLEAR( &ppUpdMask );
+	    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+
+	    if  ( docUpdParaProperties( &ppChgMask, &(bi->biParaProperties),
+				&ppUpdMask, &(rrc->rrcParagraphProperties) ) )
 		{ LDEB(1); return -1;	}
 
+	    /*  5  */
+	    if  ( bi->biParaLeftIndentTwips < 0	 )
+		{ bi->biParaLeftIndentTwips= 0;	}
+	    if  ( bi->biParaFirstIndentTwips+ bi->biParaLeftIndentTwips < 0 )
+		{ bi->biParaFirstIndentTwips= -bi->biParaLeftIndentTwips; }
+
 	    if  ( rowBi && bi->biParaInTable )
 		{
-		if  ( cellBi->biGroupChildCount == 1	&&
-		      rowBi->biGroupChildCount == 1	)
+		if  ( cellBi->biChildCount == 1	&&
+		      rowBi->biChildCount == 1	)
 		    {
 		    if  ( ! rowBi->biRowHasTableParagraphs )
 			{
@@ -519,17 +709,21 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 		    if  ( ! rowBi->biRowHasTableParagraphs )
 			{
 			BufferItem *	insBi;
-			if  ( cellBi->biGroupChildCount > 1 )
+			BufferItem *	aftBi;
+
+			if  ( cellBi->biChildCount > 1 )
 			    {
-			    if  ( docSplitGroupItem( &insBi, cellBi,
-						    bi->biNumberInParent ) )
+			    if  ( docSplitGroupItem(
+					rrc->rrcBd, &insBi, &aftBi, cellBi,
+					bi->biNumberInParent, DOClevCELL ) )
 				{ LDEB(1); return -1;	}
 			    }
 
-			if  ( rowBi->biGroupChildCount > 1 )
+			if  ( rowBi->biChildCount > 1 )
 			    {
-			    if  ( docSplitGroupItem( &insBi, rowBi,
-						cellBi->biNumberInParent ) )
+			    if  ( docSplitGroupItem(
+					rrc->rrcBd, &insBi, &aftBi, rowBi,
+					cellBi->biNumberInParent, DOClevROW ) )
 				{ LDEB(1); return -1;	}
 			    }
 
@@ -554,12 +748,32 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 		{ LDEB(1); return -1;	}
 	    }
 
-	if  ( rrc->rrcLevel == DOClevCELL )
+	if  ( rrc->rrcLevel == DOClevSECT )
 	    {
-	    if  ( docCopyCellProperties( &(bi->biCellProperties),
-						&(rrc->rrcCellProperties) ) )
+	    if  ( docCopySectionProperties( &(bi->biSectProperties),
+					     &(rrc->rrcSectionProperties) ) )
 		{ LDEB(1); return -1;	}
+
+	    if  ( rrc->rrcParagraphNumberCount > 0 )
+		{
+		if  ( bi->biSectParagraphNumbers )
+		    {
+		    XDEB(bi->biSectParagraphNumbers);
+		    free( bi->biSectParagraphNumbers );
+		    }
+
+		bi->biSectParagraphNumbers= rrc->rrcParagraphNumbers;
+		bi->biSectParagraphNumberCount= rrc->rrcParagraphNumberCount;
+
+		rrc->rrcParagraphNumbers= (ParagraphNumber *)0;
+		rrc->rrcParagraphNumberCount= 0;
+		}
 	    }
+
+	/*
+	if  ( rrc->rrcLevel == DOClevCELL )
+	    { }
+	*/
 	}
 
     rrc->rrcSplitLevel= rrc->rrcLevel;
@@ -572,6 +786,7 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 /*  Consume free text upto a control word or a group.			*/
 /*									*/
 /************************************************************************/
+
 static int docRtfReadText(	SimpleInputStream *	sis,
 				int			c,
 				int *			pC,
@@ -610,12 +825,15 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 
     for (;;)
 	{
+	int		map= 1;
+
 	c= sioInGetCharacter( sis );
 
 	switch( c )
 	    {
 	    case EOF:
 		LDEB(c); return -1;
+
 	    case '}':
 		if  ( len > 0 )
 		    {
@@ -623,6 +841,7 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 			{ LDEB(len); return -1;	}
 		    }
 		return RTFfiCLOSE;
+
 	    case '{':
 		sioInUngetLastRead( sis );
 		res= docRtfFindControl( sis, rrc, &c,
@@ -632,11 +851,14 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 		if  ( (*addParticule)( rrc, collectedText, len ) )
 		    { LDEB(len); return -1;	}
 		*pC= c; return res;
+
 	    case '\n':
 		rrc->rrcCurrentLine++;
 		/*FALLTHROUGH*/
+
 	    case '\r' :
 		continue;
+
 	    case '\\':
 		sioInUngetLastRead( sis );
 		res= docRtfFindControl( sis, rrc, &c,
@@ -651,7 +873,10 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 
 		    return res;
 		    }
+
+		map= 0;
 		/*FALLTHROUGH*/
+
 	    default:
 		if  ( len >= collectedSize )
 		    {
@@ -667,9 +892,12 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 		    collectedText= fresh;
 		    collectedSize= size;
 		    }
-		collectedText[len++]= c;
+		if  ( map )
+		    { collectedText[len++]= rrc->rrcInputMapping[c];	}
+		else{ collectedText[len++]= c;				}
 		collectedText[len  ]= '\0';
 		break;
+
 	    case '\t':
 		if  ( (*addParticule)( rrc, collectedText, len ) )
 		    { LDEB(len); return -1;	}
@@ -685,10 +913,11 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 /*  Skip an unknown or superfluous group.				*/
 /*									*/
 /************************************************************************/
+
 int docRtfSkipGroup(	SimpleInputStream *	sis,
 			const RtfControlWord *	rcw,
 			int			textLevel,
-			RtfReadingContext *	rrc		)
+			RtfReadingContext *	rrc )
     {
     int		complainUnknown= rrc->rrcComplainUnknown;
 
@@ -709,6 +938,7 @@ int docRtfSkipGroup(	SimpleInputStream *	sis,
 /*  Read an RTF control group. typically the document.			*/
 /*									*/
 /************************************************************************/
+
 int docRtfReadGroup(	SimpleInputStream *	sis,
 			int			textLevel,
 
@@ -750,14 +980,74 @@ int docRtfReadGroup(	SimpleInputStream *	sis,
     return rval;
     }
 
+/************************************************************************/
+/*									*/
+/*  Read an unknown group.						*/
+/*									*/
+/************************************************************************/
+
+int docRtfReadUnknownGroup(	SimpleInputStream *	sis,
+				RtfReadingContext *	rrc )
+    {
+    int		complainUnknown= rrc->rrcComplainUnknown;
+
+    rrc->rrcComplainUnknown= 0;
+
+    if  ( docRtfReadGroup( sis, rrc->rrcLevel,
+					(RtfControlWord *)0, 0, 0, rrc,
+					docRtfEmptyTable, docRtfEmptyTable,
+					docRtfIgnoreParticule ) )
+	{ LDEB(1); return -1;	}
+
+    rrc->rrcComplainUnknown= complainUnknown;
+
+    return 0;
+    }
+
+int docRtfReadWordGroup(	SimpleInputStream *	sis,
+				int			textLevel,
+				RtfReadingContext *	rrc,
+				int			gotArg,
+				int			arg,
+				const char *		controlWord,
+				const RtfControlWord *	contolWords,
+				const RtfControlWord *	groupWords,
+				RtfAddTextParticule	addParticule )
+    {
+    const RtfControlWord *	rcw;
+    const int			listOnly= 0;
+
+    rcw= docRtfFindWord( controlWord, contolWords, listOnly );
+    if  ( rcw )
+	{
+	if  ( rcw->rcwLevel != DOClevANY		&&
+	      docRtfAdjustLevel( rrc,
+			    rcw->rcwLevel, textLevel )	)
+	    { SDEB(controlWord); return -1; }
+
+	if  ( docRtfReadGroup( sis, textLevel,
+				rcw, gotArg, arg, rrc,
+				contolWords, groupWords,
+				addParticule ) )
+	    { SDEB(rcw->rcwWord); return -1;	}
+	}
+    else{
+	if  ( rrc->rrcComplainUnknown )
+	    { SDEB(controlWord);	}
+
+	if  ( docRtfReadUnknownGroup( sis, rrc ) )
+	    { LDEB(1); return -1;	}
+	}
+
+    return 0;
+    }
+
 int docRtfConsumeGroup(	SimpleInputStream *	sis,
 			int			textLevel,
-
 			RtfReadingContext *	rrc,
-
 			const RtfControlWord *	contolWords,
 			const RtfControlWord *	groupWords,
-			RtfAddTextParticule	addParticule	)
+			RtfAddTextParticule	addParticule )
     {
     int				res;
     const RtfControlWord *	rcw;
@@ -777,12 +1067,11 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 
     for (;;)
 	{
-	int	complainUnknown;
-
 	switch( res )
 	    {
 	    case RTFfiCLOSE:
 		return 0;
+
 	    case RTFfiCHAR:
 		if  ( textLevel != DOClevANY				&&
 		      docRtfAdjustLevel( rrc, textLevel, textLevel )	)
@@ -865,37 +1154,10 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		rcw= docRtfFindWord( controlWord, groupWords, 1 );
 		if  ( ! rcw )
 		    {
-		    rcw= docRtfFindWord( controlWord, contolWords, 0 );
-		    if  ( rcw )
-			{
-			if  ( rcw->rcwLevel != DOClevANY		&&
-			      docRtfAdjustLevel( rrc,
-					    rcw->rcwLevel, textLevel )	)
-			    { SDEB(controlWord); return -1; }
-
-			if  ( docRtfReadGroup( sis, textLevel,
-						rcw, gotArg, arg, rrc,
-						contolWords, groupWords,
-						addParticule ) )
-			    { SDEB(rcw->rcwWord); return -1;	}
-			}
-		    else{
-			complainUnknown= rrc->rrcComplainUnknown;
-
-			if  ( rrc->rrcComplainUnknown )
-			    { SDEB(controlWord);	}
-
-			rrc->rrcComplainUnknown= 0;
-
-			if  ( docRtfReadGroup( sis,
-				    rrc->rrcLevel,
-				    (RtfControlWord *)0, 0, 0, rrc,
-				    docRtfEmptyTable, docRtfEmptyTable,
-				    docRtfIgnoreParticule ) )
-			    { SDEB(rcw->rcwWord); return -1;	}
-
-			rrc->rrcComplainUnknown= complainUnknown;
-			}
+		    if  ( docRtfReadWordGroup( sis, textLevel, rrc,
+				    gotArg, arg, controlWord,
+				    contolWords, groupWords, addParticule ) )
+			{ SDEB(controlWord); return -1;	}
 		    }
 		else{
 		  groupFound:
@@ -916,31 +1178,19 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		    { LDEB(res); return -1;	}
 		break;
 	    case RTFfiSTARGROUP:
-		res= docRtfReadControlWord( sis, rrc, &c,
-					    controlWord, &gotArg, &arg );
-		if  ( res )
-		    { LDEB(res); return -1;	}
 		rcw= docRtfFindWord( controlWord, groupWords, 1 );
 		if  ( rcw )
 		    { goto groupFound; }
 
-		complainUnknown= rrc->rrcComplainUnknown;
-
-		rrc->rrcComplainUnknown= 0;
-
-		if  ( docRtfReadGroup( sis, rrc->rrcLevel,
-			    (RtfControlWord *)0, 0, 0, rrc,
-			    docRtfEmptyTable, docRtfEmptyTable,
-			    docRtfIgnoreParticule ) )
-		    { SDEB(rcw->rcwWord); return -1;	}
-
-		rrc->rrcComplainUnknown= complainUnknown;
+		if  ( docRtfReadUnknownGroup( sis, rrc ) )
+		    { LDEB(1); return -1;	}
 
 		res= docRtfFindControl( sis, rrc, &c,
 					    controlWord, &gotArg, &arg );
 		if  ( res < 0 )
 		    { LDEB(res); return -1;	}
 		break;
+
 	    case RTFfiTEXTGROUP:
 		if  ( docRtfReadGroup( sis, textLevel,
 					(RtfControlWord *)0, 0, 0, rrc,
@@ -953,6 +1203,7 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		if  ( res < 0 )
 		    { LDEB(res); return -1;	}
 		break;
+
 	    default:
 		LDEB(res); return -1;
 	    }
@@ -963,3 +1214,8 @@ int docRtfIgnoreText(	RtfReadingContext *	rrc,
 			const unsigned char *	text,
 			int			len	)
     { return 0; }
+
+int docRtfRefuseText(	RtfReadingContext *	rrc,
+			const unsigned char *	text,
+			int			len	)
+    { LDEB(1); return -1; }
