@@ -4,43 +4,51 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"tedConfig.h"
 
 #   include	<stddef.h>
 #   include	<stdio.h>
 #   include	<stdlib.h>
+#   include	<string.h>
 
-#   include	"tedApp.h"
-#   include	<appImage.h>
 #   include	<sioMemory.h>
 #   include	<sioEndian.h>
 #   include	<sioStdio.h>
 #   include	<sioHex.h>
 
-#   include	<debugon.h>
-
+#   include	<appDraw.h>
+#   include	<appImage.h>
 #   include	<appWinMeta.h>
+#   include	<appMacPict.h>
+
+#   include	"docLayout.h"
+#   include	"tedApp.h"
+
+#   include	<appDebugon.h>
 
 int tedDrawObject(		const BufferItem *	bi,
-				TextParticule *		tp,
-				int			y,
-				const DrawingContext *	dc )
+				const TextParticule *	tp,
+				int			baseline,
+				int			ox,
+				int			oy,
+				AppDrawingData *	add )
     {
     InsertedObject *	io= bi->biParaObjects+ tp->tpObjectNumber;
-    Display *		display= dc->dcDrawingData->addDisplay;
 
     if  ( tp->tpPhysicalFont < 0 )
 	{ LDEB(tp->tpPhysicalFont); return -1;	}
 
     if  ( ( io->ioKind == DOCokPICTWMETAFILE	||
+	    io->ioKind == DOCokMACPICT		||
 	    io->ioKind == DOCokPICTPNGBLIP	||
-	    io->ioKind == DOCokPICTJPEGBLIP	)	&&
+	    io->ioKind == DOCokPICTJPEGBLIP	||
+	    io->ioKind == DOCokINCLUDEPICTURE	)	&&
 	  io->ioPixmap					)
 	{
-	XCopyArea( display, io->ioPixmap, dc->dcWindow, dc->dcGc,
+	appDrawDrawPixmap( add, io->ioPixmap,
 	    0, 0, /* src: x,y */
 	    io->ioPixelsWide, io->ioPixelsHigh, /* wide, high */
-	    tp->tpX0- dc->dcOx, y- io->ioPixelsHigh- dc->dcOy ); /* dest: x,y */
+	    tp->tpX0- ox, baseline- io->ioPixelsHigh- oy ); /* dest: x,y */
 		    
 	return 0;
 	}
@@ -49,48 +57,54 @@ int tedDrawObject(		const BufferItem *	bi,
 	  io->ioResultKind == DOCokPICTWMETAFILE	&&
 	  io->ioPixmap					)
 	{
-	XCopyArea( display, io->ioPixmap, dc->dcWindow, dc->dcGc,
+	appDrawDrawPixmap( add, io->ioPixmap,
 	    0, 0, /* src: x,y */
 	    io->ioPixelsWide, io->ioPixelsHigh, /* wide, high */
-	    tp->tpX0- dc->dcOx, y- io->ioPixelsHigh- dc->dcOy ); /* dest: x,y */
+	    tp->tpX0- ox, baseline- io->ioPixelsHigh- oy ); /* dest: x,y */
 		    
 	return 0;
 	}
 
-    /*
-    LLDEB(io->ioKind,io->ioResultKind);
-    XDEB(io->ioPrivate);
-    */
+#   ifdef USE_MOTIF
+    XSetLineAttributes( add->addDisplay, add->addGc,
+					1, LineSolid, CapButt, JoinMiter );
+#   endif
 
-    XDrawRectangle( display, dc->dcWindow, dc->dcGc,
-		tp->tpX0- dc->dcOx, y- io->ioPixelsHigh- dc->dcOy,
+#   ifdef USE_GTK
+    gdk_gc_set_line_attributes( add->addGc,
+		    1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER );
+#   endif
+
+    appDrawDrawRectangle( add,
+		tp->tpX0- ox, baseline- io->ioPixelsHigh- oy,
 		io->ioPixelsWide, io->ioPixelsHigh );
     return 0;
     }
 
+typedef int (*PLAY_META_X11)(	SimpleInputStream *	sis,
+				void **			pPrivate,
+				AppColors *		ac,
+				AppDrawingData *	add,
+				APP_WINDOW		pixmap,
+				int			mapMode,
+				int			xExt,
+				int			yExt,
+				int			pixelsWide,
+				int			pixelsHigh,
+				int			twipsWide,
+				int			twipsHigh );
+
 static int tedOpenMetafileObject(	InsertedObject *	io,
+					PLAY_META_X11		playMetaX11,
 					AppColors *		ac,
-					DocumentFontList *	dfl,
 					AppDrawingData *	add,
-					const ObjectData *	od,
-					Window			win )
+					int			mapMode,
+					const MemoryBuffer *	mb )
     {
-    Display *		display= add->addDisplay;
-    int			screen= DefaultScreen( display );
-    int			depth= DefaultDepth( display, screen );
-
-    int			xExt= io->ioXExtent;
-    int			yExt= io->ioYExtent;
-
     SimpleInputStream *	sisMem;
     SimpleInputStream *	sisMeta;
 
-    MemoryBuffer	mb;
-
-    mb.mbCapacity= od->odSize;
-    mb.mbBytes= od->odBytes;
-
-    sisMem= sioInMemoryOpen( &mb );
+    sisMem= sioInMemoryOpen( mb );
     if  ( ! sisMem )
 	{ XDEB(sisMem); return -1;	}
 
@@ -98,16 +112,15 @@ static int tedOpenMetafileObject(	InsertedObject *	io,
     if  ( ! sisMeta )
 	{ XDEB(sisMem); return -1;	}
 
-    io->ioPixmap= XCreatePixmap( display, win,
-				io->ioPixelsWide, io->ioPixelsHigh, depth );
+    io->ioPixmap= appMakePixmap( add, io->ioPixelsWide, io->ioPixelsHigh );
     if  ( ! io->ioPixmap )
 	{ XDEB(io->ioPixmap); return -1;	}
 
-    if  ( appMetaPlayFileX11( sisMeta, &io->ioPrivate, ac, dfl, add,
-		    io->ioPixmap, xExt, yExt,
-		    io->ioPixelsWide, io->ioPixelsHigh,
-		    io->ioTwipsWide, io->ioTwipsHigh,
-		    win ) )
+    if  ( (*playMetaX11)( sisMeta, &(io->ioPrivate), ac, add,
+					io->ioPixmap, mapMode,
+					io->io_xWinExt, io->io_yWinExt,
+					io->ioPixelsWide, io->ioPixelsHigh,
+					io->ioTwipsWide, io->ioTwipsHigh ) )
 	{ LDEB(1);	}
 
     sioInClose( sisMeta );
@@ -116,82 +129,83 @@ static int tedOpenMetafileObject(	InsertedObject *	io,
     return 0;
     }
 
-typedef int (*TedReadBitmap)(	BitmapDescription *	pBd,
-				unsigned char **	pBuffer,
-				SimpleInputStream *	sis );
-
-static int tedOpenBitmapObject(	TedReadBitmap		readBitmap,
+static int tedOpenBitmapObject(	bmReadBitmap		readBitmap,
 				InsertedObject *	io,
 				double			pixelsPerTwip,
 				AppColors *		ac,
-				ObjectData *		od,
-				Display *		display,
-				Window			win,
-				GC			gc )
+				AppDrawingData *	add,
+				const MemoryBuffer *	mb )
     {
-    int			screen= DefaultScreen( display );
-    int			depth= DefaultDepth( display, screen );
-
     AppBitmapImage *	abi= (AppBitmapImage *)malloc( sizeof(AppBitmapImage) );
 
-    XImage *		xim= (XImage *)0;
-
     SimpleInputStream *	sisMem;
-    SimpleInputStream *	sisPng;
+    SimpleInputStream *	sisBitmap;
 
-    MemoryBuffer	mb;
-
-    mb.mbCapacity= od->odSize;
-    mb.mbBytes= od->odBytes;
+    APP_BITMAP_IMAGE	pixmap;
 
     if  ( ! abi )
 	{ XDEB(abi); return -1;	}
     appInitBitmapImage( abi );
 
-    io->ioPixmap= XCreatePixmap( display, win,
-				io->ioPixelsWide, io->ioPixelsHigh, depth );
-    if  ( ! io->ioPixmap )
-	{ XDEB(io->ioPixmap); return -1;	}
-
-    sisMem= sioInMemoryOpen( &mb );
+    sisMem= sioInMemoryOpen( mb );
     if  ( ! sisMem )
 	{ XDEB(sisMem); return -1;	}
 
-    sisPng= sioInHexOpen( sisMem );
-    if  ( ! sisPng )
+    sisBitmap= sioInHexOpen( sisMem );
+    if  ( ! sisBitmap )
 	{ XDEB(sisMem); return -1;	}
 
-    if  ( (*readBitmap)( &abi->abiBitmap, &abi->abiBuffer, sisPng ) )
+    if  ( (*readBitmap)( &abi->abiBitmap, &abi->abiBuffer, sisBitmap ) )
 	{ LDEB(1); return -1;	}
 
-    sioInClose( sisPng );
+    sioInClose( sisBitmap );
     sioInClose( sisMem );
 
-    if  ( appImgMakeImage( display, screen,
-			&xim, io->ioPixelsWide, io->ioPixelsHigh, ac, abi ) )
+    if  ( appImgMakePixmap( add, &pixmap,
+			    io->ioPixelsWide, io->ioPixelsHigh, ac, abi ) )
 	{
 	LDEB(1);
-
 	appCleanBitmapImage( abi ); free( abi );
-
 	return -1;
 	}
 
-    XPutImage( display, io->ioPixmap, gc, xim, 0, 0, 0, 0,
-					io->ioPixelsWide, io->ioPixelsHigh );
-    XDestroyImage( xim );
-
+    io->ioPixmap= pixmap;
     io->ioPrivate= (void *)abi;
 
     return 0;
     }
 
+static int tedOpenEpsfObject(		InsertedObject *	io,
+					AppColors *		ac,
+					AppDrawingData *	add,
+					const MemoryBuffer *	mb )
+    {
+#   if 0
+    SimpleInputStream *	sisMem;
+    SimpleInputStream *	sisMeta;
+
+    sisMem= sioInMemoryOpen( mb );
+    if  ( ! sisMem )
+	{ XDEB(sisMem); return -1;	}
+
+    sisMeta= sioInHexOpen( sisMem );
+    if  ( ! sisMeta )
+	{ XDEB(sisMem); return -1;	}
+
+    io->ioPixmap= appMakePixmap( add, io->ioPixelsWide, io->ioPixelsHigh );
+    if  ( ! io->ioPixmap )
+	{ XDEB(io->ioPixmap); return -1;	}
+
+    sioInClose( sisMeta );
+    sioInClose( sisMem );
+
+#   endif
+    return 0;
+    }
+
 static int tedOpenObject(	InsertedObject *	io,
 				AppColors *		ac,
-				DocumentFontList *	dfl,
-				AppDrawingData *	add,
-				Window			win,
-				GC			gc )
+				AppDrawingData *	add )
     {
     if  ( io->ioTwipsWide < 10 || io->ioTwipsHigh < 10 )
 	{ LLDEB(io->ioTwipsWide,io->ioTwipsHigh); return -1; }
@@ -200,38 +214,70 @@ static int tedOpenObject(	InsertedObject *	io,
 	{
 	case DOCokPICTWMETAFILE:
 	    if  ( ! io->ioPrivate					&&
-		  tedOpenMetafileObject( io, ac, dfl, add,
-					&io->ioObjectData, win )	)
+		  tedOpenMetafileObject( io, appMetaPlayWmfX11,
+						ac, add, io->ioMapMode,
+						&(io->ioObjectData) )	)
 		{ LDEB(1); return 0;	}
-	    break;
+	    return 0;
+
+	case DOCokPICTEMFBLIP:
+	    if  ( ! io->ioPrivate					&&
+		  tedOpenMetafileObject( io, appMetaPlayEmfX11,
+						ac, add, io->ioMapMode,
+						&(io->ioObjectData) )	)
+		{ LDEB(1); return 0;	}
+	    return 0;
+
+	case DOCokMACPICT:
+	    if  ( ! io->ioPrivate					&&
+		  tedOpenMetafileObject( io, appMacPictPlayFileX11,
+						ac, add, io->ioMapMode,
+						&(io->ioObjectData) )	)
+		{ LDEB(1); return 0;	}
+	    return 0;
 
 	case DOCokPICTPNGBLIP:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenBitmapObject( bmPngReadPng, io,
-			add->addMagnifiedPixelsPerTwip, ac,
-			&io->ioObjectData, add->addDisplay, win, gc )	)
+				    add->addMagnifiedPixelsPerTwip,
+					ac, add, &(io->ioObjectData) )	)
 		{ LDEB(1); return 0;	}
-	    break;
+	    return 0;
 
 	case DOCokPICTJPEGBLIP:
 	    if  ( ! io->ioPrivate					&&
 		  tedOpenBitmapObject( bmJpegReadJfif, io,
-			add->addMagnifiedPixelsPerTwip, ac,
-			&io->ioObjectData, add->addDisplay, win, gc )	)
+				    add->addMagnifiedPixelsPerTwip,
+					ac, add, &(io->ioObjectData) )	)
 		{ LDEB(1); return 0;	}
-	    break;
+	    return 0;
 
 	case DOCokOLEOBJECT:
 	    if  ( io->ioResultKind == DOCokPICTWMETAFILE )
 		{
-		if  ( ! io->ioPrivate				&&
-		      tedOpenMetafileObject( io, ac, dfl, add,
-					&io->ioResultData, win )	)
+		if  ( ! io->ioPrivate					&&
+		      tedOpenMetafileObject( io, appMetaPlayWmfX11,
+					ac, add, io->ioResultMapMode,
+					&(io->ioResultData) )	)
 		    { LDEB(1); return 0;	}
 
 		return 0;
 		}
 	    return 0;
+
+	case DOCokINCLUDEPICTURE:
+
+	    if  ( io->ioResultKind == DOCokEPSF )
+		{
+		if  ( ! io->ioPixmap					&&
+		      tedOpenEpsfObject( io,
+					ac, add, &io->ioResultData )	)
+		    { LDEB(1); return 0;	}
+
+		return 0;
+		}
+	    else{ LLDEB(io->ioKind,io->ioResultKind); return 0;	}
+
 	default:
 	    LDEB(io->ioKind); return 0;
 	}
@@ -241,10 +287,7 @@ static int tedOpenObject(	InsertedObject *	io,
 
 static int tedOpenParaObjects(	BufferItem *		bi,
 				AppColors *		ac,
-				DocumentFontList *	dfl,
-				AppDrawingData *	add,
-				Window			win,
-				GC			gc )
+				AppDrawingData *	add )
     {
     int			part;
     TextParticule *	tp;
@@ -262,8 +305,8 @@ static int tedOpenParaObjects(	BufferItem *		bi,
 
 	io= bi->biParaObjects+ tp->tpObjectNumber;
 
-	if  ( ! io->ioPrivate						&&
-	      tedOpenObject( io, ac, dfl, add, win, gc )	)
+	if  ( ! io->ioPrivate			&&
+	      tedOpenObject( io, ac, add )	)
 	    { LDEB(part); continue;	}
 	}
 
@@ -272,31 +315,70 @@ static int tedOpenParaObjects(	BufferItem *		bi,
 
 int tedOpenItemObjects(	BufferItem *		bi,
 			AppColors *		ac,
-			DocumentFontList *	dfl,
-			AppDrawingData *	add,
-			Window			win,
-			GC			gc )
+			AppDrawingData *	add )
     {
     int		i;
 
     switch( bi->biLevel )
 	{
-	case DOClevDOC:
 	case DOClevSECT:
+	    for ( i= 0; i < bi->biChildCount; i++ )
+		{
+		if  ( tedOpenItemObjects( bi->biChildren[i], ac, add ) )
+		    { LDEB(i); return -1;	}
+		}
+
+	    if  ( bi->biSectHeader.eiItem				&&
+		  tedOpenItemObjects( bi->biSectHeader.eiItem,
+							ac, add )	)
+		{ LDEB(1); return -1;	}
+	    if  ( bi->biSectFirstPageHeader.eiItem			&&
+		  tedOpenItemObjects( bi->biSectFirstPageHeader.eiItem,
+							ac, add )	)
+		{ LDEB(1); return -1;	}
+	    if  ( bi->biSectLeftPageHeader.eiItem			&&
+		  tedOpenItemObjects( bi->biSectLeftPageHeader.eiItem,
+							ac, add )	)
+		{ LDEB(1); return -1;	}
+	    if  ( bi->biSectRightPageHeader.eiItem			&&
+		  tedOpenItemObjects( bi->biSectRightPageHeader.eiItem,
+							ac, add )	)
+		{ LDEB(1); return -1;	}
+
+	    if  ( bi->biSectFooter.eiItem				&&
+		  tedOpenItemObjects( bi->biSectFooter.eiItem,
+							ac, add )	)
+		{ LDEB(1); return -1;	}
+	    if  ( bi->biSectFirstPageFooter.eiItem			&&
+		  tedOpenItemObjects( bi->biSectFirstPageFooter.eiItem,
+							ac, add )	)
+		{ LDEB(1); return -1;	}
+	    if  ( bi->biSectLeftPageFooter.eiItem			&&
+		  tedOpenItemObjects( bi->biSectLeftPageFooter.eiItem,
+							ac, add )	)
+		{ LDEB(1); return -1;	}
+	    if  ( bi->biSectRightPageFooter.eiItem			&&
+		  tedOpenItemObjects( bi->biSectRightPageFooter.eiItem,
+							ac, add )	)
+		{ LDEB(1); return -1;	}
+
+	    break;
+
+	case DOClevDOC:
 	case DOClevROW:
 	case DOClevCELL:
-	    for ( i= 0; i < bi->biGroupChildCount; i++ )
+	    for ( i= 0; i < bi->biChildCount; i++ )
 		{
-		if  ( tedOpenItemObjects( bi->biGroupChildren[i],
-						    ac, dfl, add, win, gc ) )
+		if  ( tedOpenItemObjects( bi->biChildren[i], ac, add ) )
 		    { LDEB(i); return -1;	}
 		}
 	    break;
-	case DOClevPARA:
-	    if  ( tedOpenParaObjects( bi, ac, dfl, add, win, gc ) )
-		{ LDEB(0); return -1;	}
 
+	case DOClevPARA:
+	    if  ( tedOpenParaObjects( bi, ac, add ) )
+		{ LDEB(0); return -1;	}
 	    break;
+
 	default:
 	    LDEB(bi->biLevel); return -1;
 	}
@@ -308,13 +390,12 @@ int tedOpenItemObjects(	BufferItem *		bi,
 void tedCloseObject(		BufferDocument *	bd,
 				BufferItem *		bi,
 				TextParticule *		tp,
-				void *			voiddisplay )
+				void *			voidadd )
     {
-    Display *		display= (Display *)voiddisplay;
     InsertedObject *	io;
 
     if  ( tp->tpKind != DOCkindOBJECT	||
-	  tp->tpPhysicalFont < 0		)
+	  tp->tpPhysicalFont < 0	)
 	{ return;	}
 
     io= bi->biParaObjects+ tp->tpObjectNumber;
@@ -325,14 +406,17 @@ void tedCloseObject(		BufferDocument *	bd,
 	case DOCokPICTPNGBLIP:
 	case DOCokPICTJPEGBLIP:
 	case DOCokOLEOBJECT:
+	case DOCokMACPICT:
 	    if  ( io->ioPrivate )
 		{
 		AppBitmapImage *	abi= (AppBitmapImage *)io->ioPrivate;
 
 		if  ( io->ioPixmap )
 		    {
-		    XFreePixmap( display, io->ioPixmap );
-		    io->ioPixmap= (Pixmap)0;
+		    AppDrawingData *	add= (AppDrawingData *)voidadd;
+
+		    appDrawFreePixmap( add, io->ioPixmap );
+		    io->ioPixmap= (APP_WINDOW)0;
 		    }
 
 		appCleanBitmapImage( abi ); free( abi );
@@ -340,6 +424,17 @@ void tedCloseObject(		BufferDocument *	bd,
 		io->ioPrivate= (void *)0;
 		}
 	    break;
+
+	case DOCokINCLUDEPICTURE:
+	    if  ( io->ioPixmap )
+		{
+		AppDrawingData *	add= (AppDrawingData *)voidadd;
+
+		appDrawFreePixmap( add, io->ioPixmap );
+		io->ioPixmap= (APP_WINDOW)0;
+		}
+	    break;
+
 	default:
 	    LDEB(io->ioKind); break;
 	}
@@ -352,20 +447,19 @@ void tedCloseObject(		BufferDocument *	bd,
 /*  Called to resize an inserted object.				*/
 /*									*/
 /************************************************************************/
+
 int tedReopenObject(	BufferDocument *	bd,
 			BufferItem *		bi,
 			TextParticule *		tp,
 			AppColors *		ac,
 			DocumentFontList *	dfl,
-			AppDrawingData *	add,
-			Window			win,
-			GC			gc )
+			AppDrawingData *	add )
     {
     InsertedObject *		io= bi->biParaObjects+ tp->tpObjectNumber;
 
-    tedCloseObject( bd, bi, tp, (void *)add->addDisplay );
+    tedCloseObject( bd, bi, tp, (void *)add );
 
-    if  ( tedOpenObject( io, ac, dfl, add, win, gc ) )
+    if  ( tedOpenObject( io, ac, add ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
@@ -375,21 +469,32 @@ static void tedScaleObjectToParagraph(	EditDocument *		ed,
 					InsertedObject *	io )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    BufferItem *		bi= td->tdSelection.bsBegin.bpBi;
     BufferDocument *		bd= td->tdDocument;
-    const DocumentGeometry *	dg= &(bd->bdGeometry);
-    FormattingFrame		ff;
-    int				pageHigh;
+    ParagraphFrame		pf;
     double			scaleX;
     double			scaleY;
 
-    docParagraphFrame( &ff, dg, -1, bi );
-    scaleX= (double)( ff.ffX1TextLines- ff.ffX0TextLines )/
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
+    const int			bottom= -1;
+    const int			stripHigh= -1;
+
+    BlockFrame			bf;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ LDEB(1); return;	}
+
+    docBlockFrameTwips( &bf, ds.dsBegin.dpBi, bd,
+				    ds.dsBegin.dpBi->biTopPosition.lpPage,
+				    ds.dsBegin.dpBi->biTopPosition.lpColumn );
+
+    docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, ds.dsBegin.dpBi );
+
+    scaleX= (double)( pf.pfX1TextLinesTwips- pf.pfX0TextLinesTwips )/
 						    (double)io->ioTwipsWide;
 
-    pageHigh= dg->dgPaperHighTwips-
-			    dg->dgTopMarginTwips- dg->dgBottomMarginTwips;
-    scaleY= (double)pageHigh/ (double)io->ioTwipsHigh;
+    scaleY= (double)pf.pfPageHigh/ (double)io->ioTwipsHigh;
 
     if  ( scaleY > scaleX )
 	{ scaleY=  scaleX;	}
@@ -408,6 +513,7 @@ static void tedScaleObjectToParagraph(	EditDocument *		ed,
 /*  Save the picture of an object for Copy/Paste.			*/
 /*									*/
 /************************************************************************/
+
 int tedSaveObjectPicture(	AppBitmapImage *	abiTo,
 				InsertedObject *	io )
     {
@@ -415,6 +521,7 @@ int tedSaveObjectPicture(	AppBitmapImage *	abiTo,
     unsigned char *	buffer;
 
     if  ( ( io->ioKind == DOCokPICTWMETAFILE	||
+	    io->ioKind == DOCokMACPICT		||
 	    io->ioKind == DOCokPICTPNGBLIP	||
 	    io->ioKind == DOCokPICTJPEGBLIP	)	&&
 	  io->ioPrivate					)
@@ -426,7 +533,7 @@ int tedSaveObjectPicture(	AppBitmapImage *	abiTo,
 	{ abiFrom= (AppBitmapImage *)io->ioPrivate; }
 
     if  ( ! abiFrom )
-	{ LDEB(io->ioKind); return -1;	}
+	{ LXXDEB(io->ioKind,io->ioPrivate,abiFrom); return -1;	}
 
     buffer= (unsigned char *)malloc( abiFrom->abiBitmap.bdBufferLength );
     if  ( ! buffer )
@@ -448,90 +555,65 @@ int tedSaveObjectPicture(	AppBitmapImage *	abiTo,
 /*  9)  Not needed as the document is marked as changed anyway.		*/
 /*									*/
 /************************************************************************/
-int tedObjectInsertBitmap(	EditDocument *		ed,
-				AppBitmapImage *	abi )
+
+InsertedObject * tedObjectMakeBitmapObject(
+					EditDocument *		ed,
+					BufferItem *		bi,
+					AppBitmapImage *	abi )
     {
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    EditApplication *		ea= ed->edApplication;
+    AppDrawingData *		add= &(ed->edDrawingData);
+
+    APP_WINDOW			pixmap;
 
     InsertedObject *		io;
 
-    Display *			display= ea->eaDisplay;
-    Widget			w= ed->edDocumentWidget;
-    Window			win= XtWindow( w );
-    GC				gc= ed->edGc;
-
     MemoryBuffer		mb;
     SimpleOutputStream *	sosMem;
     SimpleOutputStream *	sosHex;
 
     int				objectNumber;
 
-    double			xfac= ea->eaMagnifiedPixelsPerTwip;
+    double			xfac= add->addMagnifiedPixelsPerTwip;
 
-    io= docClaimObject( &objectNumber, td->tdSelection.bsBegin.bpBi );
+    io= docClaimObject( &objectNumber, bi );
     if  ( ! io )
-	{ XDEB(io); return -1;	}
+	{ XDEB(io); return (InsertedObject *)0;	}
 
-    sioMemoryInit( &mb );
+    utilInitMemoryBuffer( &mb );
 
     sosMem= sioOutMemoryOpen( &mb );
     if  ( ! sosMem )
-	{ XDEB(sosMem); return -1;	}
+	{ XDEB(sosMem); return (InsertedObject *)0;	}
 
     sosHex= sioOutHexOpen( sosMem );
     if  ( ! sosHex )
-	{ XDEB(sosHex); return -1;	}
+	{ XDEB(sosHex); return (InsertedObject *)0;	}
 
     if  ( abi->abiFormat >= 0						&&
 	  ! strcmp( bmFileFormats[abi->abiFormat].bffFileType->bftTypeId,
 								"pngFile" ) )
 	{
 	if  ( bmPngWritePng( &abi->abiBitmap, abi->abiBuffer, sosHex ) )
-	    { LDEB(1); return -1;	}
+	    { LDEB(1); return (InsertedObject *)0;	}
 
 	io->ioKind= DOCokPICTPNGBLIP;
 	}
     else{
-	if  ( appMetaSaveBitmapMetafile( &abi->abiBitmap, abi->abiBuffer,
+	if  ( appMetaSaveBitmapMetafile( &(abi->abiBitmap), abi->abiBuffer,
 								    sosHex ) )
-	    { LDEB(1); return -1;	}
+	    { LDEB(1); return (InsertedObject *)0;	}
 
 	io->ioKind= DOCokPICTWMETAFILE;
+	io->ioMapMode= 8;
 	}
 
     sioOutClose( sosHex );
     sioOutClose( sosMem );
 
-    io->ioObjectData.odBytes= mb.mbBytes;
-    io->ioObjectData.odSize= mb.mbCapacity;
+    io->ioObjectData= mb;
 
-    switch( abi->abiBitmap.bdUnit )
-	{
-	case BMunCM:
-	    io->ioTwipsWide= (int)( ( POINTS_PER_CM* 20* abi->abiBitmap.bdPixelsWide )/
-					    abi->abiBitmap.bdXResolution );
-	    io->ioTwipsHigh= (int)( ( POINTS_PER_CM* 20* abi->abiBitmap.bdPixelsHigh )/
-					    abi->abiBitmap.bdYResolution );
-	    break;
-	case BMunINCH:
-	    io->ioTwipsWide= (int)( ( 72* 20* abi->abiBitmap.bdPixelsWide )/
-					    abi->abiBitmap.bdXResolution );
-	    io->ioTwipsHigh= (int)( ( 72* 20* abi->abiBitmap.bdPixelsHigh )/
-					    abi->abiBitmap.bdYResolution );
-	    break;
-	case BMunPOINT:
-	    io->ioTwipsWide= (int)( ( 20* abi->abiBitmap.bdPixelsWide )/
-					    abi->abiBitmap.bdXResolution );
-	    io->ioTwipsHigh= (int)( ( 20* abi->abiBitmap.bdPixelsHigh )/
-					    abi->abiBitmap.bdYResolution );
-	    break;
-	default:
-	    LDEB(abi->abiBitmap.bdUnit);
-	    io->ioTwipsWide= 20* abi->abiBitmap.bdPixelsWide;
-	    io->ioTwipsHigh= 20* abi->abiBitmap.bdPixelsHigh;
-	    break;
-	}
+    bmImageSizeTwips( &(io->ioTwipsWide), &(io->ioTwipsHigh),
+							&(abi->abiBitmap) );
 
     tedScaleObjectToParagraph( ed, io );
 
@@ -541,26 +623,22 @@ int tedObjectInsertBitmap(	EditDocument *		ed,
 				( io->ioScaleY* io->ioTwipsHigh )/ 100 );
     io->ioPrivate= abi;
 
-    io->ioXExtent= (int) ( ( 1000.0* io->ioTwipsWide )/ ( 20* POINTS_PER_CM ) );
-    io->ioYExtent= (int) ( ( 1000.0* io->ioTwipsHigh )/ ( 20* POINTS_PER_CM ) );
+    io->io_xWinExt= (int) ( ( 100000.0* io->ioTwipsWide )/
+						    ( 20* POINTS_PER_M ) );
+    io->io_yWinExt= (int) ( ( 100000.0* io->ioTwipsHigh )/
+						    ( 20* POINTS_PER_M ) );
 
-    if  ( appImgMakePixmap( display, ea->eaScreen, win, gc, &io->ioPixmap,
+    if  ( appImgMakePixmap( add, &pixmap,
 				    io->ioPixelsWide, io->ioPixelsHigh,
 				    &ed->edColors, abi ) )
-	{ LDEB(1); return -1; }
-
-    tedAppReplaceSelection( ed, (unsigned char *)"", 0 );
-
-    if  ( tedInsertNewObject( ed->edDocumentWidget, io, ed ) )
-	{ LDEB(1); return -1;	}
+	{ LDEB(1); return (InsertedObject *)0; }
+    io->ioPixmap= pixmap;
 
     /*  9
     io->ioBliptag= appGetTimestamp();
     */
 
-    appDocumentChanged( ed->edApplication, ed, 1 );
-
-    return 0;
+    return io;
     }
 
 /************************************************************************/
@@ -569,16 +647,12 @@ int tedObjectInsertBitmap(	EditDocument *		ed,
 /*									*/
 /************************************************************************/
 
-static int tedObjectInsertMetafile(	EditDocument *		ed,
+static InsertedObject * tedMakeMetafileObject(
+					EditDocument *		ed,
+					BufferItem *		bi,
 					const char *		filename )
     {
-    EditApplication *		ea= ed->edApplication;
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    BufferDocument *		bd= td->tdDocument;
-
-    Widget			w= ed->edDocumentWidget;
-    Window			win= XtWindow( w );
-    GC				gc= ed->edGc;
+    AppDrawingData *		add= &(ed->edDrawingData);
 
     InsertedObject *		io;
 
@@ -589,25 +663,25 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
 
     int				objectNumber;
 
-    double			xfac= ea->eaMagnifiedPixelsPerTwip;
+    double			xfac= add->addMagnifiedPixelsPerTwip;
 
     sisIn= sioInStdioOpen( filename );
     if  ( ! sisIn )
-	{ SXDEB(filename,sisIn); return -1;	}
+	{ SXDEB(filename,sisIn); return (InsertedObject *)0;	}
 
-    io= docClaimObject( &objectNumber, td->tdSelection.bsBegin.bpBi );
+    io= docClaimObject( &objectNumber, bi );
     if  ( ! io )
-	{ XDEB(io); return -1;	}
+	{ XDEB(io); return (InsertedObject *)0;	}
 
-    sioMemoryInit( &mb );
+    utilInitMemoryBuffer( &mb );
 
     sosMem= sioOutMemoryOpen( &mb );
     if  ( ! sosMem )
-	{ XDEB(sosMem); return -1;	}
+	{ XDEB(sosMem); return (InsertedObject *)0;	}
 
     sosMeta= sioOutHexOpen( sosMem );
     if  ( ! sosMeta )
-	{ XDEB(sosMeta); return -1;	}
+	{ XDEB(sosMeta); return (InsertedObject *)0;	}
 
     {
     unsigned long		key;
@@ -623,29 +697,46 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
 
     key= sioEndianGetLeUint32( sisIn );
 
-    if  ( key != 0x9ac6cdd7 )
-	{ XDEB(key); return -1;	}
+    if  ( key == 0x9ac6cdd7 )
+	{
+	handle= sioEndianGetLeUint16( sisIn );
+	left= sioEndianGetLeInt16( sisIn );
+	top= sioEndianGetLeInt16( sisIn );
+	right= sioEndianGetLeInt16( sisIn );
+	bottom= sioEndianGetLeInt16( sisIn );
+	inch= sioEndianGetLeUint16( sisIn );
 
-    handle= sioEndianGetLeUint16( sisIn );
-    left= sioEndianGetLeInt16( sisIn );
-    top= sioEndianGetLeInt16( sisIn );
-    right= sioEndianGetLeInt16( sisIn );
-    bottom= sioEndianGetLeInt16( sisIn );
-    inch= sioEndianGetLeUint16( sisIn );
+	reserved= sioEndianGetLeUint32( sisIn );
+	checksum= sioEndianGetLeUint16( sisIn );
 
-    reserved= sioEndianGetLeUint32( sisIn );
-    checksum= sioEndianGetLeUint16( sisIn );
+	if  ( right > left )
+	    { io->ioTwipsWide= ( 20* 72* ( right- left ) )/ inch;	}
+	else{ io->ioTwipsWide= ( 20* 72* ( left- right ) )/ inch;	}
 
-    if  ( right > left )
-	{ io->ioTwipsWide= ( 20* 72* ( right- left ) )/ inch;	}
-    else{ io->ioTwipsWide= ( 20* 72* ( left- right ) )/ inch;	}
+	if  ( bottom > top )
+	    { io->ioTwipsHigh= ( 20* 72* ( bottom- top ) )/ inch;	}
+	else{ io->ioTwipsHigh= ( 20* 72* ( top- bottom ) )/ inch;	}
 
-    if  ( bottom > top )
-	{ io->ioTwipsHigh= ( 20* 72* ( bottom- top ) )/ inch;	}
-    else{ io->ioTwipsHigh= ( 20* 72* ( top- bottom ) )/ inch;	}
-
-    io->ioXExtent= right- left;
-    io->ioYExtent= top- bottom;
+	io->io_xWinExt= right- left;
+	io->io_yWinExt= top- bottom;
+	}
+    else{
+	if  ( 0 && key == 0x90001 )
+	    {
+	    io->ioTwipsWide= 20* 72* 4;		/*  10 cm	*/
+	    io->ioTwipsHigh= 20* 72* 4;		/*  10 cm	*/
+	    io->io_xWinExt= 72* 4;
+	    io->io_yWinExt= 72* 4;
+	    sioEndianPutLeUint32( key, sosMeta );
+	    }
+	else{
+	    XDEB(key);
+	    sioOutClose( sosMeta );
+	    sioOutClose( sosMem );
+	    sioInClose( sisIn );
+	    return (InsertedObject *)0;
+	    }
+	}
     }
 
     for (;;)
@@ -658,16 +749,16 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
 	    { break;	}
 
 	if  ( sioOutWriteBytes( sosMeta, buf, done ) != done )
-	    { LDEB(done); return -1;	}
+	    { LDEB(done); return (InsertedObject *)0;	}
 	}
 
     sioOutClose( sosMeta );
     sioOutClose( sosMem );
     sioInClose( sisIn );
 
-    io->ioObjectData.odBytes= mb.mbBytes;
-    io->ioObjectData.odSize= mb.mbCapacity;
+    io->ioObjectData= mb;
     io->ioKind= DOCokPICTWMETAFILE;
+    io->ioMapMode= 8;
 
     tedScaleObjectToParagraph( ed, io );
 
@@ -676,36 +767,37 @@ static int tedObjectInsertMetafile(	EditDocument *		ed,
     io->ioPixelsHigh= TWIPStoPIXELS( xfac,
 				( io->ioScaleY* io->ioTwipsHigh )/ 100 );
 
-    io->ioXExtent= (int) ( ( 1000.0* io->ioTwipsWide )/ ( 20* POINTS_PER_CM ) );
-    io->ioYExtent= (int) ( ( 1000.0* io->ioTwipsHigh )/ ( 20* POINTS_PER_CM ) );
-
-    tedAppReplaceSelection( ed, (unsigned char *)"", 0 );
-
-    if  ( tedInsertNewObject( ed->edDocumentWidget, io, ed ) )
-	{ LDEB(1); return -1;	}
+    io->io_xWinExt= (int) ( ( 100000.0* io->ioTwipsWide )/
+						    ( 20* POINTS_PER_M ) );
+    io->io_yWinExt= (int) ( ( 100000.0* io->ioTwipsHigh )/
+						    ( 20* POINTS_PER_M ) );
 
     /*  9
     io->ioBliptag= appGetTimestamp();
     */
 
-    appDocumentChanged( ed->edApplication, ed, 1 );
-
-    if  ( tedOpenObject( io, &ed->edColors, &(bd->bdFontList),
-					    &(ed->edDrawingData), win, gc ) )
+    if  ( tedOpenObject( io, &ed->edColors, &(ed->edDrawingData) ) )
 	{ LDEB(1);	}
 
-    return 0;
+    return io;
     }
 
 static int tedObjectOpenPicture(	void *		voided,
-					Widget		relative,
-					Widget		option,
+					APP_WIDGET	relative,
+					APP_WIDGET	option,
 					const char *	filename )
     {
     EditDocument *		ed= (EditDocument *)voided;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
     const char *		slash= strrchr( filename, '/' );
     const char *		dot;
 
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+
+    if  ( tedGetSelection( &ds, &sg, td ) )
+	{ LDEB(1); return -1;	}
+
     if  ( slash )
 	{ dot= strrchr( slash+ 1, '.' );	}
     else{ dot= strrchr( filename, '.' );	}
@@ -724,22 +816,28 @@ static int tedObjectOpenPicture(	void *		voided,
 						&abi->abiFormat, &factor ) )
 	    { SDEB(filename); return -1;	}
 
-	if  ( tedObjectInsertBitmap( ed, abi ) )
+	if  ( tedReplaceSelectionWithBitmapImage( ed, abi ) )
 	    { SDEB(filename); return -1; }
 
 	return 0;
 	}
     else{
-	if  ( tedObjectInsertMetafile( ed, filename ) )
-	    { SDEB(filename); return -1; }
+	InsertedObject *	io;
+
+	io= tedMakeMetafileObject( ed, ds.dsBegin.dpBi, filename );
+	if  ( ! io )
+	    { SXDEB(filename,io); return -1; }
+
+	if  ( tedReplaceSelectionWithObject( ed, io ) )
+	    { SDEB(filename); return -1;	}
 
 	return 0;
 	}
     }
 
-void tedDocInsertPicture(	Widget		option,
-				XtPointer	voided,
-				XtPointer	voidpbcs	 )
+void tedDocInsertPicture(	APP_WIDGET	option,
+				void *		voided,
+				void *		voidpbcs )
     {
     EditDocument *		ed= (EditDocument *)voided;
     EditApplication *		ea= ed->edApplication;
@@ -748,8 +846,6 @@ void tedDocInsertPicture(	Widget		option,
     static AppFileExtension *	openExtensions;
     static int			openExtensionCount;
 
-    static Widget		openPictureChooser;
-
     if  ( ! td->tdCanReplaceSelection )
 	{ LDEB(td->tdCanReplaceSelection); return;	}
 
@@ -778,7 +874,7 @@ void tedDocInsertPicture(	Widget		option,
 	openExtensionCount++;
 	}
 
-    appRunOpenChooser( &openPictureChooser, option, ed->edTopWidget,
+    appRunOpenChooser( option, ed->edToplevel.atTopWidget,
 			openExtensionCount, openExtensions, (char *)0,
 			voided, tedObjectOpenPicture, ea );
 
