@@ -4,44 +4,36 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
-#   include	<debugon.h>
+#   include	<stddef.h>
+#   include	<appDebugon.h>
 
-#   include	<X11/Xatom.h>
-#   include	<X11/IntrinsicP.h>
-#   include	<Xm/Text.h>
-#   include	<Xm/MainW.h>
-#   include	<Xm/Form.h>
-#   include	<Xm/Label.h>
-#   include	<Xm/Frame.h>
-#   include	<Xm/List.h>
-#   include	<Xm/ScrolledW.h>
-#   include	<Xm/Label.h>
-#   include	<Xm/RowColumn.h>
-#   include	<Xm/DialogS.h>
-#   include	<Xm/PanedW.h>
+#   include	<reg.h>
 
-#   include	<Xm/MwmUtil.h>
-
-#   include	<Xm/Protocols.h>
-
-
-#   include	"appFindTool.h"
 #   include	"appFrame.h"
-#   include	"appUtil.h"
+
+#   ifdef USE_MOTIF
+#   include	<X11/keysym.h>
+#   endif
 
 /************************************************************************/
+/*									*/
 /*  Represents a find tool.						*/
+/*									*/
 /************************************************************************/
+
 #   define	FILEL	400
 
 typedef struct AppFindToolResources
     {
     char *	aftrFindTitle;
     char *	aftrFindNext;
+
+    char *	aftrUseRegex;
+
     char *	aftrFindPrevious;
 
     char *	aftrReplaceTitle;
@@ -51,17 +43,19 @@ typedef struct AppFindToolResources
 
 typedef struct AppFindTool
     {
-    Widget			aftTopWidget;
-    Widget			aftMainWidget;
+    APP_WIDGET			aftTopWidget;
+    APP_WIDGET			aftMainWidget;
 
-    Widget			aftPatternText;
-    Widget			aftReplaceText;
-    Widget			aftReplaceFrame;
+    APP_WIDGET			aftPatternText;
+    APP_WIDGET			aftReplaceText;
+    APP_WIDGET			aftReplaceFrame;
 
-    Widget			aftReplaceButton;
-    Widget			aftReplaceNextButton;
-    Widget			aftFindNextButton;
-    Widget			aftFindPrevButton;
+    APP_WIDGET			aftReplaceButton;
+    APP_WIDGET			aftReplaceNextButton;
+    APP_WIDGET			aftFindNextButton;
+    APP_WIDGET			aftFindPrevButton;
+
+    APP_WIDGET			aftRegexToggle;
 
     AppToolDestroy		aftDestroy;
     FindToolFind		aftFindNext;
@@ -72,6 +66,7 @@ typedef struct AppFindTool
 
     int				aftVisible;
     int				aftLastFindResult;
+    int				aftUseRegex;
     } AppFindTool;
 
 /************************************************************************/
@@ -82,34 +77,34 @@ typedef struct AppFindTool
 
 static void appFindToolReflectPatternText(	AppFindTool *	aft )
     {
-    Boolean	sensitive;
+    int		sensitive;
     char *	pattern;
 
-    pattern= XmTextGetString( aft->aftPatternText );
+    pattern= appGetStringFromTextWidget( aft->aftPatternText );
 
     sensitive= pattern[0] != '\0';
 
-    XtFree( pattern );
+    appFreeStringFromTextWidget( pattern );
 
-    XtSetSensitive( aft->aftFindNextButton, sensitive );
-    XtSetSensitive( aft->aftFindPrevButton, sensitive );
+    appGuiEnableWidget( aft->aftFindNextButton, sensitive );
+    appGuiEnableWidget( aft->aftFindPrevButton, sensitive );
 
     return;
     }
 
 static void appFindToolReflectReplaceText(	AppFindTool *	aft )
     {
-    Boolean	sensitive;
+    int		sensitive;
     char *	replacement;
 
-    replacement= XmTextGetString( aft->aftReplaceText );
+    replacement= appGetStringFromTextWidget( aft->aftReplaceText );
 
     sensitive= replacement[0] != '\0' && aft->aftLastFindResult == 0;
 
-    XtFree( replacement );
+    appFreeStringFromTextWidget( replacement );
 
-    XtSetSensitive( aft->aftReplaceButton, sensitive );
-    XtSetSensitive( aft->aftReplaceNextButton, sensitive );
+    appGuiEnableWidget( aft->aftReplaceButton, sensitive );
+    appGuiEnableWidget( aft->aftReplaceNextButton, sensitive );
 
     return;
     }
@@ -117,31 +112,33 @@ static void appFindToolReflectReplaceText(	AppFindTool *	aft )
 static void appFindToolSetFindResult(	AppFindTool *	aft,
 					int		result )
     {
-    Boolean	sensitive;
+    int		sensitive;
 
     aft->aftLastFindResult= result;
 
     if  ( result )
-	{ sensitive= False;	}
+	{ sensitive= 0;	}
     else{
 	char *	replacement;
 
-	replacement= XmTextGetString( aft->aftReplaceText );
+	appGuiEnableWidget( aft->aftReplaceFrame, 1 );
+
+	replacement= appGetStringFromTextWidget( aft->aftReplaceText );
 
 	sensitive= replacement[0] != '\0';
 
-	XtFree( replacement );
+	appFreeStringFromTextWidget( replacement );
 	}
 
-    XtSetSensitive( aft->aftReplaceButton, sensitive );
-    XtSetSensitive( aft->aftReplaceNextButton, sensitive );
+    appGuiEnableWidget( aft->aftReplaceButton, sensitive );
+    appGuiEnableWidget( aft->aftReplaceNextButton, sensitive );
 
     return;
     }
 
-static void appFindToolPatternChanged(	Widget		w,
-					XtPointer	voidaft,
-					XtPointer	call_data )
+static void appFindToolPatternChanged(	APP_WIDGET	w,
+					void *		voidaft,
+					void *		call_data )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
@@ -150,9 +147,9 @@ static void appFindToolPatternChanged(	Widget		w,
     return;
     }
 
-static void appFindToolReplacementChanged(	Widget		w,
-						XtPointer	voidaft,
-						XtPointer	call_data )
+static void appFindToolReplacementChanged(	APP_WIDGET	w,
+						void *		voidaft,
+						void *		call_data )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
@@ -173,20 +170,34 @@ static int appFindToolGetProgram(	regProg **	pProg,
     char *		pattern;
     regProg *		prog;
 
-    pattern= XmTextGetString( aft->aftPatternText );
+    pattern= appGetStringFromTextWidget( aft->aftPatternText );
 
-    prog= regCompile( (const unsigned char *)pattern );
-    if  ( ! prog )
-	{ SXDEB(pattern,prog); XtFree( pattern ); return -1; }
+    if  ( aft->aftUseRegex )
+	{
+	prog= regCompile( (const unsigned char *)pattern );
+	if  ( ! prog )
+	    {
+	    SXDEB(pattern,prog);
+	    appFreeStringFromTextWidget( pattern ); return -1;
+	    }
+	}
+    else{
+	prog= regCompileEscaped( (const unsigned char *)pattern );
+	if  ( ! prog )
+	    {
+	    SXDEB(pattern,prog);
+	    appFreeStringFromTextWidget( pattern ); return -1;
+	    }
+	}
 
-    XtFree( pattern );
+    appFreeStringFromTextWidget( pattern );
 
     *pProg= prog; return 0;
     }
 
-static void appFindToolFindNext(	Widget		w,
-					XtPointer	voidaft,
-					XtPointer	voidpbcs	 )
+static void appFindToolFindNext(	APP_WIDGET	w,
+					void *		voidaft,
+					void *		voidpbcs )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
     regProg *		prog;
@@ -209,9 +220,9 @@ static void appFindToolFindNext(	Widget		w,
     return;
     }
 
-static void appFindToolFindPrev(	Widget		w,
-					XtPointer	voidaft,
-					XtPointer	voidpbcs	 )
+static void appFindToolFindPrev(	APP_WIDGET	w,
+					void *		voidaft,
+					void *		voidpbcs )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
     regProg *		prog;
@@ -240,9 +251,9 @@ static void appFindToolFindPrev(	Widget		w,
 /*									*/
 /************************************************************************/
 
-static void appFindToolReplaceSelection(	Widget		w,
-						XtPointer	voidaft,
-						XtPointer	voidpbcs  )
+static void appFindToolReplaceSelection(	APP_WIDGET	w,
+						void *		voidaft,
+						void *		voidpbcs )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
     char *		replacement;
@@ -250,20 +261,20 @@ static void appFindToolReplaceSelection(	Widget		w,
     if  ( ! aft->aftReplace )
 	{ XDEB(aft->aftReplace); return;	}
 
-    replacement= XmTextGetString( aft->aftReplaceText );
+    replacement= appGetStringFromTextWidget( aft->aftReplaceText );
 
     (*aft->aftReplace)( aft->aftTarget, (const unsigned char *)replacement );
 
-    XtFree( replacement );
+    appFreeStringFromTextWidget( replacement );
 
-    XtSetSensitive( aft->aftReplaceButton, False );
+    appGuiEnableWidget( aft->aftReplaceButton, 0 );
 
     return;
     }
 
-static void appFindToolReplaceNext(	Widget		w,
-					XtPointer	voidaft,
-					XtPointer	voidpbcs  )
+static void appFindToolReplaceNext(	APP_WIDGET	w,
+					void *		voidaft,
+					void *		voidpbcs )
     {
     appFindToolReplaceSelection( w, voidaft, voidpbcs );
 
@@ -272,49 +283,58 @@ static void appFindToolReplaceNext(	Widget		w,
     return;
     }
 
+static void appFindRegexToggled(	APP_WIDGET	w,
+					void *		voidaft,
+					void *		voidtbcs )
+    {
+    AppFindTool *	aft= (AppFindTool *)voidaft;
+    int			set;
+
+    set= appGuiGetToggleStateFromCallback( w, voidtbcs );
+
+    aft->aftUseRegex= ( set != 0 );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Make a row of buttons for the Find Tool.				*/
 /*									*/
 /************************************************************************/
 
-static void appFindToolMakeButtonRow(	Widget *	pLeftButton,
-					Widget *	pRightButton,
-					char *		leftLabel,
-					char *		rightLabel,
-					XtCallbackProc	leftCallback,
-					XtCallbackProc	rightCallback,
-					AppFindTool *	aft,
-					Widget		bboard,
-					Widget		parent )
+static void appFindToolMakeButtonRow(	APP_WIDGET *		pLeftButton,
+					APP_WIDGET *		pRightButton,
+					char *			leftLabel,
+					char *			rightLabel,
+					APP_BUTTON_CALLBACK	leftCallback,
+					APP_BUTTON_CALLBACK	rightCallback,
+					AppFindTool *		aft,
+					APP_WIDGET		parent )
 
     {
-    Widget		row;
+    APP_WIDGET		row;
 
-    Widget		leftButton;
-    Widget		rightButton;
+    APP_WIDGET		leftButton;
+    APP_WIDGET		rightButton;
 
-    row= appMakeButtonRow( parent, 2 );
+    const int		heightResizable= 0;
+    const int		showAsDefault= 0;
 
-    /***************/
+    row= appMakeRowInColumn( parent, 2, heightResizable );
 
-    leftButton= appMakeRowButton( row, leftLabel, leftCallback, (void *)aft,
-								0, False );
-    rightButton= appMakeRowButton( row, rightLabel, rightCallback, (void *)aft,
-								1, False );
+    appMakeButtonInRow( &(leftButton), row, leftLabel,
+			    leftCallback, (void *)aft, 0, showAsDefault );
+    appMakeButtonInRow( &(rightButton), row, rightLabel,
+			    rightCallback, (void *)aft, 1, showAsDefault );
 
-    /***************/
-
-    XtVaSetValues( bboard,
+#   ifdef USE_MOTIF
+    XtVaSetValues( XtParent( parent ),
 			XmNdefaultButton,	leftButton,
 			NULL );
+#   endif
 
 
-    XtManageChild( leftButton );
-    XtManageChild( rightButton );
-
-    XtManageChild( row );
-
     *pLeftButton= leftButton, *pRightButton= rightButton; return;
     }
 
@@ -324,26 +344,36 @@ static void appFindToolMakeButtonRow(	Widget *	pLeftButton,
 /*									*/
 /************************************************************************/
 
-static Widget appFindMakeFindFrame(	Widget			parent,
-					Display *		display,
+static APP_WIDGET appFindMakeFindFrame(	APP_WIDGET		parent,
 					AppFindToolResources *	aftr,
 					AppFindTool *		aft 	)
     {
-    Widget		frame;
-    Widget		bboard;
-    Widget		paned;
+    APP_WIDGET		frame;
+    APP_WIDGET		paned;
+    APP_WIDGET		row;
 
-    appMakeVerticalFrame( &frame, &bboard, &paned, parent,
-							aftr->aftrFindTitle );
+    const int		textEnabled= 1;
+    const int		heightResizable= 0;
 
-    appMakeColumnText( &(aft->aftPatternText), paned, 0, True );
+    appMakeColumnFrameInColumn( &frame, &paned, parent, aftr->aftrFindTitle );
 
-    XtAddCallback( aft->aftPatternText, XmNvalueChangedCallback,
+    appMakeTextInColumn( &(aft->aftPatternText), paned, 0, textEnabled );
+
+    appGuiSetTypingCallbackForText( aft->aftPatternText,
 				    appFindToolPatternChanged, (void *)aft );
 
-    XtVaSetValues( bboard,
+#   ifdef USE_MOTIF
+    XtVaSetValues( XtParent( paned ),
 			XmNinitialFocus,	aft->aftPatternText,
 			NULL );
+#   endif
+
+    /***************/
+
+    row= appMakeRowInColumn( paned, 1, heightResizable );
+    aft->aftRegexToggle=
+		    appMakeToggleInRow( row, aftr->aftrUseRegex,
+					appFindRegexToggled, (void *)aft, 0 );
 
     /***************/
 
@@ -351,42 +381,42 @@ static Widget appFindMakeFindFrame(	Widget			parent,
 		&(aft->aftFindNextButton), &(aft->aftFindPrevButton),
 		aftr->aftrFindNext, aftr->aftrFindPrevious,
 		appFindToolFindNext, appFindToolFindPrev,
-		aft, bboard, paned );
+		aft, paned );
 
     /***************/
 
-    XtManageChild( aft->aftPatternText );
-    XtManageChild( paned );
-    XtManageChild( bboard );
-    XtManageChild( frame );
-
     return frame;
     }
 
 /************************************************************************/
+/*									*/
 /*  Make the replace part of the find tool.				*/
+/*									*/
 /************************************************************************/
-static Widget appFindMakeReplaceFrame(	Widget			parent,
-					Display *		display,
+
+static APP_WIDGET appFindMakeReplaceFrame( APP_WIDGET		parent,
 					AppFindToolResources *	aftr,
 					AppFindTool *		aft )
     {
-    Widget		frame;
-    Widget		bboard;
-    Widget		paned;
+    APP_WIDGET		frame;
+    APP_WIDGET		paned;
 
-    appMakeVerticalFrame( &frame, &bboard, &paned,
+    const int		textEnabled= 1;
+
+    appMakeColumnFrameInColumn( &frame, &paned,
 					    parent, aftr->aftrReplaceTitle );
     aft->aftReplaceFrame= frame;
 
-    appMakeColumnText( &(aft->aftReplaceText), paned, 0, True );
+    appMakeTextInColumn( &(aft->aftReplaceText), paned, 0, textEnabled );
 
-    XtAddCallback( aft->aftReplaceText, XmNvalueChangedCallback,
+    appGuiSetTypingCallbackForText( aft->aftReplaceText,
 				appFindToolReplacementChanged, (void *)aft );
 
-    XtVaSetValues( bboard,
+#   ifdef USE_MOTIF
+    XtVaSetValues( XtParent( paned ),
 			XmNinitialFocus,	aft->aftReplaceText,
 			NULL );
+#   endif
 
     /***************/
 
@@ -394,41 +424,63 @@ static Widget appFindMakeReplaceFrame(	Widget			parent,
 		&(aft->aftReplaceButton), &(aft->aftReplaceNextButton),
 		aftr->aftrReplaceFound, aftr->aftrReplaceNext,
 		appFindToolReplaceSelection, appFindToolReplaceNext,
-		aft, bboard, paned );
+		aft, paned );
 
     /***************/
 
-    XtManageChild( aft->aftReplaceText );
-    XtManageChild( paned );
-    XtManageChild( bboard );
-    XtManageChild( frame );
-
     return frame;
     }
 
 /************************************************************************/
 /*  A find tool must be destroyed.					*/
 /************************************************************************/
-static void appCloseFindTool(		Widget		w,
-					XtPointer	voidaft,
-					XtPointer	voidlcs	 )
-    {
-    AppFindTool *	aft= (AppFindTool *)voidaft;
 
+static void appDestroyFindTool(	AppFindTool *	aft )
+    {
     if  ( aft->aftDestroy )
 	{ (*aft->aftDestroy)( aft->aftTarget );	}
 
-    XtDestroyWidget( w );
+    appDestroyShellWidget( aft->aftTopWidget );
 
     free( aft );
+    }
+
+static APP_CLOSE_CALLBACK( appCloseFindTool, w, voidaft )
+    {
+    AppFindTool *	aft= (AppFindTool *)voidaft;
+
+    appDestroyFindTool( aft );
 
     return;
     }
 
+#   ifdef USE_MOTIF
+static void appFindToolCancel(		APP_WIDGET		w,
+					void *			voidaft,
+					XEvent *		event,
+					Boolean *		pRefused )
+    {
+    if  ( event->type == KeyRelease )
+	{
+	XKeyEvent *		kevent= &(event->xkey);
+	AppFindTool *		aft= (AppFindTool *)voidaft;
+
+	if  ( XKeysymToKeycode( XtDisplay(w), XK_Escape ) == kevent->keycode )
+	    { appDestroyFindTool( aft ); *pRefused= 0; return; }
+	}
+
+    *pRefused= 1; return;
+    }
+#   endif
+
 /************************************************************************/
+/*									*/
 /*  make a find tool.							*/
+/*									*/
 /************************************************************************/
-static void appFindToolConfigure(	Widget			w,
+
+#   ifdef USE_MOTIF
+static void appFindToolConfigure(	APP_WIDGET		w,
 					void *			voidaft,
 					XEvent *		event,
 					Boolean *		pRefused )
@@ -449,8 +501,10 @@ static void appFindToolConfigure(	Widget			w,
     *pRefused= 1;
     return;
     }
+#   endif
 
-static void appFindToolVisible(		Widget			w,
+#   ifdef USE_MOTIF
+static void appFindToolVisible(		APP_WIDGET		w,
 					void *			voidaft,
 					XEvent *		event,
 					Boolean *		pRefused )
@@ -470,78 +524,67 @@ static void appFindToolVisible(		Widget			w,
     *pRefused= 1;
     return;
     }
+#   endif
 
-# define xx(x)  x,x
-
-static XtResource APP_FindToolResourceTable[]=
+static AppConfigurableResource APP_FindToolResourceTable[]=
     {
-	{ xx("findToolFindTitle"), XtRString, sizeof(char *),
+	APP_RESOURCE( "findToolFindTitle",
 		    offsetof(AppFindToolResources,aftrFindTitle),
-		    XtRString, "Find" },
-	{ xx("findToolFindNext"), XtRString, sizeof(char *),
+		    "Find" ),
+
+	APP_RESOURCE( "findToolUseRegex",
+		    offsetof(AppFindToolResources,aftrUseRegex),
+		    "Find Regular Expression" ),
+
+	APP_RESOURCE( "findToolFindNext",
 		    offsetof(AppFindToolResources,aftrFindNext),
-		    XtRString, "Find" },
-	{ xx("findToolFindPrevious"), XtRString, sizeof(char *),
+		    "Find" ),
+	APP_RESOURCE( "findToolFindPrevious",
 		    offsetof(AppFindToolResources,aftrFindPrevious),
-		    XtRString, "Previous" },
-	{ xx("findToolReplaceTitle"), XtRString, sizeof(char *),
+		    "Previous" ),
+
+	APP_RESOURCE( "findToolReplaceTitle",
 		    offsetof(AppFindToolResources,aftrReplaceTitle),
-		    XtRString, "Replace" },
-	{ xx("findToolReplaceFound"), XtRString, sizeof(char *),
+		    "Replace" ),
+
+	APP_RESOURCE( "findToolReplaceFound",
 		    offsetof(AppFindToolResources,aftrReplaceFound),
-		    XtRString, "Replace" },
-	{ xx("findToolReplaceNext"), XtRString, sizeof(char *),
+		    "Replace" ),
+	APP_RESOURCE( "findToolReplaceNext",
 		    offsetof(AppFindToolResources,aftrReplaceNext),
-		    XtRString, "Replace, Next" },
+		    "Replace, Next" ),
     };
 
-void * appMakeFindTool(		Widget			findOption,
-				Widget			topWidget,
-				XtAppContext		appContext,
-				const char *		applicationName,
+void * appMakeFindTool(		APP_WIDGET		findOption,
+				EditApplication *	ea,
 				const char *		widgetName,
-				Pixmap			iconPixmap,
-				Atom			closeAtom,
+				APP_BITMAP_IMAGE	iconPixmap,
+				APP_BITMAP_MASK		iconMask,
 				AppToolDestroy		destroy,
 				FindToolFind		findNext,
 				FindToolFind		findPrev,
 				FindToolReplace		replace,
-				void *			target		)
+				void *			target )
     {
-    Display *		display= XtDisplay( findOption );
     AppFindTool *	aft;
     
-    Arg			al[20];
-    int			ac= 0;
+    APP_WIDGET		findFrame;
+    APP_WIDGET		replaceFrame;
 
-    Widget		findFrame;
-    Widget		replaceFrame;
+    const int		userResizable= 1;
     
-    MwmHints		hints;
-
     static AppFindToolResources	aftr;
     static int			gotResources;
 
     if  ( ! gotResources )
 	{
-	XtGetApplicationResources( topWidget, (void *)&aftr,
-		APP_FindToolResourceTable, XtNumber(APP_FindToolResourceTable),
-		NULL, 0 );
+	appGuiGetResourceValues( ea, (void *)&aftr, APP_FindToolResourceTable,
+					sizeof(APP_FindToolResourceTable)/
+					sizeof(AppConfigurableResource) );
 
 	gotResources= 1;
 	}
 
-    hints.flags= MWM_HINTS_FUNCTIONS|MWM_HINTS_DECORATIONS;
-    hints.functions=	MWM_FUNC_MOVE		|
-			MWM_FUNC_RESIZE		|
-			MWM_FUNC_MINIMIZE	|
-			MWM_FUNC_CLOSE		;
-    hints.decorations=	MWM_DECOR_BORDER	|
-			MWM_DECOR_RESIZEH	|
-			MWM_DECOR_TITLE		|
-			MWM_DECOR_MENU		|
-			MWM_DECOR_MINIMIZE	;
-
     aft= (AppFindTool *)malloc( sizeof(AppFindTool) );
     if  ( ! aft )
 	{ XDEB(aft); return (void *)0;	}
@@ -552,58 +595,45 @@ void * appMakeFindTool(		Widget			findOption,
     aft->aftReplace= replace;
     aft->aftTarget= target;
     aft->aftVisible= 0;
+    aft->aftUseRegex= 0;
     
-    ac= 0;
-    XtSetArg( al[ac], XmNuserData,		(void *)aft ); ac++;
-    XtSetArg( al[ac], XmNdeleteResponse,	XmDO_NOTHING ); ac++;
-    XtSetArg( al[ac], XmNmwmDecorations,	hints.decorations ); ac++;
-    XtSetArg( al[ac], XmNmwmFunctions,		hints.functions ); ac++;
-    XtSetArg( al[ac], XmNkeyboardFocusPolicy,	XmEXPLICIT ); ac++;
-    XtSetArg( al[ac], XmNinput,			True ); ac++;
+    appMakeVerticalTool( &(aft->aftTopWidget), &(aft->aftMainWidget), ea,
+			    iconPixmap, iconMask,
+			    widgetName, userResizable, findOption,
+			    appCloseFindTool, (void *)aft );
 
-    /* Removes MWM border:
-    XtSetArg( al[ac], XmNoverrideRedirect,	True ); ac++;
-    */
-
-    if  ( iconPixmap )
-	{ XtSetArg( al[ac], XmNiconPixmap,	iconPixmap ); ac++; }
-
-    aft->aftTopWidget= XtAppCreateShell( applicationName,
-					    widgetName,
-					    applicationShellWidgetClass,
-					    display, al, ac );
-
-    appSetShellTitle( aft->aftTopWidget, findOption, applicationName );
+#   ifdef USE_MOTIF
+    XtVaSetValues(	aft->aftTopWidget,
+				XmNkeyboardFocusPolicy,	XmEXPLICIT,
+				NULL );
 
     XtAddEventHandler( aft->aftTopWidget, StructureNotifyMask, False,
 					appFindToolConfigure, (void *)aft );
     XtAddEventHandler( aft->aftTopWidget, StructureNotifyMask, False,
 					appFindToolVisible, (void *)aft );
+#   endif
 
-    if  ( closeAtom > 0 )
-	{
-	XmAddWMProtocolCallback( aft->aftTopWidget, closeAtom,
-					appCloseFindTool, (XtPointer)aft );
-	}
-
-    ac= 0;
-    XtSetArg( al[ac],	XmNuserData,		(void *)aft ); ac++;
-    XtSetArg( al[ac],	XmNsashWidth,		1 ); ac++;
-    XtSetArg( al[ac],	XmNsashHeight,		1 ); ac++;
-    XtSetArg( al[ac],	XmNseparatorOn,		False ); ac++;
-    XtSetArg( al[ac],	XmNmarginWidth,		5 ); ac++;
-    XtSetArg( al[ac],	XmNmarginHeight,	5 ); ac++;
-    XtSetArg( al[ac],	XmNspacing,		5 ); ac++;
-    aft->aftMainWidget= XmCreatePanedWindow( aft->aftTopWidget, WIDGET_NAME, al, ac );
-
-    findFrame= appFindMakeFindFrame( aft->aftMainWidget, display, &aftr, aft );
+    findFrame= appFindMakeFindFrame( aft->aftMainWidget, &aftr, aft );
 
     replaceFrame= appFindMakeReplaceFrame( aft->aftMainWidget,
-							display, &aftr, aft );
+							&aftr, aft );
 
-    XtManageChild( aft->aftReplaceText );
+#   ifdef USE_MOTIF
+    XtInsertEventHandler( aft->aftPatternText, KeyReleaseMask, False,
+				appFindToolCancel, (void *)aft, XtListHead );
+    XtInsertEventHandler( aft->aftReplaceText, KeyReleaseMask, False,
+				appFindToolCancel, (void *)aft, XtListHead );
 
-    XtManageChild( aft->aftMainWidget );
+    XtInsertEventHandler( aft->aftReplaceButton, KeyReleaseMask, False,
+				appFindToolCancel, (void *)aft, XtListHead );
+    XtInsertEventHandler( aft->aftReplaceNextButton, KeyReleaseMask, False,
+				appFindToolCancel, (void *)aft, XtListHead );
+
+    XtInsertEventHandler( aft->aftFindNextButton, KeyReleaseMask, False,
+				appFindToolCancel, (void *)aft, XtListHead );
+    XtInsertEventHandler( aft->aftFindPrevButton, KeyReleaseMask, False,
+				appFindToolCancel, (void *)aft, XtListHead );
+#   endif
 
     return (void *)aft;
     }
@@ -614,20 +644,20 @@ void * appMakeFindTool(		Widget			findOption,
 /*									*/
 /************************************************************************/
 
-void appShowFindTool(		Widget		reference,
+void appShowFindTool(		APP_WIDGET	relative,
 				void *		voidaft	)
     {
     AppFindTool *		aft= (AppFindTool *)voidaft;
-    Display *			display= XtDisplay( aft->aftTopWidget );
-
-    Dimension			x;
-    Dimension			y;
-    Dimension			width;
-    Dimension			height;
 
     if  ( ! aft->aftVisible )
 	{
-	XtVaGetValues( reference,
+#	ifdef USE_MOTIF
+	Dimension			x;
+	Dimension			y;
+	Dimension			width;
+	Dimension			height;
+
+	XtVaGetValues( relative,
 			    XmNx,		&x,
 			    XmNy,		&y,
 			    XmNwidth,		&width,
@@ -638,22 +668,31 @@ void appShowFindTool(		Widget		reference,
 			    XmNx,		x+ width/10,
 			    XmNy,		y+ height/10,
 			    NULL );
+#	endif
 
-	XtRealizeWidget( aft->aftTopWidget );
-	XtMapWidget( aft->aftTopWidget );
+	appShowShellWidget( aft->aftTopWidget );
 	}
     else{
+	appShowShellWidget( aft->aftTopWidget );
+
+#	ifdef USE_MOTIF
+	{
 	Window		win=  XtWindow( aft->aftTopWidget );
+	Display *	display= XtDisplay( aft->aftTopWidget );
 
-	XMapRaised( display, win );
 	XSetInputFocus( display, win, RevertToNone, CurrentTime );
 	}
+#	endif
+
+	}
 
     appFindToolSetFindResult( aft, -1 );
     appFindToolReflectPatternText( aft );
     appFindToolReflectReplaceText( aft );
 
+#   ifdef USE_MOTIF
     XmProcessTraversal( aft->aftPatternText, XmTRAVERSE_CURRENT );
+#   endif
     }
 
 void appEnableFindTool(		void *	voidaft,
@@ -661,17 +700,17 @@ void appEnableFindTool(		void *	voidaft,
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
-    XtSetSensitive( aft->aftMainWidget, enabled != 0 );
+    appGuiEnableWidget( aft->aftMainWidget, enabled != 0 );
 
     return;
     }
 
-void appFindToolTextReadonly(		void *	voidaft,
-					int	readonly )
+void appFindToolDisableReplace(		void *	voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
-    XtSetSensitive( aft->aftReplaceFrame, readonly == 0 );
+    appGuiEnableWidget( aft->aftReplaceFrame, 0 );
 
     return;
     }
+
