@@ -4,8 +4,6 @@
 /*									*/
 /************************************************************************/
 
-#   include	<X11/Intrinsic.h>
-#   include	<X11/Xatom.h>
 #   include	<ind.h>
 #   include	<appFrame.h>
 #   include	<appTool.h>
@@ -17,11 +15,11 @@ typedef void (*SpellToolCorrect)( void * target, const unsigned char * guess );
 /*  Subroutine declarations.						*/
 /************************************************************************/
 
-extern void * appMakeSpellTool(	Widget			spellOption,
+extern void * appMakeSpellTool(	APP_WIDGET		spellOption,
 				EditApplication *	ea,
 				const char *		widgetName,
-				Pixmap			iconPixmap,
-				Atom			closeAtom,
+				APP_BITMAP_IMAGE	iconPixmap,
+				APP_BITMAP_MASK		iconMask,
 				AppToolDestroy		destroy,
 				SpellToolFindNext	findNext,
 				SpellToolCorrect	correct,
