@@ -1,20 +1,23 @@
 /************************************************************************/
+/*									*/
 /*  A Symbol picker.							*/
+/*									*/
 /************************************************************************/
 
 #   include	<appFrame.h>
-#   include	<appUtil.h>
 
 typedef void (*SymbolPickerInsert)(	void *		target,
 					int		symbol,
 					int		fontFamilyNumber,
-					char *		fontFamilyName );
+					char *		fontFamilyName,
+					int		encoding,
+					int		encodingIsDefault );
 
-extern void * appMakeSymbolPicker(	Widget			symbolOption,
+extern void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
 					EditApplication *	ea,
 					const char *		widgetName,
-					Pixmap			iconPixmap,
-					Atom			closeAtom,
+					APP_BITMAP_IMAGE	iconPixmap,
+					APP_BITMAP_MASK		iconMask,
 					SymbolPickerInsert	insert,
 					AppToolDestroy		destroy,
 					void *			target );
@@ -22,7 +25,8 @@ extern void * appMakeSymbolPicker(	Widget			symbolOption,
 extern void appShowSymbolPicker(	void *	voidasp	);
 
 extern int appAdaptSymbolPickerToFontFamily(	void *	voidasp,
-						int	fontFamilyNumber );
+						int	fontFamilyNumber,
+						int	encoding );
 
 extern void appEnableSymbolPicker(	void *	voidasp,
 					int	enabled );
