@@ -1,3 +1,8 @@
+#   ifndef	APP_UNIT_H
+#   define	APP_UNIT_H
+
+#   include	<time.h>
+
 /************************************************************************/
 /*									*/
 /*  Unit types and conversions between units.				*/
@@ -21,7 +26,21 @@ extern int appUnitTypeInt(	const char *	unitTypeString );
 extern const char * appUnitTypeString(	int	unitTypeInt );
 
 extern double appUnitFromTwips(	int	twips,
-					int	unitTypeInt );
+				int	unitTypeInt );
 
 extern double appUnitToTwips(	double	units,
 				int	unitTypeInt );
+
+extern int appRomanString(	char *		to,
+				int		maxlen,
+				int		n,
+				int		upper );
+
+int appWordFormatDate(			char *			target,
+					int			maxlen,
+					const struct tm *	tm,
+					const char *		wordPicture );
+
+extern void appInvalidateTime(		struct tm *		tm );
+
+#   endif	/*  APP_UNIT_H  */
