@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"appUtilConfig.h"
 
 #   include	<sioSmtp.h>
 
@@ -10,52 +10,57 @@
 #   include	<sys/time.h>
 #   include	<sys/types.h>
 #   include	<sys/socket.h>
-#   include	<netinet/in.h>
-#   include	<unistd.h>
-#   include	<errno.h>
-#   include	<netdb.h>
 #   include	<pwd.h>
+#   include	<unistd.h>
 #   include	<fcntl.h>
+#   include	<errno.h>
 #   include	<sys/utsname.h>
+#   include	<netdb.h>
 
-#   include	<debugon.h>
+#   include	<appDebugon.h>
+
+#   ifdef	NeXT
+#	include	<libc.h>
+
+#	if HAVE_UNAME
+#	else
+#	    define USE_GETHOSTNAME
+#	endif
+#   endif
+
+#   ifdef __VMS
+#	define	HAVE_PW_GECOS	0
+#   else
+#	define	HAVE_PW_GECOS	1
+#   endif
 
 /************************************************************************/
+/*									*/
 /*  Forward declarations						*/
+/*									*/
 /************************************************************************/
 
-static int mmOpenSocket(	const char *		hostName,
-				const char *		portName,
-				void *			through,
-				SIO_SMTP_COMPLAIN	complain );
-
 static int sioSmtpSayRecipients(	int			fd,
 					const char *		recipients,
 					void *			through,
-					SIO_SMTP_COMPLAIN	complain );
+					APP_COMPLAIN		complain );
 
 static int mmWriteHeader(	int			fd,
 				const char *		name,
 				const char *		value,
 				void *			through,
-				SIO_SMTP_COMPLAIN	complain );
-
-static int mmWriteContentType(	int			fd,
-				const char *		type,
-				const char *		subtype,
-				void *			through,
-				SIO_SMTP_COMPLAIN	complain );
+				APP_COMPLAIN		complain );
 
 static int mmWrite(		int			fd,
 				const char *		buf,
 				unsigned int		len,
 				void *			through,
-				SIO_SMTP_COMPLAIN	complain );
+				APP_COMPLAIN		complain );
 
 static int mmWriteCRLF(		int			fd,
 				const char *		buf,
 				void *			through,
-				SIO_SMTP_COMPLAIN	complain );
+				APP_COMPLAIN		complain );
 
 static int sioOutSmtpWriteBytes( void *			voidmc,
 				const unsigned char *	buf,
@@ -69,70 +74,113 @@ static int sioOutSmtpClose(	void *			voidmc );
 static int mmReadCRLF(	int			fd,
 			char *			buf,
 			void *			through,
-			SIO_SMTP_COMPLAIN	complain );
+			APP_COMPLAIN		complain );
 
 typedef struct MailContext
     {
     int			mcFd;
-    int			mcLastWasCRLF;
+    int			mcAtBeginOfLine;
     void *		mcThrough;
-    SIO_SMTP_COMPLAIN	mcComplain;
+    APP_COMPLAIN	mcComplain;
     } MailContext;
 
 static char MMscratch[1024];
 
 /************************************************************************/
-/*  Open a connection to an SMTP server and send the mail headers	*/
+/*									*/
+/*  Abort an Smtp connection.						*/
+/*									*/
 /************************************************************************/
-SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
-					const char *		from,
-					const char *		to,
-					const char *		cc,
-					const char *		bcc,
-					const char *		subject,
-					const char *		type,
-					const char *		subtype,
+
+static void sioOutSmtpAbortMessage(	MailContext *		mc,
 					void *			through,
-					SIO_SMTP_COMPLAIN	complain )
+					APP_COMPLAIN		complain )
+    {
+    if  ( mmWriteCRLF( mc->mcFd, "RSET", through, complain ) )
+	{ LDEB(1); return;	}
+
+    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
+	{ LDEB(1); return;	}
+
+    if  ( mmWriteCRLF( mc->mcFd, "QUIT", through, complain ) )
+	{ LDEB(1); return;	}
+
+    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
+	{ LDEB(1); return;	}
+
+    return;
+    }
+
+
+/************************************************************************/
+/*									*/
+/*  Open a connection to an SMTP server and send the mail headers	*/
+/*									*/
+/************************************************************************/
+
+SimpleOutputStream * sioOutSmtpOpen(	const char *	mailHost,
+					const char *	mailPort,
+					const char *	from,
+					const char *	to,
+					const char *	cc,
+					const char *	bcc,
+					const char *	subject,
+					const char *	typeSlashSubtype,
+					const char *	mimeBoundary,
+					void *		through,
+					APP_COMPLAIN	complain )
     {
     MailContext *		mc= (MailContext *)0;
-    struct utsname		un;
     const char *		past;
     const char *		start;
+    const char *		nodename= (const char *)0;
 
     SimpleOutputStream *	sos;
 
+#   if HAVE_UNAME
+    struct utsname		un;
+#   endif
+
+#   ifdef USE_GETHOSTNAME
+    char			node_scratch[256+1];
+#   endif
+
     if  ( ! from || ! from[0] )
 	{
 	XDEB(from);
-	(*complain)( through, SIO_SMTPeFROM, (char *)0 );
+	(*complain)( through, APP_SYSTEMeFROM, (char *)0 );
 	goto mmError;
 	}
 
     if  ( ! to || ! to[0] )
 	{
 	XDEB(to);
-	(*complain)( through, SIO_SMTPeRCPT, (char *)0 );
+	(*complain)( through, APP_SYSTEMeRCPT, (char *)0 );
 	goto mmError;
 	}
 
     if  ( ! (mc= (MailContext *)malloc( sizeof( MailContext ) )) )
 	{
 	XDEB(mc);
-	(*complain)( through, SIO_SMTPeNOMEM, (char *)0 );
+	(*complain)( through, APP_SYSTEMeNOMEM, (char *)0 );
 	goto mmError;
 	}
 
     mc->mcFd= -1;
-    mc->mcLastWasCRLF= 1;
+    mc->mcAtBeginOfLine= 1;
     mc->mcThrough= through;
     mc->mcComplain= complain;
 
     /********************************************************************/
     /*  Open connection							*/
     /********************************************************************/
-    mc->mcFd= mmOpenSocket( ( relay && relay[0])?relay:"mailhost",
-						"smtp", through, complain );
+
+    if  ( ! mailHost || ! mailHost[0] )
+	{ mailHost= "mailhost";	}
+    if  ( ! mailPort || ! mailPort[0] )
+	{ mailPort= "smtp";	}
+
+    mc->mcFd= appOpenSocket( mailHost, mailPort, through, complain );
     if  ( mc->mcFd < 0 )
 	{ LDEB(mc->mcFd); goto mmError;	}
 
@@ -143,7 +191,7 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
 	if  ( strncmp( MMscratch, "220", 3 ) )
 	    {
 	    SDEB(MMscratch);
-	    (void) (*complain)( through, SIO_SMTPeSMTP, MMscratch );
+	    (void) (*complain)( through, APP_SYSTEMeSMTP, MMscratch );
 	    goto mmError;
 	    }
 	} while( MMscratch[3] != ' ' );
@@ -151,10 +199,24 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
     /********************************************************************/
     /*  Say hello							*/
     /********************************************************************/
+#   if HAVE_UNAME
     if  ( uname( &un ) < 0 )
 	{ LDEB(1); goto mmError;				}
+    nodename= un.nodename;
+#   endif
 
-    sprintf( MMscratch, "HELO %s", un.nodename );
+#   ifdef USE_GETHOSTNAME
+    if  ( gethostname( node_scratch, sizeof(node_scratch)- 1 ) )
+	{ LDEB(1); goto mmError;				}
+
+    node_scratch[sizeof(node_scratch)-1]= '\0';
+    nodename= node_scratch;
+#   endif
+
+    if  ( ! nodename )
+	{ XDEB(nodename); goto mmError;	}
+
+    sprintf( MMscratch, "HELO %s", nodename );
     if  ( mmWriteCRLF( mc->mcFd, MMscratch, through, complain ) )
 	{ LDEB(1); goto mmError;				}
 
@@ -165,7 +227,7 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
 	if  ( strncmp( MMscratch, "250", 3 ) )
 	    {
 	    SDEB(MMscratch);
-	    (void) (*complain)( through, SIO_SMTPeSMTP, MMscratch );
+	    (void) (*complain)( through, APP_SYSTEMeSMTP, MMscratch );
 	    goto mmError;
 	    }
 	} while( MMscratch[3] != ' ' );
@@ -188,7 +250,7 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
 	if  ( strncmp( MMscratch, "250", 3 ) )
 	    {
 	    SDEB(MMscratch);
-	    (void) (*complain)( through, SIO_SMTPeSMTP, MMscratch );
+	    (void) (*complain)( through, APP_SYSTEMeSMTP, MMscratch );
 	    goto mmAbort;
 	    }
 	} while( MMscratch[3] != ' ' );
@@ -223,7 +285,7 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
 	if  ( strncmp( MMscratch, "354", 3 ) )
 	    {
 	    SDEB(MMscratch);
-	    (void) (*complain)( through, SIO_SMTPeSMTP, MMscratch );
+	    (void) (*complain)( through, APP_SYSTEMeSMTP, MMscratch );
 	    goto mmAbort;
 	    }
 	} while( MMscratch[3] != ' ' );
@@ -235,10 +297,38 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
 	  mmWriteHeader( mc->mcFd, "To", to, through, complain )	||
 	  mmWriteHeader( mc->mcFd, "Cc", cc, through, complain )	||
 	  mmWriteHeader( mc->mcFd, "Subject", subject,
-					through, complain )		||
-	  mmWriteContentType( mc->mcFd, type, subtype,
-					through, complain )		||
-	  mmWriteCRLF( mc->mcFd, "\n", through, complain )		)
+					    through, complain )		)
+	{ LDEB(1); goto mmAbort;	}
+	
+    if  ( mimeBoundary )
+	{
+	const char *	name= "Content-Type";
+	int		len= strlen( name );
+	const char *	name2= " boundary=\"";
+
+	if  ( mmWrite( mc->mcFd, name, len, through, complain )		||
+	      mmWrite( mc->mcFd, ": ", 2, through, complain )		||
+	      mmWrite( mc->mcFd, typeSlashSubtype,
+				strlen( typeSlashSubtype ),
+				through, complain )			||
+	      mmWriteCRLF( mc->mcFd, ";", through, complain )		)
+	    { LDEB(1); goto mmAbort;	}
+
+	len= strlen( name2 );
+	if  ( mmWrite( mc->mcFd, name2, len, through, complain )	||
+	      mmWrite( mc->mcFd, mimeBoundary,
+				strlen( mimeBoundary ),
+				through, complain )			||
+	      mmWriteCRLF( mc->mcFd, "\"",  through, complain )		)
+	    { LDEB(1); goto mmAbort;	}
+	}
+    else{
+	if  ( mmWriteHeader( mc->mcFd, "Content-Type", typeSlashSubtype,
+						    through, complain )	)
+	    { LDEB(1); goto mmAbort;	}
+	}
+
+    if  ( mmWriteCRLF( mc->mcFd, "\n", through, complain )		)
 	{ LDEB(1); goto mmAbort;	}
 
     sos= sioOutOpen( (void *)mc, sioOutSmtpWriteBytes,
@@ -249,20 +339,10 @@ SimpleOutputStream * sioOutSmtpOpen(	const char *		relay,
 
     return sos;
 
-mmAbort:
-    if  ( mmWriteCRLF( mc->mcFd, "RSET", through, complain ) )
-	{ LDEB(1); goto mmError;				}
+  mmAbort:
+    sioOutSmtpAbortMessage( mc, through, complain );
 
-    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
-	{ LDEB(1); goto mmError;				}
-
-    if  ( mmWriteCRLF( mc->mcFd, "QUIT", through, complain ) )
-	{ LDEB(1); goto mmError;				}
-
-    if  ( mmReadCRLF( mc->mcFd, MMscratch, through, complain ) )
-	{ LDEB(1); goto mmError;				}
-
-mmError:
+  mmError:
     if  ( mc )
 	{
 	if  ( mc->mcFd != -1 )
@@ -274,15 +354,21 @@ mmError:
     }
 
 /************************************************************************/
+/*									*/
 /*  Refuse to seek.							*/
+/*									*/
 /************************************************************************/
+
 static int sioOutSmtpSeek(	void *	voidmc,
 				long	pos )
     { LDEB(pos); return -1;	}
 
 /************************************************************************/
+/*									*/
 /*  Send the next part of the mail body					*/
+/*									*/
 /************************************************************************/
+
 static int sioOutSmtpWriteBytes(	void *			voidmc,
 					const unsigned char *	buf,
 					int			len )
@@ -313,10 +399,10 @@ static int sioOutSmtpWriteBytes(	void *			voidmc,
 					mc->mcThrough, mc->mcComplain ) )
 		{ LDEB(1); return -1;				}
 	    todo--; past++;
-	    mc->mcLastWasCRLF= 1;
+	    mc->mcAtBeginOfLine= 1;
 	    }
 	else{
-	    mc->mcLastWasCRLF= 0;
+	    mc->mcAtBeginOfLine= 0;
 	    }
 
 	start= past;
@@ -326,14 +412,17 @@ static int sioOutSmtpWriteBytes(	void *			voidmc,
     }
 
 /************************************************************************/
+/*									*/
 /*  Finish sending the mail body and close the connection		*/
+/*									*/
 /************************************************************************/
+
 static int sioOutSmtpClose(	void *		voidmc )
     {
     int			rval= 0;
     MailContext *	mc= (MailContext *)voidmc;
 
-    if  ( ! mc->mcLastWasCRLF						&&
+    if  ( ! mc->mcAtBeginOfLine						&&
 	  mmWriteCRLF( mc->mcFd, "\n", mc->mcThrough, mc->mcComplain )	)
 	{ LDEB(1); rval= -1;	}
 
@@ -347,78 +436,6 @@ static int sioOutSmtpClose(	void *		voidmc )
     return rval;
     }
 
-/************************************************************************/
-/*  Open a socket							*/
-/************************************************************************/
-static int mmOpenSocket(	const char *		hostName,
-				const char *		portName,
-				void *			through,
-				SIO_SMTP_COMPLAIN	complain )
-    {
-    struct sockaddr_in	sina;
-
-    int			port;
-    char		c;
-    int			b0, b1, b2, b3;
-    unsigned char *	mach= (unsigned char *)&(sina.sin_addr.s_addr);
-    int			fd;
-
-    memset( &sina, 0, sizeof(sina) );
-
-    if  ( sscanf( hostName, "%d.%d.%d.%d%c", &b0, &b1, &b2, &b3, &c ) == 4 )
-	{
-	mach[0]= b0; mach[1]= b1; mach[2]= b2; mach[3]= b3;
-	sina.sin_family= AF_INET;
-	}
-    else{
-	struct hostent *	host= gethostbyname( hostName );
-
-	if  ( ! host )
-	    {
-	    SDEB(strerror(errno));
-	    (*complain)( through, SIO_SMTPeHOST, hostName );
-	    return -1;
-	    }
-
-	memcpy( &sina.sin_addr, host->h_addr, host->h_length );
-	sina.sin_family= host->h_addrtype;
-	}
-
-    c= 0;
-    if  ( sscanf( portName, "%d%c", &port, &c ) == 1 && c == 0 )
-	{ sina.sin_port= htons( port ); }
-    else{
-	struct servent *	serv= getservbyname( portName, "tcp" );
-
-	if  ( ! serv )
-	    {
-	    SDEB(strerror(errno));
-	    (*complain)( through, SIO_SMTPeSERV, portName );
-	    return -1;
-	    }
-
-	sina.sin_port= serv->s_port;
-	}
-
-    fd= socket( AF_INET, SOCK_STREAM, 0 );
-    if  ( fd < 0 )
-	{
-	SDEB(strerror(errno));
-	(*complain)( through, SIO_SMTPeSOCK, strerror(errno) );
-	return -1;
-	}
-
-    if  ( connect( fd, (struct sockaddr *)&sina, sizeof(sina) ) )
-	{
-	SDEB(strerror(errno));
-	(*complain)( through, SIO_SMTPeCONN, hostName );
-	close( fd );
-	return -1;
-	}
-
-    return fd;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Write a list of recipients to the SMTP connection			*/
@@ -433,7 +450,7 @@ static int mmOpenSocket(	const char *		hostName,
 static int sioSmtpSayRecipients(	int			fd,
 					const char *		recipients,
 					void *			through,
-					SIO_SMTP_COMPLAIN	complain )
+					APP_COMPLAIN		complain )
     {
     int	ok= 0;
 
@@ -476,7 +493,7 @@ static int sioSmtpSayRecipients(	int			fd,
 
 	    if  ( ! strncmp( MMscratch, "250", 3 ) )
 		{ ok++;						}
-	    else{ (void) (*complain)( through, SIO_SMTPeSMTP, MMscratch ); }
+	    else{ (void) (*complain)( through, APP_SYSTEMeSMTP, MMscratch ); }
 	    } while( MMscratch[3] != ' ' );
 
 	if  ( *comma == ',' )
@@ -491,24 +508,27 @@ static int sioSmtpSayRecipients(	int			fd,
     }	
 
 /************************************************************************/
+/*									*/
 /*  SMTP connection read and write utilities				*/
+/*									*/
 /************************************************************************/
+
 static int mmReadCRLF(	int			fd,
 			char *			buf,
 			void *			through,
-			SIO_SMTP_COMPLAIN	complain )
+			APP_COMPLAIN		complain )
     {
     static char smallbuf[2];
     int		nbytes= 0;
 
-    for ( ;; )
+    for (;;)
 	{
 	int	done= read( fd, smallbuf, 1 );
 
 	if  ( done != 1 )
 	    {
 	    LSDEB(done,strerror(errno));
-	    (*complain)( through, SIO_SMTPeREAD, (char *)0 );
+	    (*complain)( through, APP_SYSTEMeREAD, (char *)0 );
 	    return -1;
 	    }
 
@@ -526,7 +546,7 @@ static int mmReadCRLF(	int			fd,
 static int mmWriteCRLF(	int			fd,
 			const char *		buf,
 			void *			through,
-			SIO_SMTP_COMPLAIN	complain )
+			APP_COMPLAIN		complain )
     {
     static char		crlf[]	= "\015\012";
 
@@ -557,7 +577,7 @@ static int mmWriteHeader(	int			fd,
 				const char *		name,
 				const char *		value,
 				void *			through,
-				SIO_SMTP_COMPLAIN	complain )
+				APP_COMPLAIN		complain )
     {
     unsigned int	len;
 
@@ -573,39 +593,18 @@ static int mmWriteHeader(	int			fd,
     return 0;
     }
 
-static int mmWriteContentType(	int			fd,
-				const char *		type,
-				const char *		subtype,
-				void *			through,
-				SIO_SMTP_COMPLAIN	complain )
-    {
-    unsigned int	len;
-
-    if  ( ! type || ! type[0] || ! subtype || ! subtype[0] )
-	{ return 0;						}
-
-    len= strlen( type );
-    if  ( mmWrite( fd, "Content-Type: ", 14, through, complain )	||
-	  mmWrite( fd, type, len, through, complain )			||
-	  mmWrite( fd, "/", 1, through, complain )			||
-	  mmWriteCRLF( fd, subtype, through, complain )			)
-	{ LDEB(1); return -1;	}
-
-    return 0;
-    }
-
 static int mmWrite(	int			fd,
 			const char *		buf,
 			unsigned int		len,
 			void *			through,
-			SIO_SMTP_COMPLAIN	complain )
+			APP_COMPLAIN		complain )
     {
     int		done= write( fd, buf, len );
 
     if  ( done != (int)len )
 	{
 	LLDEB(len,done); SDEB(strerror(errno));
-	(*complain)( through, SIO_SMTPeWRITE, (char *)0 );
+	(*complain)( through, APP_SYSTEMeWRITE, (char *)0 );
 	return -1;
 	}
 
@@ -623,43 +622,67 @@ char *	sioSmtpGuessMailAddress( void )
     static char *	rval;
     char *		to;
 
-    const char *	domainname= (char *)0;
-    const char *	fullhost= (char *)0;
+    const char *	domainname= (const char *)0;
+    const char *	fullhost= (const char *)0;
+    const char *	nodename= (const char *)0;
     int			l;
 
     int			didFullName= 0;
 
-    struct utsname	un;
     struct passwd *	pwd;
 
+#   if HAVE_UNAME
+    struct utsname	un;
+#   endif
+
+#   ifdef USE_GETHOSTNAME
+    char			node_scratch[256+1];
+#   endif
+
     if  ( rval )
 	{ return rval;	}
 
+#   if HAVE_UNAME
     if  ( uname( &un ) < 0 )
 	{ LDEB(1); return (char *)0;	}
+    nodename= un.nodename;
+#   endif
+
+#   ifdef USE_GETHOSTNAME
+    if  ( gethostname( node_scratch, sizeof(node_scratch)- 1 ) )
+	{ LDEB(1); return (char *)0;		}
+
+    node_scratch[sizeof(node_scratch)-1]= '\0';
+    nodename= node_scratch;
+#   endif
+
+    if  ( ! nodename )
+	{ XDEB(nodename); return (char *)0;	}
 
 #   ifdef  UTSNAME_DOMAIN
     if  ( un.UTSNAME_DOMAIN[0] && strcmp( un.UTSNAME_DOMAIN, "(none)" ) )
 	{ domainname= un.UTSNAME_DOMAIN;	}
-    else{
 #   endif
-	if  ( strchr( un.nodename, '.' ) )
-	    { fullhost= un.nodename; }
-	else{
-	    struct hostent *	host= gethostbyname( un.nodename );
 
-	    l= strlen( un.nodename );
+    if  ( ! domainname )
+	{
+	if  ( strchr( nodename, '.' ) )
+	    { fullhost= nodename; }
+	else{
+	    struct hostent *	host= gethostbyname( nodename );
+
+	    l= strlen( nodename );
 	    if  ( host )
 		{
 		char **		a= host->h_aliases;
 
-		if  ( ! strncmp( un.nodename, host->h_name, l )	&&
+		if  ( ! strncmp( nodename, host->h_name, l )	&&
 		      host->h_name[l] == '.'			)
 		    { fullhost= host->h_name;	}
 		else{
 		    if  ( a ) while( *a )
 			{
-			if  ( ! strncmp( un.nodename, *a, l )	&&
+			if  ( ! strncmp( nodename, *a, l )	&&
 			      (*a)[l] == '.'			)
 			    { fullhost= *a; break;	}
 
@@ -686,11 +709,9 @@ char *	sioSmtpGuessMailAddress( void )
 		}
 
 	    if  ( ! fullhost )
-		{ fullhost= un.nodename;	}
+		{ fullhost= nodename;	}
 	    }
-#	ifdef  UTSNAME_DOMAIN
 	}
-#	endif
 
     if  ( ! domainname && ! fullhost )
 	{ XXDEB(domainname,fullhost); return (char *)0;	}
@@ -701,12 +722,14 @@ char *	sioSmtpGuessMailAddress( void )
 
     l= 0;
     l += strlen( pwd->pw_name );
+#   if HAVE_PW_GECOS
     if  ( pwd->pw_gecos && *pwd->pw_gecos )
 	{ l += strlen( pwd->pw_gecos )+ 3;	}
+#   endif
 
     if  ( domainname )
 	{
-	l += strlen( un.nodename );
+	l += strlen( nodename );
 	l += 1;
 	l += strlen( domainname );
 	}
@@ -717,6 +740,7 @@ char *	sioSmtpGuessMailAddress( void )
 	{ XDEB(rval); return (char *)0;	}
 
     to= rval; *to= '\0';
+#   if HAVE_PW_GECOS
     if  ( pwd->pw_gecos && *pwd->pw_gecos )
 	{
 	char *	s;
@@ -744,11 +768,12 @@ char *	sioSmtpGuessMailAddress( void )
 	    }
 
 	}
+#   endif
     strcpy( to, pwd->pw_name ); to += strlen( to );
     strcpy( to, "@" ); to += strlen( to );
     if  ( domainname )
 	{
-	strcpy( to, un.nodename ); to += strlen( to );
+	strcpy( to, nodename ); to += strlen( to );
 	strcpy( to, "." ); to += strlen( to );
 	strcpy( to, domainname ); to += strlen( to );
 	}
