@@ -1,9 +1,9 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	<string.h>
 #   include	"bmSegments.h"
 #   include	"bmintern.h"
-#   include	<debugon.h>
+#   include	<appDebugon.h>
 
 /************************************************************************/
 /*									*/
@@ -766,7 +766,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		{ break;			}
 
 	    /*
-	    DEB(printf( "++ RUN %5d .. %5d AT %3d\n", x0, x1, nnlu ));
+	    APP_DEB(appDebug( "++ RUN %5d .. %5d AT %3d\n", x0, x1, nnlu ));
 	    */
 
 	    /************************************************************/
@@ -783,7 +783,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		if  ( olu[beg].lu_use == LU_UNSPEC )
 		    {
 		    /*
-		    DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
+		    APP_DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
 				"ENDED", comps, olu, beg, beg+1 ));
 		    */
 		    if  ( bcTerminate( comps+ compid, olu+ beg, row ) )
@@ -791,7 +791,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		    }
 		else{
 		    /*
-		    DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
+		    APP_DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
 				"READY", comps, olu, beg, beg+1 ));
 		    */
 		    }
@@ -805,7 +805,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		if  ( olu[end].lu_x0 >= x1 )
 		    { break;	}
 		/*
-		DEB(bcPrint( olu[end].lu_x0, olu[end].lu_x1, end,
+		APP_DEB(bcPrint( olu[end].lu_x0, olu[end].lu_x1, end,
 			    "PREVIOUS", comps, olu, end, end+1 ));
 		*/
 		end++;
@@ -818,7 +818,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		if  ( beg < end- 1 )
 		    {
 		    /*
-		    DEB(bcPrint( x0, x1, nnlu,
+		    APP_DEB(bcPrint( x0, x1, nnlu,
 				"MERGE", comps, olu, beg, end ));
 		    */
 		    sn= bmSegmentNode( comps[compid].ci_compo, row );
@@ -836,7 +836,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 						    olu[beg].lu_edge, 1 ) )
 			    { LDEB(1); goto failure;	}
 			/*
-			DEB(bcPrintEdge(beg,olu[beg].lu_edge));
+			APP_DEB(bcPrintEdge(beg,olu[beg].lu_edge));
 			*/
 			olu[beg].lu_use= LU_MERGED;
 			olu[beg].lu_node= sn;
@@ -856,7 +856,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		    olu[beg].lu_edge= se;
 		    olu[beg].lu_compid= compid;
 		    /*
-		    DEB(bcPrint( x0, x1, nnlu,
+		    APP_DEB(bcPrint( x0, x1, nnlu,
 				"-----", comps, olu, beg, end ));
 		    */
 		    }
@@ -864,7 +864,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		if  ( olu[beg].lu_use != LU_UNSPEC )
 		    {
 		    /*
-		    DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
+		    APP_DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
 				"SPLIT", comps, olu, beg, beg+1 ));
 		    */
 		    sn= nlu[nnlu-1].lu_node;
@@ -892,7 +892,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		else{
 		    compid= olu[beg].lu_compid;
 		    /*
-		    DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
+		    APP_DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
 				"EXTENDED", comps, olu, beg, beg+1 ));
 		    */
 		    if  ( bmAddRunToSegmentEdge( comps[compid].ci_compo,
@@ -919,7 +919,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		nlu[nnlu].lu_use= LU_UNSPEC;
 		nnlu++;
 		/*
-		DEB(bcPrint( x0, x1, nnlu,
+		APP_DEB(bcPrint( x0, x1, nnlu,
 			    "NEW", comps, nlu, nnlu-1, nnlu ));
 		*/
 		}
@@ -932,7 +932,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 	    if  ( olu[beg].lu_use == LU_UNSPEC )
 		{
 		/*
-		DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
+		APP_DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
 			    "ENDED", comps, olu, beg, beg+1 ));
 		*/
 		if  ( bcTerminate( comps+ compid, olu+ beg, row ) )
@@ -940,7 +940,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 		}
 	    else{
 		/*
-		DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
+		APP_DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
 			    "READY", comps, olu, beg, beg+1 ));
 		*/
 		}
@@ -952,7 +952,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 	}
 
     /*
-    DEB(printf( "%-5d ROWS:\n", row ));
+    APP_DEB(appDebug( "%-5d ROWS:\n", row ));
     */
     beg= 0;
     while( beg < onlu )
@@ -961,7 +961,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 	if  ( olu[beg].lu_use == LU_UNSPEC )
 	    {
 	    /*
-	    DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
+	    APP_DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
 			"BOTTOM", comps, olu, beg, beg+1 ));
 	    */
 	    if  ( bcTerminate( comps+ compid, olu+ beg, row ) )
@@ -969,7 +969,7 @@ int bcComponents(	BitmapSegment ***		pSegments,
 	    }
 	else{
 	    /*
-	    DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
+	    APP_DEB(bcPrint( olu[beg].lu_x0, olu[beg].lu_x1, beg,
 			"ERROR", comps, olu, beg, beg+1 ));
 	    */
 	    goto failure;
