@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	<stdio.h>
 #   include	<stdlib.h>
@@ -8,11 +8,15 @@
 #   include	<sioHex.h>
 #   include	<sioEndian.h>
 #   include	<sioStdio.h>
-#   include	<debugon.h>
+
+#   include	<appDebugon.h>
 
 /************************************************************************/
+/*									*/
 /*  Read a 3 or 4 byte color table.					*/
+/*									*/
 /************************************************************************/
+
 static int	bmBmpRead3ColorTable(	int			colorCount,
 					SimpleInputStream *	sis,
 					RGB8Color *		palette	)
@@ -25,6 +29,7 @@ static int	bmBmpRead3ColorTable(	int			colorCount,
 	palette[i].rgb8Blue=	sioInGetCharacter( sis );
 	palette[i].rgb8Green=	sioInGetCharacter( sis );
 	palette[i].rgb8Red=	sioInGetCharacter( sis );
+	palette[i].rgb8Alpha=	255;
 	}
 
     return 0;
@@ -59,6 +64,7 @@ static int	bmBmpRead4ColorTable(	int			colorCount,
 	palette[i].rgb8Green=	sioInGetCharacter( sis );
 	palette[i].rgb8Red=	sioInGetCharacter( sis );
 	(void) sioInGetCharacter( sis );
+	palette[i].rgb8Alpha=	255;
 	}
 
     return 0;
@@ -82,13 +88,16 @@ static int	bmBmpWrite4ColorTable(	int			colorCount,
     }
 
 /************************************************************************/
+/*									*/
 /*  Read a Microsoft BMP header.					*/
 /*  1)  Read fields common to all formats.				*/
 /*  2)  Set all distinguishing properties to nonsense, then dixede on	*/
 /*	the file type.							*/
 /*  3)  If it is not a direct color image, read palette.		*/
 /*  4)  Assume reasonable values for missing fields.			*/
+/*									*/
 /************************************************************************/
+
 static int bmpReadImageHeader(	BmpFileHeader *		bfh,
 				SimpleInputStream *	sis,
 				RGB8Color **		pPalette )
@@ -120,8 +129,8 @@ static int bmpReadImageHeader(	BmpFileHeader *		bfh,
 	    /*  4  */
 	    bfh->bfhCompression= 0;
 	    bfh->bfhBufferLength= 0;
-	    bfh->bfhXResolution= 100* POINTS_PER_CM;
-	    bfh->bfhYResolution= 100* POINTS_PER_CM;
+	    bfh->bfhXResolution= POINTS_PER_M;
+	    bfh->bfhYResolution= POINTS_PER_M;
 	    bfh->bfhColorCount= colorCount;
 	    bfh->bfhImportantColors= colorCount;
 
@@ -223,8 +232,11 @@ static int bmpReadImageHeader(	BmpFileHeader *		bfh,
     }
 
 /************************************************************************/
+/*									*/
 /*  Read the file header as it appears in a BMP file.			*/
+/*									*/
 /************************************************************************/
+
 static int bmpReadFileHeader(	BmpFileHeader *		bfh,
 				SimpleInputStream *	sis,
 				RGB8Color **		pPalette )
@@ -249,8 +261,11 @@ static int bmpReadFileHeader(	BmpFileHeader *		bfh,
     }
 
 /************************************************************************/
+/*									*/
 /*  Read an uncompresse image from a BMP file.				*/
+/*									*/
 /************************************************************************/
+
 static int bmBmpReadUncompressed(	SimpleInputStream *	sis,
 					unsigned char *		buffer,
 					int			pixelsHigh,
@@ -306,9 +321,13 @@ static int bmBmpReadUncompressed(	SimpleInputStream *	sis,
     }
 
 /************************************************************************/
+/*									*/
 /*  Translate a microsoft header to our bitmap descriptors.		*/
+/*									*/
 /*  1)  Boolean to value.						*/
+/*									*/
 /************************************************************************/
+
 static int bmpHeaderToDescription(	BitmapDescription *	bd,
 					int *			pBytesPerRow,
 					int *			pPrivateFormat,
@@ -330,7 +349,7 @@ static int bmpHeaderToDescription(	BitmapDescription *	bd,
 	if  ( bfh->bfhColorCount == 0 )
 	    {
 	    bd->bdColorCount=
-		(1U << (bfh->bfhPlanes * bfh->bfhBitsPerPixel ));
+		( 1U << (bfh->bfhPlanes * bfh->bfhBitsPerPixel ));
 	    }
 	else{ bd->bdColorCount= bfh->bfhColorCount; }
 
@@ -346,22 +365,25 @@ static int bmpHeaderToDescription(	BitmapDescription *	bd,
 
     *pBytesPerRow= ( bfh->bfhBitsPerPixel* bd->bdPixelsWide + 7 )/ 8;
 
-    bd->bdUnit= BMunCM;
-    bd->bdXResolution= bfh->bfhXResolution/100;
-    bd->bdYResolution= bfh->bfhYResolution/100;
+    bd->bdUnit= BMunM;
+    bd->bdXResolution= bfh->bfhXResolution;
+    bd->bdYResolution= bfh->bfhYResolution;
 
     /********************************************************************/
     /*  Check people that even run microsoft systems in their brain.	*/
     /********************************************************************/
-    if  ( bd->bdXResolution == 0 || bd->bdXResolution == 0 )
-	{ bd->bdYResolution= bd->bdXResolution= 1; bd->bdUnit= BMunPOINT; }
+    if  ( bd->bdXResolution < 500 || bd->bdXResolution < 500 )
+	{ bd->bdYResolution= bd->bdXResolution= 1; bd->bdUnit= BMunPIXEL; }
 
     return 0;
     }
 
 /************************************************************************/
+/*									*/
 /*  Convert a bitmap description to a Bitmap header.			*/
+/*									*/
 /************************************************************************/
+
 static int bmbmpDescriptionToHeader(	BmpFileHeader *		bfh,
 					RGB8Color **		pPalette,
 					int			privateFormat,
@@ -389,18 +411,26 @@ static int bmbmpDescriptionToHeader(	BmpFileHeader *		bfh,
 
     switch( bd->bdUnit )
 	{
-	case BMunCM:
-	    bfh->bfhXResolution= 100* bd->bdXResolution;
-	    bfh->bfhYResolution= 100* bd->bdYResolution;
+	case BMunM:
+	    bfh->bfhXResolution= bd->bdXResolution;
+	    bfh->bfhYResolution= bd->bdYResolution;
 	    break;
+
 	case BMunINCH:
 	    bfh->bfhXResolution= 40* bd->bdXResolution;
 	    bfh->bfhYResolution= 40* bd->bdYResolution;
 	    break;
+
 	case BMunPOINT:
-	    bfh->bfhXResolution= 100* POINTS_PER_CM* bd->bdXResolution;
-	    bfh->bfhYResolution= 100* POINTS_PER_CM* bd->bdYResolution;
+	    bfh->bfhXResolution= POINTS_PER_M* bd->bdXResolution;
+	    bfh->bfhYResolution= POINTS_PER_M* bd->bdYResolution;
 	    break;
+
+	case BMunPIXEL:
+	    bfh->bfhXResolution= 0;
+	    bfh->bfhYResolution= 0;
+	    break;
+
 	default:
 	    LDEB(bd->bdUnit); return -1;
 	}
@@ -464,14 +494,15 @@ static int bmbmpDescriptionToHeader(	BmpFileHeader *		bfh,
 		{
 		case 16:
 		    if  ( hasAlpha )
-			{ bfh->bfhBitsPerPixel= 8; }
+			{
+			bfh->bfhBitsPerPixel= 8;
+			bfh->bfhCompression= 1;
+			}
 		    else{
 			LLDEB(bd->bdColorEncoding,bd->bdBitsPerPixel);
 			return -1;
 			}
 		    bfh->bfhColorCount= bd->bdColorCount;
-		    bfh->bfhCompression= 1;
-		    /**/bfh->bfhCompression= 0;
 		    palette= bd->bdRGB8Palette;
 		    break;
 		case 8:
@@ -505,365 +536,12 @@ static int bmbmpDescriptionToHeader(	BmpFileHeader *		bfh,
     *pPalette= palette; return 0;
     }
 
-/************************************************************************/
-/*  Try to see the file as a momochrome file.				*/
-/*  This will make it possible to save it in monochrome formats such	*/
-/*  the TIFF fax formats.						*/
-/************************************************************************/
-static void bmpSqueeze_8_4(	const unsigned char *	fr,
-				unsigned char *		to,
-				int			wide )
-    {
-    int			col;
-    unsigned char	s[8];
-
-    for ( col= 0; col+ 1 < wide; fr += 2, col += 2, to++ )
-	{
-	*to=	( ( fr[0] & 0x0f ) << 4 )	|
-		( ( fr[1] & 0x0f )      )	;
-	}
-
-    if  ( col < wide )
-	{
-	memset( s, 0, 8 );
-	memcpy( s, fr, wide- col ); fr= s;
-
-	*to=	( ( fr[0] & 0x0f ) << 4 )	|
-		( ( fr[1] & 0x0f )      )	;
-	}
-    }
-
-static void bmpSqueeze_8_2(	const unsigned char *	fr,
-				unsigned char *		to,
-				int			wide )
-    {
-    int			col;
-    unsigned char	s[8];
-
-    for ( col= 0; col+ 3 < wide;
-				fr += 4, col += 4, to++ )
-	{
-	*to=	( ( fr[0] & 0x03 ) << 6 )	|
-		( ( fr[1] & 0x03 ) << 4 )	|
-		( ( fr[2] & 0x03 ) << 2 )	|
-		( ( fr[3] & 0x03 )      )	;
-	}
-
-    if  ( col < wide )
-	{
-	memset( s, 0, 8 );
-	memcpy( s, fr, wide- col ); fr= s;
-
-	*to=	( ( fr[0] & 0x03 ) << 6 )	|
-		( ( fr[1] & 0x03 ) << 4 )	|
-		( ( fr[2] & 0x03 ) << 2 )	|
-		( ( fr[3] & 0x03 )      )	;
-	}
-    }
-
-static void bmpSqueeze_8_1(	const unsigned char *	fr,
-				unsigned char *		to,
-				int			wide )
-    {
-    int			col;
-    unsigned char	s[8];
-
-    for ( col= 0; col+ 7 < wide; fr += 8, col += 8, to++ )
-	{
-	*to=	( ( fr[0] & 0x01 ) << 7 )	|
-		( ( fr[1] & 0x01 ) << 6 )	|
-		( ( fr[2] & 0x01 ) << 5 )	|
-		( ( fr[3] & 0x01 ) << 4 )	|
-		( ( fr[4] & 0x01 ) << 3 )	|
-		( ( fr[5] & 0x01 ) << 2 )	|
-		( ( fr[6] & 0x01 ) << 1 )	|
-		( ( fr[7] & 0x01 )      )	;
-	}
-
-    if  ( col < wide )
-	{
-	memset( s, 0, 8 );
-	memcpy( s, fr, wide- col ); fr= s;
-
-	*to=	( ( fr[0] & 0x01 ) << 7 )	|
-		( ( fr[1] & 0x01 ) << 6 )	|
-		( ( fr[2] & 0x01 ) << 5 )	|
-		( ( fr[3] & 0x01 ) << 4 )	|
-		( ( fr[4] & 0x01 ) << 3 )	|
-		( ( fr[5] & 0x01 ) << 2 )	|
-		( ( fr[6] & 0x01 ) << 1 )	|
-		( ( fr[7] & 0x01 )      )	;
-	}
-    }
-
-static void bmpSqueeze_4_2(	const unsigned char *	fr,
-				unsigned char *		to,
-				int			wide )
-    {
-    int			col;
-    unsigned char	s[8];
-
-    for ( col= 0; col+ 3 < wide; fr += 2, col += 4, to++ )
-	{
-	*to=	( ( fr[0] & 0x30 ) << 2 )	|
-		( ( fr[0] & 0x03 ) << 6 )	|
-		( ( fr[1] & 0x30 ) << 2 )	|
-		( ( fr[1] & 0x03 )      )	;
-	}
-
-    if  ( col < wide )
-	{
-	memset( s, 0, 8 );
-	memcpy( s, fr, ( wide- col+ 1 )/2 );
-	fr= s;
-
-	*to=	( ( fr[0] & 0x30 ) << 2 )	|
-		( ( fr[0] & 0x03 ) << 6 )	|
-		( ( fr[1] & 0x30 ) << 2 )	|
-		( ( fr[1] & 0x03 )      )	;
-	}
-    }
-
-static void bmpSqueeze_4_1(	const unsigned char *	fr,
-				unsigned char *		to,
-				int			wide )
-    {
-    int			col;
-    unsigned char	s[8];
-
-    for ( col= 0; col+ 7 < wide; fr += 4, col += 8, to++ )
-	{
-	*to=	( ( fr[0] & 0x10 ) << 3 )	|
-		( ( fr[0] & 0x01 ) << 6 )	|
-		( ( fr[1] & 0x10 ) << 1 )	|
-		( ( fr[1] & 0x01 ) << 4 )	|
-		( ( fr[2] & 0x10 ) >> 1 )	|
-		( ( fr[2] & 0x01 ) << 2 )	|
-		( ( fr[3] & 0x10 ) >> 3 )	|
-		( ( fr[3] & 0x01 )      )	;
-	}
-
-    if  ( col < wide )
-	{
-	memset( s, 0, 8 );
-	memcpy( s, fr, ( wide- col+ 1 )/2 );
-	fr= s;
-
-	*to=	( ( fr[0] & 0x10 ) << 3 )	|
-		( ( fr[0] & 0x01 ) << 6 )	|
-		( ( fr[1] & 0x10 ) << 1 )	|
-		( ( fr[1] & 0x01 ) << 4 )	|
-		( ( fr[2] & 0x10 ) >> 1 )	|
-		( ( fr[2] & 0x01 ) << 2 )	|
-		( ( fr[3] & 0x10 ) >> 3 )	|
-		( ( fr[3] & 0x01 )      )	;
-	}
-    }
-
-static void bmpSqueeze_2_1(	const unsigned char *	fr,
-				unsigned char *		to,
-				int			wide )
-    {
-    int			col;
-    unsigned char	s[8];
-
-    for ( col= 0; col+ 7 < wide; fr += 2, col += 8, to++ )
-	{
-	*to=	( ( fr[0] & 0x40 ) << 1 )	|
-		( ( fr[0] & 0x10 ) << 2 )	|
-		( ( fr[0] & 0x04 ) << 3 )	|
-		( ( fr[0] & 0x01 ) << 4 )	|
-		( ( fr[1] & 0x40 ) << 3 )	|
-		( ( fr[1] & 0x10 ) << 2 )	|
-		( ( fr[1] & 0x04 ) << 1 )	|
-		( ( fr[1] & 0x01 )      )	;
-	}
-
-    if  ( col < wide )
-	{
-	memset( s, 0, 8 );
-	memcpy( s, fr, ( wide- col+ 3 )/4 );
-	fr= s;
-
-	*to=	( ( fr[0] & 0x40 ) << 1 )	|
-		( ( fr[0] & 0x10 ) << 2 )	|
-		( ( fr[0] & 0x04 ) << 3 )	|
-		( ( fr[0] & 0x01 ) << 4 )	|
-		( ( fr[1] & 0x40 ) << 3 )	|
-		( ( fr[1] & 0x10 ) << 2 )	|
-		( ( fr[1] & 0x04 ) << 1 )	|
-		( ( fr[1] & 0x01 )      )	;
-	}
-    }
-
-static int bmpSqueezeBuffer(	const BitmapDescription *	bd,
-				int				bitsPerPixel,
-				int				bytesPerRow,
-				unsigned char *			buffer )
-    {
-    int				row;
-
-    const unsigned char *	fr;
-    unsigned char *		to;
-
-    switch( bd->bdBitsPerPixel )
-	{
-	case 8:
-	    switch( bitsPerPixel )
-		{
-		case 8:
-		    return 0;
-		case 4:
-		    for ( row= 0; row < bd->bdPixelsHigh; row++ )
-			{
-			fr= buffer+ row* bd->bdBytesPerRow;
-			to= buffer+ row* bytesPerRow;
-
-			bmpSqueeze_8_4( fr, to, bd->bdPixelsWide );
-			}
-		    return 0;
-		case 2:
-		    for ( row= 0; row < bd->bdPixelsHigh; row++ )
-			{
-			fr= buffer+ row* bd->bdBytesPerRow;
-			to= buffer+ row* bytesPerRow;
-
-			bmpSqueeze_8_2( fr, to, bd->bdPixelsWide );
-			}
-		    return 0;
-		case 1:
-		    for ( row= 0; row < bd->bdPixelsHigh; row++ )
-			{
-			fr= buffer+ row* bd->bdBytesPerRow;
-			to= buffer+ row* bytesPerRow;
-
-			bmpSqueeze_8_1( fr, to, bd->bdPixelsWide );
-			}
-		    return 0;
-		default:
-		    LLDEB(bd->bdBitsPerPixel,bitsPerPixel); return -1;
-		}
-	case 4:
-	    switch( bitsPerPixel )
-		{
-		case 4:
-		    return 0;
-		case 2:
-		    for ( row= 0; row < bd->bdPixelsHigh; row++ )
-			{
-			fr= buffer+ row* bd->bdBytesPerRow;
-			to= buffer+ row* bytesPerRow;
-
-			bmpSqueeze_4_2( fr, to, bd->bdPixelsWide );
-			}
-		    return 0;
-		case 1:
-		    for ( row= 0; row < bd->bdPixelsHigh; row++ )
-			{
-			fr= buffer+ row* bd->bdBytesPerRow;
-			to= buffer+ row* bytesPerRow;
-
-			bmpSqueeze_4_1( fr, to, bd->bdPixelsWide );
-			}
-		    return 0;
-		default:
-		    LLDEB(bd->bdBitsPerPixel,bitsPerPixel); return -1;
-		}
-	case 2:
-	    switch( bitsPerPixel )
-		{
-		case 2:
-		    return 0;
-		case 1:
-		    for ( row= 0; row < bd->bdPixelsHigh; row++ )
-			{
-			fr= buffer+ row* bd->bdBytesPerRow;
-			to= buffer+ row* bytesPerRow;
-
-			bmpSqueeze_2_1( fr, to, bd->bdPixelsWide );
-			}
-		    return 0;
-		default:
-		    LLDEB(bd->bdBitsPerPixel,bitsPerPixel); return -1;
-		}
-	case 1:
-	    switch( bitsPerPixel )
-		{
-		case 1:
-		    return 0;
-		default:
-		    LLDEB(bd->bdBitsPerPixel,bitsPerPixel); return -1;
-		}
-	    break;
-	default:
-	    LLDEB(bd->bdBitsPerPixel,bitsPerPixel); return -1;
-	}
-    }
-
-static int bmpMakeMonochrome(	BitmapDescription *	bd,
-				RGB8Color *		palette,
-				unsigned char *		buffer )
-    {
-    int		hasAlpha= bd->bdHasAlpha != 0;
-    int		bitsPerPixel;
-    int		bytesPerRow;
-
-    if  ( bd->bdColorCount == 2 )
-	{
-	bitsPerPixel= 1+ hasAlpha;
-	bytesPerRow= ( bd->bdPixelsWide* bitsPerPixel+ 7 )/ 8;
-
-	if  ( palette[0].rgb8Red	== 0	&&
-	      palette[0].rgb8Green	== 0	&&
-	      palette[0].rgb8Blue	== 0	&&
-	      palette[1].rgb8Red	== 255	&&
-	      palette[1].rgb8Green	== 255	&&
-	      palette[1].rgb8Blue	== 255	)
-	    {
-	    if  ( bmpSqueezeBuffer( bd, bitsPerPixel, bytesPerRow, buffer ) )
-		{ LDEB(1); return -1;	}
-
-	    free( (char *)palette );
-	    bd->bdBitsPerSample= 1;
-	    bd->bdColorEncoding= BMcoWHITEBLACK;
-	    bd->bdSamplesPerPixel= 1+ hasAlpha;
-	    bd->bdBitsPerPixel= bitsPerPixel;
-	    bd->bdBytesPerRow= bytesPerRow;
-	    bd->bdBufferLength= bd->bdPixelsHigh* bytesPerRow;
-	    return 0;
-	    }
-
-	if  ( palette[0].rgb8Red	== 255	&&
-	      palette[0].rgb8Green	== 255	&&
-	      palette[0].rgb8Blue	== 255	&&
-	      palette[1].rgb8Red	== 0	&&
-	      palette[1].rgb8Green	== 0	&&
-	      palette[1].rgb8Blue	== 0	)
-	    {
-	    if  ( bmpSqueezeBuffer( bd, bitsPerPixel, bytesPerRow, buffer ) )
-		{ LDEB(1); return -1;	}
-
-	    free( (char *)palette );
-	    bd->bdBitsPerSample= 1;
-	    bd->bdColorEncoding= BMcoBLACKWHITE;
-	    bd->bdSamplesPerPixel= 1+ hasAlpha;
-	    bd->bdBitsPerPixel= bitsPerPixel;
-	    bd->bdBytesPerRow= bytesPerRow;
-	    bd->bdBufferLength= bd->bdPixelsHigh* bytesPerRow;
-	    return 0;
-	    }
-
-	return -1;
-	}
-
-    return -1;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Read the data bytes from a Microsoft BMP image.			*/
 /*									*/
 /************************************************************************/
+
 static int bmBmpReadImageBytes(	unsigned char **		pBuffer,
 				SimpleInputStream *		sis,
 				int				compression,
@@ -1108,7 +786,7 @@ int bmReadBmpFile(	const char *		filename,
     *pPrivateFormat= bfh.bfhSizeOfRestOfHeader;
 
     if  ( bfh.bfhColorCount > 0				&&
-	  ! bmpMakeMonochrome( bd, palette, buffer )	)
+	  ! bmMakeMonochrome( bd, palette, buffer )	)
 	{ return 0;	}
 
     if  ( bfh.bfhBitsPerPixel != 24 || bfh.bfhColorCount != 0 )
@@ -1122,6 +800,7 @@ int bmReadBmpFile(	const char *		filename,
 /*  1)  Write fields common to all formats.				*/
 /*  2)  If it is not a direct color image, write palette.		*/
 /************************************************************************/
+
 static int bmpWriteImageHeader(	const BmpFileHeader *	bfh,
 				SimpleOutputStream *	sos,
 				RGB8Color *		palette )
@@ -1245,6 +924,7 @@ static int bmpWriteFileHeader(	const BmpFileHeader *	bfh,
 /*  3)	Repeating run.							*/
 /*									*/
 /************************************************************************/
+
 static int bmBmpWriteScanLine(	SimpleOutputStream *	sos,
 				int			pixelsPerByte,
 				int			maxByteCount,
@@ -1351,6 +1031,7 @@ static int bmBmpWriteScanLine(	SimpleOutputStream *	sos,
 /*  Write the image bytes of a BMP file.				*/
 /*									*/
 /************************************************************************/
+
 static int bmBmpWriteImageBytes(const BitmapDescription *	bd,
 				const unsigned char *		buffer,
 				int				compression,
@@ -1367,6 +1048,8 @@ static int bmBmpWriteImageBytes(const BitmapDescription *	bd,
     int				bytesPerRow;
     unsigned char *		inflated= (unsigned char *)0;
 
+    int				white= -1;
+
     switch( compression )
 	{
 	case 0:
@@ -1411,14 +1094,39 @@ static int bmBmpWriteImageBytes(const BitmapDescription *	bd,
 		bytesPerRow *= 2;
 		}
 
+	    if  ( compression == 1		&&
+		  bd->bdHasAlpha		&&
+		  bd->bdBitsPerPixel == 16	)
+		{
+		bytesPerRow= bytesPerRow/ 2;
+
+		inflated= malloc( bytesPerRow );
+		if  ( ! inflated )
+		    { LXDEB(bd->bdBytesPerRow,inflated); return -1;	}
+
+		for ( i= 0; i < bd->bdColorCount; i++ )
+		    {
+		    if  ( bd->bdRGB8Palette[i].rgb8Red == 255	&&
+			  bd->bdRGB8Palette[i].rgb8Green == 255	&&
+			  bd->bdRGB8Palette[i].rgb8Blue == 255	)
+			{ white= i; break;	}
+		    }
+
+		if  ( white < 0 )
+		    { LDEB(white); white= 0;	}
+		}
+
 	    maxByteCount= 255/ compression;
 	    for ( row= 0; row < bd->bdPixelsHigh; row++ )
 		{
+		unsigned char *	to;
+
 		from= buffer+ ( bd->bdPixelsHigh- row- 1 )* bd->bdBytesPerRow;
 
-		if  ( inflated )
+		if  ( compression == 2		&&
+		      bd->bdBitsPerPixel == 2	)
 		    {
-		    unsigned char *	to= inflated;
+		    to= inflated;
 
 		    for ( i= 0; i < bd->bdBytesPerRow; to += 2, from++, i++ )
 			{
@@ -1431,6 +1139,22 @@ static int bmBmpWriteImageBytes(const BitmapDescription *	bd,
 		    from= inflated;
 		    }
 
+		if  ( compression == 1		&&
+		      bd->bdHasAlpha		&&
+		      bd->bdBitsPerPixel == 16	)
+		    {
+		    to= inflated;
+
+		    for ( i= 0; i < bytesPerRow; from += 2, to++, i++ )
+			{
+			if  ( from[1] )
+			    { *to= *from;	}
+			else{ *to= white;	}
+			}
+
+		    from= inflated;
+		    }
+
 		done= bmBmpWriteScanLine( sos, compression, maxByteCount,
 				from, bd->bdPixelsWide, bytesPerRow );
 
@@ -1479,7 +1203,8 @@ int bmWriteBmpFile(	const char *			filename,
     if  ( ! sos )
 	{ SDEB(filename); return -1;	}
 
-    if  ( bmbmpDescriptionToHeader( &bfh, &palette, privateFormat, 0, bd ) )
+    if  ( bmbmpDescriptionToHeader( &bfh, &palette,
+					privateFormat, bd->bdHasAlpha, bd ) )
 	{ SDEB(filename); sioOutClose( sos ); return -1;	}
 
     done= bmpWriteFileHeader( &bfh, sos, palette );
@@ -1515,14 +1240,18 @@ int bmCanWriteBmpFile( const BitmapDescription *	bd,
     { return 0;	}
 
 /************************************************************************/
+/*									*/
 /*  Read a Microsoft Ico file.						*/
+/*									*/
 /*  1)	Read and check header.						*/
 /*  2)  Translate header to a BitmapDescription.			*/
 /*  2a) Only use XOR part of image.					*/
 /*  3)  Do not seek to data. It immediately follows.			*/
 /*  4)  Read image.							*/
 /*  5)  Combine with transparency mask.					*/
+/*									*/
 /************************************************************************/
+
 int bmReadIcoFile(	const char *		filename,
 			unsigned char **	pBuffer,
 			BitmapDescription *	bd,
@@ -1684,7 +1413,7 @@ int bmReadIcoFile(	const char *		filename,
     *pPrivateFormat= bfh.bfhSizeOfRestOfHeader;
 
     if  ( bfh.bfhColorCount > 0				&&
-	  ! bmpMakeMonochrome( bd, palette, buffer )	)
+	  ! bmMakeMonochrome( bd, palette, buffer )	)
 	{ return 0;	}
 
     if  ( bfh.bfhBitsPerPixel != 24 || bfh.bfhColorCount != 0 )
@@ -1699,6 +1428,7 @@ int bmReadIcoFile(	const char *		filename,
 /*  metafile in a word processing document.				*/
 /*									*/
 /************************************************************************/
+
 int bmBmpReadDib(	BitmapDescription *	pBd,
 			unsigned char **	pBuffer,
 			SimpleInputStream *	sis )
@@ -1738,10 +1468,11 @@ int bmBmpReadDib(	BitmapDescription *	pBd,
 
     bytesRead += done;
 
-    if  ( bfh.bfhColorCount == 0			||
-	  bmpMakeMonochrome( &bd, palette, buffer )	)
+    if  ( bd.bdColorCount != 2				||
+	  bd.bdColorEncoding != BMcoRGB8PALETTE		||
+	  bmMakeMonochrome( &bd, palette, buffer )	)
 	{
-	if  ( bfh.bfhBitsPerPixel != 24 || bfh.bfhColorCount != 0 )
+	if  ( bd.bdColorEncoding == BMcoRGB8PALETTE )
 	    { bd.bdRGB8Palette= palette;	}
 	}
 
