@@ -1,4 +1,4 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	"bmintern.h"
 
@@ -10,7 +10,7 @@
 #   undef	y0
 #   undef	y1
 
-#   include	<debugon.h>
+#   include	<appDebugon.h>
 
 #   ifndef	M_PI
 #	define	M_PI	3.14159265358979323846
@@ -79,8 +79,43 @@ static void bmFlip4Init( int n )
     }
 
 /************************************************************************/
-/*  Flip a scan line consisting of 1 bit pixels				*/
+/*									*/
+/*  Flip the bytes in a scan line.					*/
+/*									*/
 /************************************************************************/
+
+int bmFlipBytes(		unsigned char *	buffer,
+				int		byteCount,
+				int		bitsPerPixel )
+    {
+    switch( bitsPerPixel )
+	{
+	case  1:
+	    bmFlip1Init( 8/bitsPerPixel ); break;
+	case  2:
+	    bmFlip2Init( 8/bitsPerPixel ); break;
+	case  4:
+	    bmFlip4Init( 8/bitsPerPixel ); break;
+	case  8:
+	    return 0;
+	case 24:
+	default:
+	    LDEB(bitsPerPixel); return -1;
+	}
+
+    while( byteCount-- > 0 )
+	{ *buffer= bmFlipRight[*buffer]; buffer++;	}
+
+    return 0;
+    }
+
+
+/************************************************************************/
+/*									*/
+/*  Flip a scan line consisting of 1 bit pixels				*/
+/*									*/
+/************************************************************************/
+
 static void bmFlipBits(	unsigned char *		to,
 			const unsigned char *	from,
 			int			pixelsWide,
@@ -178,11 +213,12 @@ int bmVerticalFlip(	BitmapDescription *		bdOut,
 /************************************************************************/
 /*  Mirror bitmap images: Rotate 180 degrees.				*/
 /************************************************************************/
+
 int bmUpsideDown(	BitmapDescription *		bdOut,
 			const BitmapDescription *	bdIn,
 			unsigned char **		pBufOut,
 			const unsigned char *		bufIn,
-			int				ignoredInt	)
+			int				ignoredInt )
     {
     const unsigned char *	from;
     unsigned char *		to;
@@ -277,11 +313,12 @@ int bmHorizontalFlip(	BitmapDescription *		bdOut,
 /************************************************************************/
 /*  Rotate a bitmap by 0,90,180,270 degrees.				*/
 /************************************************************************/
+
 int bmRotate90(	BitmapDescription *		bdOut,
 		const BitmapDescription *	bdIn,
 		unsigned char **		pBufOut,
 		const unsigned char *		bufIn,
-		int				angle	)
+		int				angle )
     {
     const unsigned char *	from;
     unsigned char *		to;
@@ -492,6 +529,7 @@ int bmRotate90(	BitmapDescription *		bdOut,
 /************************************************************************/
 /*  Rotate a 1, 2, 4 bit scan line over a sharp angle.			*/
 /************************************************************************/
+
 static void bmRotateScanline124( const unsigned char *	from,
 				int			pixelsWide,
 				unsigned char *		bufOut,
@@ -551,6 +589,7 @@ static void bmRotateScanline124( const unsigned char *	from,
 /************************************************************************/
 /*  Rotate an 8 bit scan line over a sharp angle.			*/
 /************************************************************************/
+
 static void bmRotateScanline8(	const unsigned char *	from,
 				int			pixelsWide,
 				unsigned char *		bufOut,
@@ -588,6 +627,7 @@ static void bmRotateScanline8(	const unsigned char *	from,
 /************************************************************************/
 /*  Rotate a 24 bit scan line over a sharp angle.			*/
 /************************************************************************/
+
 static void bmRotateScanline24(	const unsigned char *	from,
 				int			pixelsWide,
 				unsigned char *		bufOut,
@@ -633,11 +673,12 @@ static void bmRotateScanline24(	const unsigned char *	from,
 /*  Rotate a bitmap over an arbitrary angle.				*/
 /*  1)  Sanity of the input.						*/
 /************************************************************************/
+
 int bmRotate(	BitmapDescription *		bdOut,
 		const BitmapDescription *	bdIn,
 		unsigned char **		pBufOut,
 		const unsigned char *		bufIn,
-		double				angle	)
+		double				angle )
     {
     int				rotations;
 
