@@ -1,7 +1,10 @@
-#include "config.h"
-#include <stdio.h>
-#include <unistd.h>
-#include <reg.h>
+#   include	"libregConfig.h"
+
+#   include	<stdio.h>
+#   include	<unistd.h>
+#   include	<string.h>
+
+#   include	<reg.h>
 
 void regErrorMessage(char * s)
 {
