@@ -19,20 +19,26 @@
  *	3. Altered versions must be plainly marked as such, and must not
  *		be misrepresented as being the original software.
  *
+ * NOTE that Mark de Does (M.de.Does@inter.nl.net) made many
+ * modifications to this source code without changing the original 
+ * approach.
+ *
  * Beware that some of this code is subtly aware of the way operator
- * precedence is structured in regular expressions.  Serious changes in
+ * precedence is structured in regular expressions. Serious changes in
  * regular-expression syntax might require a total rethink.
+ *
  */
-#include "config.h"
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <reg.h>
-#include "regmagic.h"
+#   include	"libregConfig.h"
 
-#include <debugon.h>
+#   include	<stdio.h>
+#   include	<string.h>
+#   include	<stdlib.h>
+#   include	<ctype.h>
+#   include	<reg.h>
+#   include	"regmagic.h"
+
+#   include	<appDebugon.h>
 
 /*
  * The "internal use only" fields in regexp.h are present to pass info from
@@ -149,14 +155,15 @@
 #define	SPSTART		04	/* Starts with * or +. */
 #define	WORST		0	/* Worst case. */
 
-/*
- * Global work variables for regcomp().
- */
-static const unsigned char *regparse;		/* Input-scan pointer. */
-static int regnpar;		/* () count. */
-static unsigned char regdummy;
-static unsigned char *regcode;	/* Code-emit pointer; &regdummy = don't. */
-static long regsize;		/* Code size. */
+typedef struct ExpressionParsingJob
+    {
+    int				epjParanthesisCount;
+    int				epjEmitCode;
+
+    long			epjSize;
+    unsigned char *		epjTarget;
+    const unsigned char *	epjSource;
+    } ExpressionParsingJob;
 
 /*
  * Static routine declarations
@@ -169,31 +176,46 @@ static int regmatch(	unsigned char *		prog,
 			unsigned char *		string,
 			unsigned char **	pPast );
 
-static unsigned char * regnext(	register unsigned char *p );
+static unsigned char * regGetNextNode(	register unsigned char *p );
+
+static unsigned char * regParseGetNextNode(	unsigned char *		p,
+						ExpressionParsingJob *	epj );
+
 static char * regprop( unsigned char *op );
 static int regtry(	regProg *	prog,
 			unsigned char *	string );
-static void regoptail(	unsigned char *	p,
-			unsigned char *	val );
 
-static void regtail(	unsigned char *	p,
-			unsigned char *	val );
+static void regoptail(	unsigned char *		p,
+			unsigned char *		val,
+			ExpressionParsingJob *	epj );
 
-static void reginsert(	unsigned char	op,
-			unsigned char *	opnd );
+static void regtail(	unsigned char *		p,
+			unsigned char *		val,
+			ExpressionParsingJob *	epj );
 
-static void regc( unsigned char b );
+static void regInsertNode(	unsigned char		op,
+				unsigned char *		opnd,
+				ExpressionParsingJob *	epj );
 
-static unsigned char * regnode( unsigned char op );
+static void regc(	unsigned char		b,
+			ExpressionParsingJob *	epj );
 
-static unsigned char * regatom(		int *	flagp );
+static unsigned char * regnode( unsigned char		op,
+				ExpressionParsingJob *	epj );
 
-static unsigned char * regpiece(	int *flagp );
+static unsigned char * regAtomicExpression( int *			flagp,
+				ExpressionParsingJob *	epj );
 
-static unsigned char * regbranch(	int *flagp );
+static unsigned char * regpiece( int *			flagp,
+				ExpressionParsingJob *	epj );
+
+static unsigned char * regbranch( int *			flagp,
+				ExpressionParsingJob *	epj );
+
+static unsigned char * reg(	int			paren,
+				int *			flagp,
+				ExpressionParsingJob *	epj );
 
-static unsigned char * reg(	int	paren,
-				int *	flagp );
 
 /*
  - regcomp - compile a regular expression into internal code
@@ -210,7 +232,59 @@ static unsigned char * reg(	int	paren,
  * Beware that the optimization-preparation code in here knows about some
  * of the structure of the compiled regexp.
  */
-regProg * regCompile( const unsigned char *exp )
+static void regInitParsingJob(	ExpressionParsingJob *	epj )
+    {
+    epj->epjParanthesisCount= 1;
+    epj->epjEmitCode= 0;
+    epj->epjSize= 0;
+    epj->epjTarget= (unsigned char *)0;
+    }
+
+regProg * regCompileEscaped( const unsigned char *	exp )
+    {
+    int				l;
+    unsigned char *		scratch;
+    regProg *			rval;
+
+    const unsigned char *	from;
+    unsigned char *		to;
+
+    l= strlen( (const char *)exp );
+    scratch= malloc( l+ 1 );
+    if  ( ! scratch )
+	{ LXDEB(l,scratch); return (regProg *)0;	}
+
+    from= exp; to= scratch;
+    while( *from )
+	{
+	if  ( *from == '*'	||
+	      *from == '+'	||
+	      *from == '?'	||
+	      *from == '^'	||
+	      *from == '$'	||
+	      *from == '.'	||
+	      *from == '['	||
+	      *from == '('	||
+	      *from == '|'	||
+	      *from == ')'	||
+	      *from == '\\'	)
+	    { *(to++)= '\\';	}
+
+	*(to++)= *(from++);
+	}
+
+    *to= '\0';
+
+    rval= regCompile( scratch );
+    if  ( ! rval )
+	{ XDEB(rval);	}
+
+    free( scratch );
+
+    return rval;
+    }
+
+regProg * regCompile( const unsigned char *	exp )
 {
     register regProg *		r;
     register unsigned char *	scan;
@@ -218,34 +292,41 @@ regProg * regCompile( const unsigned char *exp )
     register int		len;
     int				flags;
 
+    ExpressionParsingJob	epj;
+
+    unsigned char		dummy;
+
     if (exp == NULL)
 	    FAIL("NULL argument");
 
     /* First pass: determine size, legality. */
-    regparse = exp;
-    regnpar = 1;
-    regsize = 0L;
-    regcode = &regdummy;
-    regc(MAGIC);
-    if (reg(0, &flags) == NULL)
+    regInitParsingJob( &epj );
+    epj.epjEmitCode= 0;
+    epj.epjTarget= &dummy;
+    epj.epjSource= exp;
+
+    regc( MAGIC, &epj );
+    if (reg(0, &flags, &epj ) == NULL)
 	    return(NULL);
 
     /* Small enough for pointer-storage convention? */
-    if (regsize >= 32767L)		/* Probably could be 65535L. */
-	    FAIL("regexp too big");
+    if  ( epj.epjSize+ sizeof(regProg)+ 50 >= 32767L )
+	{ LDEB(epj.epjSize); return (regProg *)0;	}
 
     /* Allocate space. */
-    r = (regProg *)malloc(sizeof(regProg) + (unsigned)regsize);
+    r = (regProg *)malloc(sizeof(regProg) + (unsigned)epj.epjSize);
     if (r == NULL)
 	    FAIL("out of space");
 
     /* Second pass: emit code. */
-    regparse = exp;
-    regnpar = 1;
-    regcode = r->rpProgram;
-    regc(MAGIC);
-    if (reg(0, &flags) == NULL)
-	    return(NULL);
+    regInitParsingJob( &epj );
+    epj.epjEmitCode= 1;
+    epj.epjTarget= r->rpProgram;
+    epj.epjSource= exp;
+
+    regc(MAGIC, &epj );
+    if  ( reg(0, &flags, &epj ) == NULL)
+	{ free( r ); return(NULL);	}
 
     /* Dig out information for optimizations. */
     r->regstart = '\0';	/* Worst-case defaults. */
@@ -253,7 +334,7 @@ regProg * regCompile( const unsigned char *exp )
     r->rpRegmust = NULL;
     r->regmlen = 0;
     scan = r->rpProgram+1;			/* First BRANCH. */
-    if (OP(regnext(scan)) == END) {		/* Only one top-level choice. */
+    if (OP(regGetNextNode(scan)) == END) {	/* Only one top-level choice. */
 	    scan = OPERAND(scan);
 
 	    /* Starting-point info. */
@@ -273,7 +354,7 @@ regProg * regCompile( const unsigned char *exp )
 	    if (flags&SPSTART) {
 		    longest = NULL;
 		    len = 0;
-		    for (; scan != NULL; scan = regnext(scan))
+		    for (; scan != NULL; scan = regGetNextNode(scan))
 			    if (OP(scan) == EXACTLY && strlen((char *)OPERAND(scan)) >= len) {
 				    longest = OPERAND(scan);
 				    len = strlen((char *)OPERAND(scan));
@@ -294,78 +375,91 @@ regProg * regCompile( const unsigned char *exp )
  * Combining parenthesis handling with the base level of regular expression
  * is a trifle forced, but the need to tie the tails of the branches to what
  * follows makes it hard to avoid.
+ *
+ * paren: Parenthesized?
+ *
  */
-static unsigned char * reg(	int	paren,		/* Parenthesized? */
-				int *	flagp )
-{
-	register unsigned char *	ret;
-	register unsigned char *	br;
-	register unsigned char *	ender;
-	register int			parno= -1;
-	int				flags;
 
-	*flagp = HASWIDTH;	/* Tentatively. */
+static unsigned char * reg(	int			paren,
+				int *			flagp,
+				ExpressionParsingJob *	epj )
+    {
+    register unsigned char *	ret;
+    register unsigned char *	br;
+    register unsigned char *	ender;
+    register int		parno= -1;
+    int				flags;
 
-	/* Make an OPEN node, if parenthesized. */
-	if (paren) {
-		if (regnpar >= NSUBEXP)
-			FAIL("too many ()");
-		parno = regnpar;
-		regnpar++;
-		ret = regnode(OPEN+parno);
-	} else
-		ret = NULL;
+    *flagp = HASWIDTH;	/* Tentatively. */
 
-	/* Pick up the branches, linking them together. */
-	br = regbranch(&flags);
-	if (br == NULL)
-		return(NULL);
-	if (ret != NULL)
-		regtail(ret, br);	/* OPEN -> first. */
-	else
-		ret = br;
-	if (!(flags&HASWIDTH))
-		*flagp &= ~HASWIDTH;
-	*flagp |= flags&SPSTART;
-	while (*regparse == '|') {
-		regparse++;
-		br = regbranch(&flags);
-		if (br == NULL)
-			return(NULL);
-		regtail(ret, br);	/* BRANCH -> BRANCH. */
-		if (!(flags&HASWIDTH))
-			*flagp &= ~HASWIDTH;
-		*flagp |= flags&SPSTART;
+    /* Make an OPEN node, if parenthesized. */
+    if  ( paren )
+	{
+	if  ( epj->epjParanthesisCount >= NSUBEXP )
+	    {
+	    LLDEB( epj->epjParanthesisCount,NSUBEXP);
+	    return (unsigned char *)0;
+	    }
+
+	parno= epj->epjParanthesisCount;
+	epj->epjParanthesisCount++;
+	ret= regnode( OPEN+parno, epj );
+	}
+    else{ ret = NULL; 	}
+
+    /* Pick up the branches, linking them together. */
+    br = regbranch( &flags, epj );
+    if (br == NULL)
+	    return(NULL);
+    if (ret != NULL)
+	    regtail( ret, br, epj );	/* OPEN -> first. */
+    else
+	    ret = br;
+    if (!(flags&HASWIDTH))
+	    *flagp &= ~HASWIDTH;
+    *flagp |= flags&SPSTART;
+    while (*epj->epjSource == '|') {
+	    epj->epjSource++;
+	    br = regbranch( &flags, epj );
+	    if (br == NULL)
+		    return(NULL);
+	    regtail(ret, br, epj );	/* BRANCH -> BRANCH. */
+	    if (!(flags&HASWIDTH))
+		    *flagp &= ~HASWIDTH;
+	    *flagp |= flags&SPSTART;
+    }
+
+    /* Make a closing node, and hook it on the end. */
+    ender = regnode((paren) ? CLOSE+parno : END, epj );	
+    regtail(ret, ender, epj );
+
+    /* Hook the tails of the branches to the closing node. */
+    for (br = ret; br != NULL; br = regParseGetNextNode(br,epj))
+	    regoptail(br, ender,epj);
+
+    /* Check for proper termination. */
+    if  ( paren )
+	{
+	if  ( *epj->epjSource != ')' )
+	    { LSDEB(paren,(char *)epj->epjSource); return (unsigned char *)0; }
+
+	epj->epjSource++;
+	}
+    else{
+	if  ( *epj->epjSource != '\0' )
+	    { SDEB((char *)epj->epjSource); return (unsigned char *)0; }
 	}
 
-	/* Make a closing node, and hook it on the end. */
-	ender = regnode((paren) ? CLOSE+parno : END);	
-	regtail(ret, ender);
-
-	/* Hook the tails of the branches to the closing node. */
-	for (br = ret; br != NULL; br = regnext(br))
-		regoptail(br, ender);
-
-	/* Check for proper termination. */
-	if (paren && *regparse++ != ')') {
-		FAIL("unmatched ()");
-	} else if (!paren && *regparse != '\0') {
-		if (*regparse == ')') {
-			FAIL("unmatched ()");
-		} else
-			FAIL("junk on end");	/* "Can't happen". */
-		/* NOTREACHED */
-	}
-
-	return(ret);
-}
+    return(ret);
+    }
 
 /*
  - regbranch - one alternative of an | operator
  *
  * Implements the concatenation operator.
  */
-static unsigned char * regbranch( int *flagp )
+static unsigned char * regbranch(	int *			flagp,
+					ExpressionParsingJob *	epj )
 {
 	register unsigned char *	ret;
 	register unsigned char *	chain;
@@ -374,303 +468,466 @@ static unsigned char * regbranch( int *flagp )
 
 	*flagp = WORST;		/* Tentatively. */
 
-	ret = regnode(BRANCH);
+	ret= regnode( BRANCH, epj );
 	chain = NULL;
-	while (*regparse != '\0' && *regparse != '|' && *regparse != ')') {
-		latest = regpiece(&flags);
+	while (*epj->epjSource != '\0' && *epj->epjSource != '|' && *epj->epjSource != ')') {
+		latest = regpiece(&flags,epj);
 		if (latest == NULL)
 			return(NULL);
 		*flagp |= flags&HASWIDTH;
 		if (chain == NULL)	/* First piece. */
 			*flagp |= flags&SPSTART;
 		else
-			regtail(chain, latest);
+			regtail(chain, latest,epj);
 		chain = latest;
 	}
 	if (chain == NULL)	/* Loop ran zero times. */
-		(void) regnode(NOTHING);
+		(void) regnode( NOTHING, epj );
 
 	return(ret);
 }
 
-/*
- - regpiece - something followed by possible [*+?]
- *
- * Note that the branching code sequences used for ? and the general cases
- * of * and + are somewhat optimized:  they use the same NOTHING node as
- * both the endmarker for their branch list and the body of the last branch.
- * It might seem that this node could be dispensed with entirely, but the
- * endmarker role is not redundant.
- */
-static unsigned char * regpiece( int *flagp )
-{
-	register unsigned char *	ret;
-	register unsigned char		op;
-	register unsigned char *	next;
-	int				flags;
+/************************************************************************/
+/*									*/
+/*  regpiece - something followed by possible [*+?]			*/
+/*									*/
+/*  Note that the branching code sequences used for ? and the general	*/
+/*  cases of * and + are somewhat optimized:  they use the same NOTHING	*/
+/*  node as both the endmarker for their branch list and the body of	*/
+/*  the last branch. It might seem that this node could be dispensed	*/
+/*  with entirely, but the endmarker role is not redundant.		*/
+/*									*/
+/*  1)  Consume one atom.						*/
+/*  2)  If it is not followed by a repetition operator, just return it.	*/
+/*  3)  Things like ^ and $ are consumed by regAtomicExpression(). It	*/
+/*	is absurd to repeat them: Check.				*/
+/*  4)  ???								*/
+/*									*/
+/************************************************************************/
 
-	ret = regatom(&flags);
-	if (ret == NULL)
-		return(NULL);
+static unsigned char * regpiece(	int *			flagp,
+					ExpressionParsingJob *	epj )
+    {
+    register unsigned char *	ret;
+    register unsigned char	op;
+    register unsigned char *	next;
+    int				flags;
 
-	op = *regparse;
-	if (!ISMULT(op)) {
-		*flagp = flags;
-		return(ret);
+    /*  1  */
+    ret= regAtomicExpression( &flags, epj );
+    if  ( ret == NULL )
+	{ XDEB(ret); return ret;	}
+
+    /*  2  */
+    op= *epj->epjSource;
+    if  ( ! ISMULT( op ) )
+	{ *flagp = flags; return ret;	}
+
+    /*  3  */
+    if  ( ! ( flags & HASWIDTH ) && op != '?' )
+	{ CDEB(op); return (unsigned char *)0; }
+
+    /*  4  */
+    *flagp= (op != '+') ? (WORST|SPSTART) : (WORST|HASWIDTH);
+
+    switch( op )
+	{
+	case '*':
+	    if  ( flags & SIMPLE )
+		{ regInsertNode( STAR, ret, epj ); break;	}
+
+	    /* Emit x* as (x&|), where & means "self". */
+	    regInsertNode( BRANCH, ret, epj );
+							/* Either x */
+	    regoptail(ret, regnode( BACK, epj ), epj );
+							/* and loop back */
+	    regoptail(ret, ret, epj );
+	    regtail( ret, regnode( BRANCH, epj ), epj );
+							/* or */
+	    regtail( ret, regnode( NOTHING, epj ), epj );
+							/* null. */
+	    break;
+
+	case '+':
+	    if  ( flags & SIMPLE )
+		{ regInsertNode( PLUS, ret, epj ); break;	}
+
+	    /* Emit x+ as x(&|), where & means "self". */
+	    next= regnode(BRANCH, epj );
+							/* Either */
+	    regtail(ret, next, epj );
+	    regtail( regnode( BACK, epj ), ret, epj );
+							/* loop back */
+	    regtail(next, regnode(BRANCH, epj ), epj);
+							/* or */
+	    regtail(ret, regnode(NOTHING, epj ), epj );
+							/* null. */
+	    break;
+
+	case '?':
+	    /* Emit x? as (x|) */
+	    regInsertNode(BRANCH, ret, epj );		
+							/* Either x */
+	    regtail(ret, regnode(BRANCH, epj  ), epj );
+							/* or */
+	    next = regnode(NOTHING, epj );
+							/* null. */
+	    regtail(ret, next, epj );
+	    regoptail(ret, next, epj );
+
+	    break;
+
+	default:
+	    SDEB((char *)epj->epjSource);
+	    return (unsigned char *)0;
 	}
 
-	if (!(flags&HASWIDTH) && op != '?')
-		FAIL("*+ operand could be empty");
-	*flagp = (op != '+') ? (WORST|SPSTART) : (WORST|HASWIDTH);
-
-	if (op == '*' && (flags&SIMPLE))
-		reginsert(STAR, ret);
-	else if (op == '*') {
-		/* Emit x* as (x&|), where & means "self". */
-		reginsert(BRANCH, ret);			/* Either x */
-		regoptail(ret, regnode(BACK));		/* and loop */
-		regoptail(ret, ret);			/* back */
-		regtail(ret, regnode(BRANCH));		/* or */
-		regtail(ret, regnode(NOTHING));		/* null. */
-	} else if (op == '+' && (flags&SIMPLE))
-		reginsert(PLUS, ret);
-	else if (op == '+') {
-		/* Emit x+ as x(&|), where & means "self". */
-		next = regnode(BRANCH);			/* Either */
-		regtail(ret, next);
-		regtail(regnode(BACK), ret);		/* loop back */
-		regtail(next, regnode(BRANCH));		/* or */
-		regtail(ret, regnode(NOTHING));		/* null. */
-	} else if (op == '?') {
-		/* Emit x? as (x|) */
-		reginsert(BRANCH, ret);			/* Either x */
-		regtail(ret, regnode(BRANCH));		/* or */
-		next = regnode(NOTHING);		/* null. */
-		regtail(ret, next);
-		regoptail(ret, next);
+    epj->epjSource++;
+    if  ( ISMULT( *epj->epjSource ) )
+	{
+	SDEB((char *)epj->epjSource);
+	return (unsigned char *)0;
 	}
-	regparse++;
-	if (ISMULT(*regparse))
-		FAIL("nested *?+");
 
-	return(ret);
-}
+    return(ret);
+    }
 
-/*
- - regatom - the lowest level
- *
- * Optimization:  gobbles an entire sequence of ordinary characters so that
- * it can turn them into a single node, which is smaller to store and
- * faster to run.  Backslashed characters are exceptions, each becoming a
- * separate node; the code is simpler that way and it's not worth fixing.
- */
-static unsigned char * regatom( int *	flagp )
+/************************************************************************/
+/*									*/
+/*  regFinishExactPattern().						*/
+/*									*/
+/*  Finish an exact pattern.						*/
+/*									*/
+/*  1)  Stop at the end of the input.					*/
+/*  2)  Stop at meta-characters. Sometimes a metacharacter is an error,	*/
+/*	sometimes they end the exact pattern. By refusing them all,	*/
+/*	they will either be processed or rejected at the correct place.	*/
+/*  3)  In stead of a backslash process the next character.		*/
+/*  4)  Trailing backslashes are refused.				*/
+/*  5)  Repetitions: *+? are not part of an exact pattern.		*/
+/*  6)  Include the next charcter in the pattern.			*/
+/*  7)  Terminate the pattern.						*/
+/*  8)  Exact patterns match characters anyway. Only those with length	*/
+/*	1 are simple.							*/
+/*									*/
+/************************************************************************/
+
+static void regFinishExactPattern(	int *			flagp,
+					ExpressionParsingJob *	epj )
+    {
+    int		len= 1;
+
+    for (;;)
+	{
+	/*  1  */
+	if  ( ! epj->epjSource[0] )
+	    { break;	}
+
+	/*  2  */
+	if  ( strchr( "^$.[()|?+*", epj->epjSource[0] ) )
+	    { break;	}
+
+	/*  3  */
+	if  ( epj->epjSource[0] == '\\' )
+	    {
+	    /*  4  */
+	    if  ( ! epj->epjSource[1] )
+		{ break;	}
+	    /*  5  */
+	    if  ( ISMULT( epj->epjSource[2] ) )
+		{ break;	}
+
+	    epj->epjSource++;
+	    }
+
+	/*  5  */
+	if  ( ISMULT( epj->epjSource[1] ) )
+	    { break;	}
+
+	/*  6  */
+	regc( *epj->epjSource++, epj ); len++;
+	}
+
+    /*  7  */
+    regc( '\0', epj );
+
+    /*  8  */
+    if  ( len == 1 )
+	{ *flagp |= HASWIDTH|SIMPLE;	}
+    else{ *flagp |= HASWIDTH;		}
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  regAtomicExpression().						*/
+/*									*/
+/*  Parse an atomic expression.						*/
+/*									*/
+/*  Optimization:  gobbles an entire sequence of ordinary characters so	*/
+/*  that it can turn them into a single node, which is smaller to store	*/
+/*  and faster to run.							*/
+/*									*/
+/************************************************************************/
+
+static unsigned char * regAtomicExpression(	int *			flagp,
+						ExpressionParsingJob *	epj )
 {
-	register unsigned char *	ret;
-	int				flags;
+    unsigned char *	ret;
+    int			flags;
 
-	*flagp = WORST;		/* Tentatively. */
+    *flagp = WORST;		/* Tentatively. */
 
-	switch (*regparse++) {
+    switch( *epj->epjSource )
+	{
 	case '^':
-		ret = regnode(BOL);
-		break;
-	case '$':
-		ret = regnode(EOL);
-		break;
-	case '.':
-		ret = regnode(ANY);
-		*flagp |= HASWIDTH|SIMPLE;
-		break;
-	case '[': {
-			register int class;
-			register int classend;
+	    ret= regnode( BOL, epj );
+	    epj->epjSource++;
+	    break;
 
-			if (*regparse == '^') {	/* Complement of range. */
-				ret = regnode(ANYBUT);
-				regparse++;
-			} else
-				ret = regnode(ANYOF);
-			if (*regparse == ']' || *regparse == '-')
-				regc(*regparse++);
-			while (*regparse != '\0' && *regparse != ']') {
-				if (*regparse == '-') {
-					regparse++;
-					if (*regparse == ']' || *regparse == '\0')
-						regc('-');
-					else {
-						class = UCHARAT(regparse-2)+1;
-						classend = UCHARAT(regparse);
-						if (class > classend+1)
-							FAIL("invalid [] range");
-						for (; class <= classend; class++)
-							regc(class);
-						regparse++;
-					}
-				} else
-					regc(*regparse++);
-			}
-			regc('\0');
-			if (*regparse != ']')
-				FAIL("unmatched []");
-			regparse++;
-			*flagp |= HASWIDTH|SIMPLE;
+	case '$':
+	    ret= regnode( EOL, epj );
+	    epj->epjSource++;
+	    break;
+
+	case '.':
+	    ret = regnode( ANY, epj );
+	    *flagp |= HASWIDTH|SIMPLE;
+	    epj->epjSource++;
+	    break;
+
+	case '[':
+	    epj->epjSource++;
+	    if  ( *epj->epjSource == '^') /* Complement of range. */
+		{
+		ret = regnode( ANYBUT, epj );
+		epj->epjSource++;
 		}
-		break;
+	    else{ ret = regnode( ANYOF, epj );	}
+
+	    if  ( *epj->epjSource == ']'	||
+		  *epj->epjSource == '-'	)
+		{ regc( *epj->epjSource++, epj );	}
+
+	    while( *epj->epjSource != '\0'	&&
+		   *epj->epjSource != ']'	)
+		{
+		if  ( *epj->epjSource == '-' )
+		    {
+		    epj->epjSource++;
+
+		    if  ( *epj->epjSource == ']'	||
+			  *epj->epjSource == '\0'	)
+			{ regc('-', epj );	}
+		    else{
+			int	class = UCHARAT(epj->epjSource-2)+1;
+			int	classend = UCHARAT(epj->epjSource);
+
+			if  ( class > classend+1 )
+			    {
+			    SDEB((char *)(epj->epjSource-2));
+			    return (unsigned char *)0;
+			    }
+
+			for (; class <= classend; class++)
+			    { regc(class, epj );	}
+
+			epj->epjSource++;
+			}
+		    }
+		else{ regc(*epj->epjSource++, epj );	}
+		}
+
+	    regc('\0', epj );
+	    if  ( *epj->epjSource != ']' )
+		{ SDEB((char *)epj->epjSource); return (unsigned char *)0; }
+
+	    epj->epjSource++;
+	    *flagp |= HASWIDTH|SIMPLE;
+	    break;
+
 	case '(':
-		ret = reg(1, &flags);
-		if (ret == NULL)
-			return(NULL);
-		*flagp |= flags&(HASWIDTH|SPSTART);
-		break;
+	    epj->epjSource++;
+	    ret= reg( 1, &flags, epj );
+	    if (ret == NULL)
+		    return(NULL);
+	    *flagp |= flags&(HASWIDTH|SPSTART);
+	    break;
+
 	case '\0':
 	case '|':
 	case ')':
-		FAIL("internal urp");	/* Supposed to be caught earlier. */
-		break;
+	    /* Supposed to be caught earlier. */
+	    SDEB((char *)epj->epjSource);
+	    return (unsigned char *)0;
+
 	case '?':
 	case '+':
 	case '*':
-		FAIL("?+* follows nothing");
-		break;
+	    SDEB((char *)epj->epjSource);
+	    return (unsigned char *)0;
+
 	case '\\':
-		if (*regparse == '\0')
-			FAIL("trailing \\");
-		ret = regnode(EXACTLY);
-		regc(*regparse++);
-		regc('\0');
-		*flagp |= HASWIDTH|SIMPLE;
-		break;
-	default: {
-			register int len;
-			register char ender;
-
-			regparse--;
-			len = strcspn( (char *)regparse, META);
-			if (len <= 0)
-				FAIL("internal disaster");
-			ender = *(regparse+len);
-			if (len > 1 && ISMULT(ender))
-				len--;		/* Back off clear of ?+* operand. */
-			*flagp |= HASWIDTH;
-			if (len == 1)
-				*flagp |= SIMPLE;
-			ret = regnode(EXACTLY);
-			while (len > 0) {
-				regc(*regparse++);
-				len--;
-			}
-			regc('\0');
+	    if  ( epj->epjSource[1] == '\0' )
+		{
+		SDEB((char *)epj->epjSource);
+		return (unsigned char *)0;
 		}
-		break;
+
+	    epj->epjSource++;
+
+	    /*FALLTHROUGH*/
+
+	default:
+	    ret= regnode( EXACTLY, epj );
+	    regc( *epj->epjSource++, epj );
+
+	    regFinishExactPattern( flagp, epj );
+	    break;
 	}
 
-	return(ret);
+    return(ret);
 }
 
-/*
- - regnode - emit a node
- */
-static unsigned char * regnode( unsigned char op )
-{
-	register unsigned char *	ret;
-	register unsigned char *	ptr;
+/************************************************************************/
+/*									*/
+/*  regnode(): emit a node.						*/
+/*									*/
+/*  1)  If an insertion point is given, insert a node and a null 'next'	*/
+/*	pointer. Otherwise just count the number of bytes that is	*/
+/*	needed.								*/
+/*  2)  Operand; Null "next" pointer.					*/
+/*									*/
+/************************************************************************/
 
-	ret = regcode;
-	if (ret == &regdummy) {
-		regsize += 3;
-		return(ret);
+static unsigned char * regnode( unsigned char		op,
+				ExpressionParsingJob *	epj )
+    {
+    unsigned char *	retval= epj->epjTarget;
+
+    if  ( ! epj->epjEmitCode )
+	{ epj->epjSize += 3;	}
+    else{
+	*(epj->epjTarget++)= op;
+	*(epj->epjTarget++)= '\0';
+	*(epj->epjTarget++)= '\0';
 	}
 
-	ptr = ret;
-	*ptr++ = op;
-	*ptr++ = '\0';		/* Null "next" pointer. */
-	*ptr++ = '\0';
-	regcode = ptr;
+    return retval;
+    }
 
-	return(ret);
-}
+/************************************************************************/
+/*									*/
+/*  regc(). Insert a singel byte in a compiled regular expression, or	*/
+/*  if we are just counting.. Just count it.				*/
+/*									*/
+/************************************************************************/
 
-/*
- - regc - emit (if appropriate) a byte of code
- */
-static void regc( unsigned char b )
-{
-	if (regcode != &regdummy)
-		*regcode++ = b;
-	else
-		regsize++;
-}
+static void regc(	unsigned char		b,
+			ExpressionParsingJob *	epj )
+    {
+    if  ( ! epj->epjEmitCode )
+	{ epj->epjSize++;		}
+    else{ *(epj->epjTarget++)= b;	}
 
-/*
- - reginsert - insert an operator in front of already-emitted operand
- *
- * Means relocating the operand.
- */
-static void reginsert(	unsigned char	op,
-			unsigned char *	opnd )
-{
-	register unsigned char *	src;
-	register unsigned char *	dst;
-	register unsigned char *	place;
+    return;
+    }
 
-	if (regcode == &regdummy) {
-		regsize += 3;
-		return;
+/************************************************************************/
+/*									*/
+/*  regInsertNode() insert an operator in front of the already-emitted	*/
+/*  operand. The routine is used to insert logic before a parsed atomic	*/
+/*  regular expression to implement the repetitive operators +*?.	*/
+/*									*/
+/*  This means moving the operand right by three bytes.			*/
+/*									*/
+/*  1)  Insert node where the operand used to be.			*/
+/*									*/
+/************************************************************************/
+
+static void regInsertNode(	unsigned char		op,
+				unsigned char *		opnd,
+				ExpressionParsingJob *	epj )
+    {
+    register unsigned char *	target= epj->epjTarget;
+    register unsigned char *	src;
+    register unsigned char *	dst;
+
+    if  ( ! epj->epjEmitCode )
+	{ epj->epjSize += 3;		}
+    else{
+	src= target;
+	target += 3;
+	dst= target;
+
+	while( src > opnd )
+	    { *(--dst)= *(--src);	}
+
+	/*  1  */
+	*(opnd++)= op;
+	*(opnd++)= '\0';
+	*(opnd++)= '\0';
+
+	epj->epjTarget= target;
 	}
 
-	src = regcode;
-	regcode += 3;
-	dst = regcode;
-	while (src > opnd)
-		*--dst = *--src;
-
-	place = opnd;		/* Op node, where operand used to be. */
-	*place++ = op;
-	*place++ = '\0';
-	*place++ = '\0';
-}
+    return;
+    }
 
 /*
  - regtail - set the next-pointer at the end of a node chain
  */
-static void regtail(	unsigned char *	p,
-			unsigned char *	val )
+static void regtail(	unsigned char *		p,
+			unsigned char *		val,
+			ExpressionParsingJob *	epj )
 {
-	register unsigned char *	scan;
-	register unsigned char *	temp;
-	register int			offset;
+    register unsigned char *	scan;
+    register unsigned char *	temp;
+    register int		offset;
 
-	if (p == &regdummy)
-		return;
+    if  ( ! epj->epjEmitCode )
+	{ return;	}
 
-	/* Find last node. */
-	scan = p;
-	for (;;) {
-		temp = regnext(scan);
-		if (temp == NULL)
-			break;
-		scan = temp;
+    /* Find last node. */
+    scan= p;
+    for (;;)
+	{
+	temp = regParseGetNextNode(scan,epj);
+	if (temp == NULL)
+		break;
+	scan= temp;
 	}
 
-	if (OP(scan) == BACK)
-		offset = scan - val;
-	else
-		offset = val - scan;
-	*(scan+1) = (offset>>8)&0377;
-	*(scan+2) = offset&0377;
+    if (OP(scan) == BACK)
+	    offset = scan - val;
+    else
+	    offset = val - scan;
+    *(scan+1) = (offset>>8)&0377;
+    *(scan+2) = offset&0377;
 }
 
 /*
  - regoptail - regtail on operand of first argument; nop if operandless
  */
-static void regoptail(	unsigned char *	p,
-			unsigned char *	val )
+static void regoptail(	unsigned char *		p,
+			unsigned char *		val,
+			ExpressionParsingJob *	epj )
 {
-	/* "Operandless" and "op != BRANCH" are synonymous in practice. */
-	if (p == NULL || p == &regdummy || OP(p) != BRANCH)
-		return;
-	regtail(OPERAND(p), val);
+    /* "Operandless" and "op != BRANCH" are synonymous in practice. */
+    if  ( p == NULL || ! epj->epjEmitCode || OP(p) != BRANCH )
+	{ return;	}
+
+    regtail( OPERAND(p), val, epj );
+}
+
+static unsigned char * regParseGetNextNode(	unsigned char *		p,
+						ExpressionParsingJob *	epj )
+{
+    if  ( ! epj->epjEmitCode )
+	{ return (unsigned char *)0;	}
+
+    return regGetNextNode( p );
 }
 
 /*
@@ -684,9 +941,9 @@ static unsigned char *regbol;		/* Beginning of input, for ^ check. */
 static unsigned char **regstartp;	/* Pointer to rpStartp array. */
 static unsigned char **regendp;		/* Ditto for rpEndp. */
 
-#ifdef DEBUG
+#ifdef APP_DEBUG
 int regnarrate = 0;
-void regdump();
+void regdump( regProg *r );
 #endif
 
 /************************************************************************/
@@ -717,7 +974,8 @@ int regFindLeftToRight(	regProg *	prog,
 
 	while( s )
 	    {
-	    if  ( ! strncmp( (char *)s, (char *)prog->rpRegmust, prog->regmlen ) )
+	    if  ( ! strncmp( (char *)s,
+				(char *)prog->rpRegmust, prog->regmlen ) )
 		{ break;	}
 
 	    s= (unsigned char *)strchr( (char *)s+ 1, prog->rpRegmust[0] );
@@ -786,7 +1044,8 @@ int regFindRightToLeft(	regProg *		prog,
 
 	while( s )
 	    {
-	    if  ( ! strncmp( (char *)s, (char *)prog->rpRegmust, prog->regmlen ) )
+	    if  ( ! strncmp( (char *)s,
+				(char *)prog->rpRegmust, prog->regmlen ) )
 		{ break;	}
 
 	    s= (unsigned char *)strchr( (char *)s+ 1, prog->rpRegmust[0] );
@@ -884,24 +1143,24 @@ static int regmatch(	unsigned char *		prog,
 
     scan = prog;
 
-#   ifdef DEBUG
+#   ifdef APP_DEBUG
     if (scan != NULL && regnarrate)
-	    fprintf(stderr, "%s(\n", (char *)regprop(scan));
+	    appDebug( "%s(\n", (char *)regprop(scan));
 #   endif
 
     while (scan != NULL)
 	{
-#	ifdef DEBUG
+#	ifdef APP_DEBUG
 	if (regnarrate)
-		fprintf(stderr, "%s...\n", regprop(scan));
+		appDebug( "%s...\n", regprop(scan));
 #	endif
 
-	next= regnext( scan );
+	next= regGetNextNode( scan );
 
 	switch( OP(scan) )
 	    {
 	    case BOL:
-		if  ( string != regbol)
+		if  ( string != regbol )
 		    { return 0;	}
 		break;
 
@@ -1023,7 +1282,7 @@ static int regmatch(	unsigned char *		prog,
 			    { *pPast= string; return 1; }
 
 			string= save;
-			scan= regnext(scan);
+			scan= regGetNextNode(scan);
 			} while ( scan != NULL && OP(scan) == BRANCH );
 
 		    return 0;
@@ -1097,7 +1356,7 @@ static int regmatch(	unsigned char *		prog,
 static int regrepeat(	unsigned char *		p,
 			unsigned char *		scan,
 			unsigned char **	pPast )
-{
+    {
     int			count= 0;
     unsigned char *	opnd;
 
@@ -1106,47 +1365,42 @@ static int regrepeat(	unsigned char *		p,
     switch (OP(p))
 	{
 	case ANY:
-		count = strlen( (char *)scan);
-		scan += count;
-		break;
+	    count = strlen( (char *)scan );
+	    scan += count;
+	    break;
+
 	case EXACTLY:
-		while (*opnd == *scan) {
-			count++;
-			scan++;
-		}
-		break;
+	    while( *opnd == *scan )
+		{ count++; scan++;	}
+	    break;
+
 	case ANYOF:
-		while (*scan != '\0' && strchr( (char *)opnd, *scan) != NULL) {
-			count++;
-			scan++;
-		}
-		break;
+	    while( *scan && strchr( (char *)opnd, *scan ) )
+		{ count++; scan++; }
+	    break;
+
 	case ANYBUT:
-		while (*scan != '\0' && strchr( (char *)opnd, *scan) == NULL) {
-			count++;
-			scan++;
-		}
-		break;
+	    while( *scan && ! strchr( (char *)opnd, *scan ) )
+		{ count++; scan++; }
+	    break;
+
 	default:		/* Oh dear.  Called inappropriately. */
-		regErrorMessage("internal foulup");
-		count = 0;	/* Best compromise. */
-		break;
+	    regErrorMessage("internal foulup");
+	    count = 0;	/* Best compromise. */
+	    break;
 	}
 
 
     *pPast= scan; return count;
-}
+    }
 
 /*
- - regnext - dig the "next" pointer out of a node
+ - regGetNextNode - dig the "next" pointer out of a node
  */
-static unsigned char * regnext(	register unsigned char *p )
+static unsigned char * regGetNextNode(	register unsigned char *p )
 {
 	register int offset;
 
-	if (p == &regdummy)
-		return(NULL);
-
 	offset = NEXT(p);
 	if (offset == 0)
 		return(NULL);
@@ -1157,7 +1411,7 @@ static unsigned char * regnext(	register unsigned char *p )
 		return(p+offset);
 }
 
-#ifdef DEBUG
+#ifdef APP_DEBUG
 
 /*
  - regdump - dump a regexp onto stdout in vaguely comprehensible form
@@ -1172,12 +1426,12 @@ void regdump( regProg *r )
 	s = r->rpProgram + 1;
 	while (op != END) {	/* While that wasn't END last time... */
 		op = OP(s);
-		printf("%2d%s", s-r->rpProgram, regprop(s));	/* Where, what. */
-		next = regnext(s);
+		appDebug("%2d%s", s-r->rpProgram, regprop(s));	/* Where, what. */
+		next = regGetNextNode(s);
 		if (next == NULL)		/* Next ptr. */
-			printf("(0)");
+			appDebug("(0)");
 		else 
-			printf("(%d)", (s-r->rpProgram)+(next-s));
+			appDebug("(%d)", (s-r->rpProgram)+(next-s));
 		s += 3;
 		if (op == ANYOF || op == ANYBUT || op == EXACTLY) {
 			/* Literal string, where present. */
@@ -1192,12 +1446,12 @@ void regdump( regProg *r )
 
 	/* Header fields of interest. */
 	if (r->regstart != '\0')
-		printf("start `%c' ", r->regstart);
+		appDebug("start `%c' ", r->regstart);
 	if (r->reganch)
-		printf("anchored ");
+		appDebug("anchored ");
 	if (r->rpRegmust != NULL)
-		printf("must have \"%s\"", r->rpRegmust);
-	printf("\n");
+		appDebug("must have \"%s\"", r->rpRegmust);
+	appDebug("\n");
 }
 
 /*
@@ -1280,35 +1534,3 @@ static char * regprop( unsigned char *op )
 	return(buf);
 }
 #endif
-
-/*
- * The following is provided for those people who do not have strcspn() in
- * their C libraries.  They should get off their butts and do something
- * about it; at least one public-domain implementation of those (highly
- * useful) string routines has been published on Usenet.
- */
-#ifdef STRCSPN
-/*
- * strcspn - find length of initial segment of s1 consisting entirely
- * of characters not from s2
- */
-
-static int
-strcspn(s1, s2)
-char *s1;
-char *s2;
-{
-	register char *scan1;
-	register char *scan2;
-	register int count;
-
-	count = 0;
-	for (scan1 = s1; *scan1 != '\0'; scan1++) {
-		for (scan2 = s2; *scan2 != '\0';)	/* ++ moved down. */
-			if (*scan1 == *scan2++)
-				return(count);
-		count++;
-	}
-	return(count);
-}
-#endif
